package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.AmDoorMonitorAccountFi;
import cn.cecep.talroad.service.IAmDoorMonitorAccountFiService;
import cn.cecep.talroad.vo.AmDoorMonitorAccountFiVo;
import cn.cecep.talroad.vo.query.AmDoorMonitorAccountFiQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 门禁监控Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控--门禁监控")
@RestController
@RequestMapping("/amDoorMonitorAccountFi")
public class AmDoorMonitorAccountFiController extends BasesController
{
    @Autowired
    private IAmDoorMonitorAccountFiService amDoorMonitorAccountFiService;

    /**
     * 查询门禁监控列表
     */
    @ApiOperation(value = "查询门禁监控")
    @GetMapping("/list")
    public AjaxResults<IPage<AmDoorMonitorAccountFi>> list(AmDoorMonitorAccountFiQuery amDoorMonitorAccountFi)
    {
        Page<AmDoorMonitorAccountFi> page = new Page<>(amDoorMonitorAccountFi.getPageNum(), amDoorMonitorAccountFi.getPageSize());
        IPage<AmDoorMonitorAccountFi> list = amDoorMonitorAccountFiService.page(page, AmDoorMonitorAccountFiQuery.createLambdaQueryWrapper(amDoorMonitorAccountFi));
        return AjaxResults.success(list);
    }


    /**
     * 导出门禁监控列表
     */
    @ApiOperation(value = "导出门禁监控")
    @Log(title = "门禁监控", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody AmDoorMonitorAccountFiQuery amDoorMonitorAccountFi)
    {
        List<AmDoorMonitorAccountFi> list = amDoorMonitorAccountFiService.list(AmDoorMonitorAccountFiQuery.createLambdaQueryWrapper(amDoorMonitorAccountFi));
        ExcelUtil<AmDoorMonitorAccountFi> util = new ExcelUtil<AmDoorMonitorAccountFi>(AmDoorMonitorAccountFi.class);
        util.exportExcel(response, list, "门禁监控数据");
    }

    /**
     * 获取门禁监控详细信息
     */
    @ApiOperation(value = "获取门禁监控详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<AmDoorMonitorAccountFi> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(amDoorMonitorAccountFiService.getById(id));
    }
    /**
     * 新增门禁监控
     */
    @ApiOperation(value = "新增门禁监控")
    @Log(title = "门禁监控", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody AmDoorMonitorAccountFi amDoorMonitorAccountFi) {
        return toAjax(amDoorMonitorAccountFiService.save(amDoorMonitorAccountFi));
    }
    /**
     * 修改门禁监控
     */
    @ApiOperation(value = "修改门禁监控")
    @Log(title = "门禁监控", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody AmDoorMonitorAccountFiVo amDoorMonitorAccountFiVo) {
        if(ObjectUtil.isEmpty(amDoorMonitorAccountFiVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(amDoorMonitorAccountFiVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        AmDoorMonitorAccountFi amDoorMonitorAccountFi = amDoorMonitorAccountFiService.getById(amDoorMonitorAccountFiVo.getId());
        if(ObjectUtil.isEmpty(amDoorMonitorAccountFi)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", amDoorMonitorAccountFiVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(amDoorMonitorAccountFiVo,amDoorMonitorAccountFi);
        return toAjax(amDoorMonitorAccountFiService.saveOrUpdate(amDoorMonitorAccountFi));
    }

    /**
     * 删除门禁监控
     */
    @ApiOperation(value = "删除门禁监控")
    @RequiresPermissions("talroad:fi:remove")
    @Log(title = "门禁监控", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(amDoorMonitorAccountFiService.removeByIds(Arrays.asList(ids)));
    }
}
