package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BEnterpriseIrEesType;
import cn.cecep.talroad.service.IBEnterpriseIrEesTypeService;
import cn.cecep.talroad.vo.BEnterpriseIrEesTypeVo;
import cn.cecep.talroad.vo.query.BEnterpriseIrEesTypeQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 知识库-生态环境标准-分类Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-14*/
@Api(tags = "知识库-生态环境标准-分类")
@RestController
@RequestMapping("/eesType")
public class BEnterpriseIrEesTypeController extends BasesController
{
    @Autowired
    private IBEnterpriseIrEesTypeService bEnterpriseIrEesTypeService;

    /**
     * 查询行业列表
     */
    @ApiOperation(value = "知识库树结构")
    @GetMapping("/selectTreeList")
    public AjaxResults<List<BEnterpriseIrEesType>> selectTreeList(String eesClassify)
    {
        return AjaxResults.success(bEnterpriseIrEesTypeService.selectTreeList(eesClassify));
    }

    /**
     * 查询知识库-生态环境标准-分类列表
     */
    @ApiOperation(value = "查询知识库-生态环境标准-分类")
    @GetMapping("/list")
    public AjaxResults<IPage<BEnterpriseIrEesType>> list(BEnterpriseIrEesTypeQuery bEnterpriseIrEesType)
    {
        Page<BEnterpriseIrEesType> page = new Page<>(bEnterpriseIrEesType.getPageNum(), bEnterpriseIrEesType.getPageSize());
        IPage<BEnterpriseIrEesType> list = bEnterpriseIrEesTypeService.page(page, BEnterpriseIrEesTypeQuery.createLambdaQueryWrapper(bEnterpriseIrEesType));
        return AjaxResults.success(list);
    }


    /**
     * 导出知识库-生态环境标准-分类列表
     */
    @ApiOperation(value = "导出知识库-生态环境标准-分类")
    @Log(title = "知识库-生态环境标准-分类", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BEnterpriseIrEesType bEnterpriseIrEesType)
    {
        List<BEnterpriseIrEesType> list = bEnterpriseIrEesTypeService.selectBEnterpriseIrEesTypeList(bEnterpriseIrEesType);
        ExcelUtil<BEnterpriseIrEesType> util = new ExcelUtil<BEnterpriseIrEesType>(BEnterpriseIrEesType.class);
        util.exportExcel(response, list, "知识库-生态环境标准-分类数据");
    }

    /**
     * 获取知识库-生态环境标准-分类详细信息
     */
    @ApiOperation(value = "获取知识库-生态环境标准-分类详细信息")
    @GetMapping(value = "/{id}")
    public AjaxResults<BEnterpriseIrEesType> getInfo(@PathVariable("id") String id) {
        return AjaxResults.success(bEnterpriseIrEesTypeService.getById(id));
    }
    /**
     * 新增知识库-生态环境标准-分类
     */
    @ApiOperation(value = "新增知识库-生态环境标准-分类")
    @Log(title = "知识库-生态环境标准-分类", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BEnterpriseIrEesType bEnterpriseIrEesType) {
        return toAjax(bEnterpriseIrEesTypeService.save(bEnterpriseIrEesType));
    }
    /**
     * 修改知识库-生态环境标准-分类
     */
    @ApiOperation(value = "修改知识库-生态环境标准-分类")
    @Log(title = "知识库-生态环境标准-分类", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BEnterpriseIrEesTypeVo bEnterpriseIrEesTypeVo) {
        if(ObjectUtil.isEmpty(bEnterpriseIrEesTypeVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bEnterpriseIrEesTypeVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BEnterpriseIrEesType bEnterpriseIrEesType = bEnterpriseIrEesTypeService.getById(bEnterpriseIrEesTypeVo.getId());
        if(ObjectUtil.isEmpty(bEnterpriseIrEesType)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bEnterpriseIrEesTypeVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bEnterpriseIrEesTypeVo,bEnterpriseIrEesType);
        return toAjax(bEnterpriseIrEesTypeService.saveOrUpdate(bEnterpriseIrEesType));
    }

    /**
     * 删除知识库-生态环境标准-分类
     */
    @ApiOperation(value = "删除知识库-生态环境标准-分类")
    @RequiresPermissions("talroad:type:remove")
    @Log(title = "知识库-生态环境标准-分类", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bEnterpriseIrEesTypeService.removeByIds(Arrays.asList(ids)));
    }
}
