package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BEnterpriseIrLegal;
import cn.cecep.talroad.service.IBEnterpriseIrLegalService;
import cn.cecep.talroad.vo.BEnterpriseIrLegalVo;
import cn.cecep.talroad.vo.query.BEnterpriseIrLegalQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 知识库-法律法规Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-14*/
@Api(tags = "知识库-法律法规")
@RestController
@RequestMapping("/legal")
public class BEnterpriseIrLegalController extends BasesController
{
    @Autowired
    private IBEnterpriseIrLegalService bEnterpriseIrLegalService;

    /**
     * 查询知识库-法律法规列表
     */
    @ApiOperation(value = "查询知识库-法律法规")
    @GetMapping("/list")
    public AjaxResults<IPage<BEnterpriseIrLegal>> list(BEnterpriseIrLegalQuery bEnterpriseIrLegal)
    {
        Page<BEnterpriseIrLegal> page = new Page<>(bEnterpriseIrLegal.getPageNum(), bEnterpriseIrLegal.getPageSize());
        IPage<BEnterpriseIrLegal> list = bEnterpriseIrLegalService.page(page, BEnterpriseIrLegalQuery.createLambdaQueryWrapper(bEnterpriseIrLegal));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出知识库-法律法规列表
     */
    @ApiOperation(value = "导出知识库-法律法规")
    @Log(title = "知识库-法律法规", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BEnterpriseIrLegal bEnterpriseIrLegal)
    {
        List<BEnterpriseIrLegal> list = bEnterpriseIrLegalService.selectBEnterpriseIrLegalList(bEnterpriseIrLegal);
        ExcelUtil<BEnterpriseIrLegal> util = new ExcelUtil<BEnterpriseIrLegal>(BEnterpriseIrLegal.class);
        util.exportExcel(response, list, "知识库-法律法规数据");
    }

    /**
     * 获取知识库-法律法规详细信息
     */
    @ApiOperation(value = "获取知识库-法律法规详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BEnterpriseIrLegal> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bEnterpriseIrLegalService.getById(id));
    }
    /**
     * 新增知识库-法律法规
     */
    @ApiOperation(value = "新增知识库-法律法规")
    @Log(title = "知识库-法律法规", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BEnterpriseIrLegal bEnterpriseIrLegal) {
        return toAjax(bEnterpriseIrLegalService.save(bEnterpriseIrLegal));
    }
    /**
     * 修改知识库-法律法规
     */
    @ApiOperation(value = "修改知识库-法律法规")
    @Log(title = "知识库-法律法规", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BEnterpriseIrLegalVo bEnterpriseIrLegalVo) {
        if(ObjectUtil.isEmpty(bEnterpriseIrLegalVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bEnterpriseIrLegalVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BEnterpriseIrLegal bEnterpriseIrLegal = bEnterpriseIrLegalService.getById(bEnterpriseIrLegalVo.getId());
        if(ObjectUtil.isEmpty(bEnterpriseIrLegal)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bEnterpriseIrLegalVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bEnterpriseIrLegalVo,bEnterpriseIrLegal);
        return toAjax(bEnterpriseIrLegalService.saveOrUpdate(bEnterpriseIrLegal));
    }

    /**
     * 删除知识库-法律法规
     */
    @ApiOperation(value = "删除知识库-法律法规")
    @RequiresPermissions("talroad:legal:remove")
    @Log(title = "知识库-法律法规", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bEnterpriseIrLegalService.removeByIds(Arrays.asList(ids)));
    }
}
