package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BEnterpriseIrSdoept;
import cn.cecep.talroad.service.IBEnterpriseIrSdoeptService;
import cn.cecep.talroad.vo.BEnterpriseIrSdoeptVo;
import cn.cecep.talroad.vo.query.BEnterpriseIrSdoeptQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 知识库-排污治理科普库Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-14*/
@Api(tags = "知识库-排污治理科普库")
@RestController
@RequestMapping("/sdoept")
public class BEnterpriseIrSdoeptController extends BasesController
{
    @Autowired
    private IBEnterpriseIrSdoeptService bEnterpriseIrSdoeptService;

    /**
     * 查询知识库-排污治理科普库列表
     */
    @ApiOperation(value = "查询知识库-排污治理科普库")
    @GetMapping("/list")
    public AjaxResults<IPage<BEnterpriseIrSdoept>> list(BEnterpriseIrSdoeptQuery bEnterpriseIrSdoept)
    {
        Page<BEnterpriseIrSdoept> page = new Page<>(bEnterpriseIrSdoept.getPageNum(), bEnterpriseIrSdoept.getPageSize());
        IPage<BEnterpriseIrSdoept> list = bEnterpriseIrSdoeptService.page(page, BEnterpriseIrSdoeptQuery.createLambdaQueryWrapper(bEnterpriseIrSdoept));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出知识库-排污治理科普库列表
     */
    @ApiOperation(value = "导出知识库-排污治理科普库")
    @Log(title = "知识库-排污治理科普库", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BEnterpriseIrSdoept bEnterpriseIrSdoept)
    {
        List<BEnterpriseIrSdoept> list = bEnterpriseIrSdoeptService.selectBEnterpriseIrSdoeptList(bEnterpriseIrSdoept);
        ExcelUtil<BEnterpriseIrSdoept> util = new ExcelUtil<BEnterpriseIrSdoept>(BEnterpriseIrSdoept.class);
        util.exportExcel(response, list, "知识库-排污治理科普库数据");
    }

    /**
     * 获取知识库-排污治理科普库详细信息
     */
    @ApiOperation(value = "获取知识库-排污治理科普库详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BEnterpriseIrSdoept> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bEnterpriseIrSdoeptService.getById(id));
    }
    /**
     * 新增知识库-排污治理科普库
     */
    @ApiOperation(value = "新增知识库-排污治理科普库")
    @Log(title = "知识库-排污治理科普库", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BEnterpriseIrSdoept bEnterpriseIrSdoept) {
        return toAjax(bEnterpriseIrSdoeptService.save(bEnterpriseIrSdoept));
    }
    /**
     * 修改知识库-排污治理科普库
     */
    @ApiOperation(value = "修改知识库-排污治理科普库")
    @Log(title = "知识库-排污治理科普库", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BEnterpriseIrSdoeptVo bEnterpriseIrSdoeptVo) {
        if(ObjectUtil.isEmpty(bEnterpriseIrSdoeptVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bEnterpriseIrSdoeptVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BEnterpriseIrSdoept bEnterpriseIrSdoept = bEnterpriseIrSdoeptService.getById(bEnterpriseIrSdoeptVo.getId());
        if(ObjectUtil.isEmpty(bEnterpriseIrSdoept)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bEnterpriseIrSdoeptVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bEnterpriseIrSdoeptVo,bEnterpriseIrSdoept);
        return toAjax(bEnterpriseIrSdoeptService.saveOrUpdate(bEnterpriseIrSdoept));
    }

    /**
     * 删除知识库-排污治理科普库
     */
    @ApiOperation(value = "删除知识库-排污治理科普库")
    @RequiresPermissions("talroad:sdoept:remove")
    @Log(title = "知识库-排污治理科普库", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bEnterpriseIrSdoeptService.removeByIds(Arrays.asList(ids)));
    }
}
