package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainActionReports;
import cn.cecep.talroad.service.IBMainActionReportsService;
import cn.cecep.talroad.vo.BMainActionReportsVo;
import cn.cecep.talroad.vo.query.BMainActionReportsQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 执行报告Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-11*/
@Api(tags = "业务--执行报告")
@RestController
@RequestMapping("/bMainActionReports")
public class BMainActionReportsController extends BasesController
{
    @Autowired
    private IBMainActionReportsService bMainActionReportsService;

    /**
     * 查询执行报告列表
     */
    @ApiOperation(value = "查询执行报告")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainActionReports>> list(BMainActionReportsQuery bMainActionReports)
    {
        Page<BMainActionReports> page = new Page<>(bMainActionReports.getPageNum(), bMainActionReports.getPageSize());
        IPage<BMainActionReports> list = bMainActionReportsService.page(page, BMainActionReportsQuery.createLambdaQueryWrapper(bMainActionReports));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出执行报告列表
     */
    @ApiOperation(value = "导出执行报告")
    @Log(title = "执行报告", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainActionReports bMainActionReports)
    {
        List<BMainActionReports> list = bMainActionReportsService.selectBMainActionReportsList(bMainActionReports);
        ExcelUtil<BMainActionReports> util = new ExcelUtil<BMainActionReports>(BMainActionReports.class);
        util.exportExcel(response, list, "执行报告数据");
    }

    /**
     * 获取执行报告详细信息
     */
    @ApiOperation(value = "获取执行报告详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainActionReports> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bMainActionReportsService.getById(id));
    }
    /**
     * 新增执行报告
     */
    @ApiOperation(value = "新增执行报告")
    @Log(title = "执行报告", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainActionReports bMainActionReports) {
        return toAjax(bMainActionReportsService.save(bMainActionReports));
    }
    /**
     * 修改执行报告
     */
    @ApiOperation(value = "修改执行报告")
    @Log(title = "执行报告", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainActionReportsVo bMainActionReportsVo) {
        if(ObjectUtil.isEmpty(bMainActionReportsVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainActionReportsVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainActionReports bMainActionReports = bMainActionReportsService.getById(bMainActionReportsVo.getId());
        if(ObjectUtil.isEmpty(bMainActionReports)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainActionReportsVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainActionReportsVo,bMainActionReports);
        return toAjax(bMainActionReportsService.saveOrUpdate(bMainActionReports));
    }

    /**
     * 删除执行报告
     */
    @ApiOperation(value = "删除执行报告")
    @RequiresPermissions("talroad:reports:remove")
    @Log(title = "执行报告", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bMainActionReportsService.removeByIds(Arrays.asList(ids)));
    }
}
