package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainAllowablePollutionWater;
import cn.cecep.talroad.service.IBMainAllowablePollutionWaterService;
import cn.cecep.talroad.vo.BMainAllowablePollutionWaterVo;
import cn.cecep.talroad.vo.query.BMainAllowablePollutionWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 总许可量- 废水排放总许可量要求Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-31*/
@Api(tags = "总许可量- 废水排放总许可量要求")
@RestController
@RequestMapping("/bmainAllowablePollutionWater")
public class BMainAllowablePollutionWaterController extends BasesController
{
    @Autowired
    private IBMainAllowablePollutionWaterService bMainAllowablePollutionWaterService;

    /**
     * 查询总许可量- 废水排放总许可量要求列表
     */
    @ApiOperation(value = "查询总许可量- 废水排放总许可量要求")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainAllowablePollutionWater>> list(BMainAllowablePollutionWaterQuery bMainAllowablePollutionWater)
    {
        Page<BMainAllowablePollutionWater> page = new Page<>(bMainAllowablePollutionWater.getPageNum(), bMainAllowablePollutionWater.getPageSize());
        IPage<BMainAllowablePollutionWater> list = bMainAllowablePollutionWaterService.selectBMainAllowablePollutionWaterPageList(page, BMainAllowablePollutionWaterQuery.createLambdaQueryWrapper(bMainAllowablePollutionWater));
        return AjaxResults.success(list);
    }


    /**
     * 导出总许可量- 废水排放总许可量要求列表
     */
    @ApiOperation(value = "导出总许可量- 废水排放总许可量要求")
    @Log(title = "总许可量- 废水排放总许可量要求", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainAllowablePollutionWater bMainAllowablePollutionWater)
    {
        List<BMainAllowablePollutionWater> list = bMainAllowablePollutionWaterService.selectBMainAllowablePollutionWaterList(bMainAllowablePollutionWater);
        ExcelUtil<BMainAllowablePollutionWater> util = new ExcelUtil<BMainAllowablePollutionWater>(BMainAllowablePollutionWater.class);
        util.exportExcel(response, list, "总许可量- 废水排放总许可量要求数据");
    }

    /**
     * 获取总许可量- 废水排放总许可量要求详细信息
     */
    @ApiOperation(value = "获取总许可量- 废水排放总许可量要求详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainAllowablePollutionWater> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bMainAllowablePollutionWaterService.getById(id));
    }
    /**
     * 新增总许可量- 废水排放总许可量要求
     */
    @ApiOperation(value = "新增总许可量- 废水排放总许可量要求")
    @Log(title = "总许可量- 废水排放总许可量要求", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainAllowablePollutionWater bMainAllowablePollutionWater) {
        return toAjax(bMainAllowablePollutionWaterService.save(bMainAllowablePollutionWater));
    }
    /**
     * 修改总许可量- 废水排放总许可量要求
     */
    @ApiOperation(value = "修改总许可量- 废水排放总许可量要求")
    @Log(title = "总许可量- 废水排放总许可量要求", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainAllowablePollutionWaterVo bMainAllowablePollutionWaterVo) {
        if(ObjectUtil.isEmpty(bMainAllowablePollutionWaterVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainAllowablePollutionWaterVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainAllowablePollutionWater bMainAllowablePollutionWater = bMainAllowablePollutionWaterService.getById(bMainAllowablePollutionWaterVo.getId());
        if(ObjectUtil.isEmpty(bMainAllowablePollutionWater)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainAllowablePollutionWaterVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainAllowablePollutionWaterVo,bMainAllowablePollutionWater);
        return toAjax(bMainAllowablePollutionWaterService.saveOrUpdate(bMainAllowablePollutionWater));
    }

    /**
     * 删除总许可量- 废水排放总许可量要求
     */
    @ApiOperation(value = "删除总许可量- 废水排放总许可量要求")
    @RequiresPermissions("talroad:bmainAllowablePollutionWater:remove")
    @Log(title = "总许可量- 废水排放总许可量要求", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bMainAllowablePollutionWaterService.removeByIds(Arrays.asList(ids)));
    }
}
