package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainEmissionOnlineMonWater;
import cn.cecep.talroad.service.IBMainEmissionOnlineMonWaterService;
import cn.cecep.talroad.vo.BMainEmissionOnlineMonWaterVo;
import cn.cecep.talroad.vo.query.BMainEmissionOnlineMonWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.text.ParseException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 企业排放信息-在线监测排放趋势-废水Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-31*/
@Api(tags = "企业排放信息-在线监测排放趋势-废水")
@RestController
@RequestMapping("/bmainEmissionOnlineMonWater")
public class BMainEmissionOnlineMonWaterController extends BasesController
{
    @Autowired
    private IBMainEmissionOnlineMonWaterService bMainEmissionOnlineMonWaterService;

    /**
     * 查询企业排放信息-在线监测排放趋势-废水列表
     */
    @ApiOperation(value = "查询企业排放信息-在线监测排放趋势-废水")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainEmissionOnlineMonWater>> list(BMainEmissionOnlineMonWaterQuery bMainEmissionOnlineMonWater) throws ParseException {
        Page<BMainEmissionOnlineMonWater> page = new Page<>(bMainEmissionOnlineMonWater.getPageNum(), bMainEmissionOnlineMonWater.getPageSize());
        IPage<BMainEmissionOnlineMonWater> list = bMainEmissionOnlineMonWaterService.page(page, BMainEmissionOnlineMonWaterQuery.createLambdaQueryWrapper(bMainEmissionOnlineMonWater));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出企业排放信息-在线监测排放趋势-废水列表
     */
    @ApiOperation(value = "导出企业排放信息-在线监测排放趋势-废水")
    @Log(title = "企业排放信息-在线监测排放趋势-废水", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainEmissionOnlineMonWater bMainEmissionOnlineMonWater)
    {
        List<BMainEmissionOnlineMonWater> list = bMainEmissionOnlineMonWaterService.selectBMainEmissionOnlineMonWaterList(bMainEmissionOnlineMonWater);
        ExcelUtil<BMainEmissionOnlineMonWater> util = new ExcelUtil<BMainEmissionOnlineMonWater>(BMainEmissionOnlineMonWater.class);
        util.exportExcel(response, list, "企业排放信息-在线监测排放趋势-废水数据");
    }

    /**
     * 获取企业排放信息-在线监测排放趋势-废水详细信息
     */
    @ApiOperation(value = "获取企业排放信息-在线监测排放趋势-废水详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainEmissionOnlineMonWater> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bMainEmissionOnlineMonWaterService.getById(id));
    }
    /**
     * 新增企业排放信息-在线监测排放趋势-废水
     */
    @ApiOperation(value = "新增企业排放信息-在线监测排放趋势-废水")
    @Log(title = "企业排放信息-在线监测排放趋势-废水", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainEmissionOnlineMonWater bMainEmissionOnlineMonWater) {
        return toAjax(bMainEmissionOnlineMonWaterService.save(bMainEmissionOnlineMonWater));
    }
    /**
     * 修改企业排放信息-在线监测排放趋势-废水
     */
    @ApiOperation(value = "修改企业排放信息-在线监测排放趋势-废水")
    @Log(title = "企业排放信息-在线监测排放趋势-废水", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainEmissionOnlineMonWaterVo bMainEmissionOnlineMonWaterVo) {
        if(ObjectUtil.isEmpty(bMainEmissionOnlineMonWaterVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainEmissionOnlineMonWaterVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainEmissionOnlineMonWater bMainEmissionOnlineMonWater = bMainEmissionOnlineMonWaterService.getById(bMainEmissionOnlineMonWaterVo.getId());
        if(ObjectUtil.isEmpty(bMainEmissionOnlineMonWater)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainEmissionOnlineMonWaterVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainEmissionOnlineMonWaterVo,bMainEmissionOnlineMonWater);
        return toAjax(bMainEmissionOnlineMonWaterService.saveOrUpdate(bMainEmissionOnlineMonWater));
    }

    /**
     * 删除企业排放信息-在线监测排放趋势-废水
     */
    @ApiOperation(value = "删除企业排放信息-在线监测排放趋势-废水")
    @RequiresPermissions("talroad:bmainEmissionOnlineMonWater:remove")
    @Log(title = "企业排放信息-在线监测排放趋势-废水", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bMainEmissionOnlineMonWaterService.removeByIds(Arrays.asList(ids)));
    }
}
