package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainEmissionPollutionSolidWaste;
import cn.cecep.talroad.service.IBMainEmissionPollutionSolidWasteService;
import cn.cecep.talroad.vo.BMainEmissionPollutionSolidWasteVo;
import cn.cecep.talroad.vo.query.BMainEmissionPollutionSolidWasteQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 企业排放信息-不同污染物年度排放趋势-固废Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-31*/
@Api(tags = "企业排放信息-不同污染物年度排放趋势-固废")
@RestController
@RequestMapping("/bmainEmissionPollutionSolidWaste")
public class BMainEmissionPollutionSolidWasteController extends BasesController
{
    @Autowired
    private IBMainEmissionPollutionSolidWasteService bMainEmissionPollutionSolidWasteService;

    /**
     * 查询企业排放信息-不同污染物年度排放趋势-固废列表
     */
    @ApiOperation(value = "查询企业排放信息-不同污染物年度排放趋势-固废")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainEmissionPollutionSolidWaste>> list(BMainEmissionPollutionSolidWasteQuery bMainEmissionPollutionSolidWaste)
    {
        Page<BMainEmissionPollutionSolidWaste> page = new Page<>(bMainEmissionPollutionSolidWaste.getPageNum(), bMainEmissionPollutionSolidWaste.getPageSize());
        IPage<BMainEmissionPollutionSolidWaste> list = bMainEmissionPollutionSolidWasteService.page(page, BMainEmissionPollutionSolidWasteQuery.createLambdaQueryWrapper(bMainEmissionPollutionSolidWaste));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出企业排放信息-不同污染物年度排放趋势-固废列表
     */
    @ApiOperation(value = "导出企业排放信息-不同污染物年度排放趋势-固废")
    @Log(title = "企业排放信息-不同污染物年度排放趋势-固废", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainEmissionPollutionSolidWaste bMainEmissionPollutionSolidWaste)
    {
        List<BMainEmissionPollutionSolidWaste> list = bMainEmissionPollutionSolidWasteService.selectBMainEmissionPollutionSolidWasteList(bMainEmissionPollutionSolidWaste);
        ExcelUtil<BMainEmissionPollutionSolidWaste> util = new ExcelUtil<BMainEmissionPollutionSolidWaste>(BMainEmissionPollutionSolidWaste.class);
        util.exportExcel(response, list, "企业排放信息-不同污染物年度排放趋势-固废数据");
    }

    /**
     * 获取企业排放信息-不同污染物年度排放趋势-固废详细信息
     */
    @ApiOperation(value = "获取企业排放信息-不同污染物年度排放趋势-固废详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainEmissionPollutionSolidWaste> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bMainEmissionPollutionSolidWasteService.getById(id));
    }
    /**
     * 新增企业排放信息-不同污染物年度排放趋势-固废
     */
    @ApiOperation(value = "新增企业排放信息-不同污染物年度排放趋势-固废")
    @Log(title = "企业排放信息-不同污染物年度排放趋势-固废", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainEmissionPollutionSolidWaste bMainEmissionPollutionSolidWaste) {
        return toAjax(bMainEmissionPollutionSolidWasteService.save(bMainEmissionPollutionSolidWaste));
    }
    /**
     * 修改企业排放信息-不同污染物年度排放趋势-固废
     */
    @ApiOperation(value = "修改企业排放信息-不同污染物年度排放趋势-固废")
    @Log(title = "企业排放信息-不同污染物年度排放趋势-固废", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainEmissionPollutionSolidWasteVo bMainEmissionPollutionSolidWasteVo) {
        if(ObjectUtil.isEmpty(bMainEmissionPollutionSolidWasteVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainEmissionPollutionSolidWasteVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainEmissionPollutionSolidWaste bMainEmissionPollutionSolidWaste = bMainEmissionPollutionSolidWasteService.getById(bMainEmissionPollutionSolidWasteVo.getId());
        if(ObjectUtil.isEmpty(bMainEmissionPollutionSolidWaste)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainEmissionPollutionSolidWasteVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainEmissionPollutionSolidWasteVo,bMainEmissionPollutionSolidWaste);
        return toAjax(bMainEmissionPollutionSolidWasteService.saveOrUpdate(bMainEmissionPollutionSolidWaste));
    }

    /**
     * 删除企业排放信息-不同污染物年度排放趋势-固废
     */
    @ApiOperation(value = "删除企业排放信息-不同污染物年度排放趋势-固废")
    @RequiresPermissions("talroad:bmainEmissionPollutionSolidWaste:remove")
    @Log(title = "企业排放信息-不同污染物年度排放趋势-固废", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bMainEmissionPollutionSolidWasteService.removeByIds(Arrays.asList(ids)));
    }
}
