package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainEmissionPollutionWater;
import cn.cecep.talroad.service.IBMainEmissionPollutionWaterService;
import cn.cecep.talroad.vo.BMainEmissionPollutionWaterVo;
import cn.cecep.talroad.vo.query.BMainEmissionPollutionWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 企业排放信息-不同污染物年度排放趋势-废水Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-31*/
@Api(tags = "企业排放信息-不同污染物年度排放趋势-废水")
@RestController
@RequestMapping("/bmainEmissionPollutionWater")
public class BMainEmissionPollutionWaterController extends BasesController
{
    @Autowired
    private IBMainEmissionPollutionWaterService bMainEmissionPollutionWaterService;

    /**
     * 查询企业排放信息-不同污染物年度排放趋势-废水列表
     */
    @ApiOperation(value = "查询企业排放信息-不同污染物年度排放趋势-废水")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainEmissionPollutionWater>> list(BMainEmissionPollutionWaterQuery bMainEmissionPollutionWater)
    {
        Page<BMainEmissionPollutionWater> page = new Page<>(bMainEmissionPollutionWater.getPageNum(), bMainEmissionPollutionWater.getPageSize());
        IPage<BMainEmissionPollutionWater> list = bMainEmissionPollutionWaterService.page(page, BMainEmissionPollutionWaterQuery.createLambdaQueryWrapper(bMainEmissionPollutionWater));
        return AjaxResults.success(list);
    }


    /**
     * 导出企业排放信息-不同污染物年度排放趋势-废水列表
     */
    @ApiOperation(value = "导出企业排放信息-不同污染物年度排放趋势-废水")
    @Log(title = "企业排放信息-不同污染物年度排放趋势-废水", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainEmissionPollutionWater bMainEmissionPollutionWater)
    {
        List<BMainEmissionPollutionWater> list = bMainEmissionPollutionWaterService.selectBMainEmissionPollutionWaterList(bMainEmissionPollutionWater);
        ExcelUtil<BMainEmissionPollutionWater> util = new ExcelUtil<BMainEmissionPollutionWater>(BMainEmissionPollutionWater.class);
        util.exportExcel(response, list, "企业排放信息-不同污染物年度排放趋势-废水数据");
    }

    /**
     * 获取企业排放信息-不同污染物年度排放趋势-废水详细信息
     */
    @ApiOperation(value = "获取企业排放信息-不同污染物年度排放趋势-废水详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainEmissionPollutionWater> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bMainEmissionPollutionWaterService.getById(id));
    }
    /**
     * 新增企业排放信息-不同污染物年度排放趋势-废水
     */
    @ApiOperation(value = "新增企业排放信息-不同污染物年度排放趋势-废水")
    @Log(title = "企业排放信息-不同污染物年度排放趋势-废水", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainEmissionPollutionWater bMainEmissionPollutionWater) {
        return toAjax(bMainEmissionPollutionWaterService.save(bMainEmissionPollutionWater));
    }
    /**
     * 修改企业排放信息-不同污染物年度排放趋势-废水
     */
    @ApiOperation(value = "修改企业排放信息-不同污染物年度排放趋势-废水")
    @Log(title = "企业排放信息-不同污染物年度排放趋势-废水", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainEmissionPollutionWaterVo bMainEmissionPollutionWaterVo) {
        if(ObjectUtil.isEmpty(bMainEmissionPollutionWaterVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainEmissionPollutionWaterVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainEmissionPollutionWater bMainEmissionPollutionWater = bMainEmissionPollutionWaterService.getById(bMainEmissionPollutionWaterVo.getId());
        if(ObjectUtil.isEmpty(bMainEmissionPollutionWater)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainEmissionPollutionWaterVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainEmissionPollutionWaterVo,bMainEmissionPollutionWater);
        return toAjax(bMainEmissionPollutionWaterService.saveOrUpdate(bMainEmissionPollutionWater));
    }

    /**
     * 删除企业排放信息-不同污染物年度排放趋势-废水
     */
    @ApiOperation(value = "删除企业排放信息-不同污染物年度排放趋势-废水")
    @RequiresPermissions("talroad:bmainEmissionPollutionWater:remove")
    @Log(title = "企业排放信息-不同污染物年度排放趋势-废水", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bMainEmissionPollutionWaterService.removeByIds(Arrays.asList(ids)));
    }
}
