package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BMainFactoryVerificationQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BMainFactoryVerification;
import cn.cecep.talroad.vo.BMainFactoryVerificationVo;
import cn.cecep.talroad.service.IBMainFactoryVerificationService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 移动执法信息-企业核查Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-30*/
@Api(tags = "移动执法信息-企业核查")
@RestController
@RequestMapping("/verification")
public class BMainFactoryVerificationController extends BasesController
{
    @Autowired
    private IBMainFactoryVerificationService bMainFactoryVerificationService;

    /**
     * 查询移动执法信息-企业核查列表
     */
    @ApiOperation(value = "查询移动执法信息-企业核查")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainFactoryVerification>> list(BMainFactoryVerificationQuery bMainFactoryVerification)
    {
        Page<BMainFactoryVerification> page = new Page<>(bMainFactoryVerification.getPageNum(), bMainFactoryVerification.getPageSize());
        IPage<BMainFactoryVerification> list = bMainFactoryVerificationService.page(page, BMainFactoryVerificationQuery.createLambdaQueryWrapper(bMainFactoryVerification));
        return AjaxResults.success(list);
    }


    /**
     * 导出移动执法信息-企业核查列表
     */
    @ApiOperation(value = "导出移动执法信息-企业核查")
    @Log(title = "移动执法信息-企业核查", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BMainFactoryVerificationQuery bMainFactoryVerification)
    {
        List<BMainFactoryVerification> list = bMainFactoryVerificationService.list(BMainFactoryVerificationQuery.createLambdaQueryWrapper(bMainFactoryVerification));
        ExcelUtil<BMainFactoryVerification> util = new ExcelUtil<BMainFactoryVerification>(BMainFactoryVerification.class);
        util.exportExcel(response, list, "移动执法信息-企业核查数据");
    }

    /**
     * 获取移动执法信息-企业核查详细信息
     */
    @ApiOperation(value = "获取任务来源下拉框")
    @GetMapping(value = "/taskSourceSelectList")
    public AjaxResults<List<String>> taskSourceSelectList() {
        QueryWrapper<BMainFactoryVerification> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("distinct task_source")
                .orderByAsc("task_source");
        List<BMainFactoryVerification> list = bMainFactoryVerificationService.list(queryWrapper);

        List<String> collect = list.stream().map(BMainFactoryVerification::getTaskSource).collect(Collectors.toList());
        return AjaxResults.success(collect);
    }

    /**
     * 获取移动执法信息-企业核查详细信息
     */
    @ApiOperation(value = "获取移动执法信息-企业核查详细信息")
    @GetMapping(value = "/{id}")
    public AjaxResults<BMainFactoryVerification> getInfo(@PathVariable("id") String id) {
        return AjaxResults.success(bMainFactoryVerificationService.getById(id));
    }
    /**
     * 新增移动执法信息-企业核查
     */
    @ApiOperation(value = "新增移动执法信息-企业核查")
    @Log(title = "移动执法信息-企业核查", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainFactoryVerification bMainFactoryVerification) {
        return toAjax(bMainFactoryVerificationService.save(bMainFactoryVerification));
    }
    /**
     * 修改移动执法信息-企业核查
     */
    @ApiOperation(value = "修改移动执法信息-企业核查")
    @Log(title = "移动执法信息-企业核查", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainFactoryVerificationVo bMainFactoryVerificationVo) {
        if(ObjectUtil.isEmpty(bMainFactoryVerificationVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainFactoryVerificationVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainFactoryVerification bMainFactoryVerification = bMainFactoryVerificationService.getById(bMainFactoryVerificationVo.getId());
        if(ObjectUtil.isEmpty(bMainFactoryVerification)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainFactoryVerificationVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainFactoryVerificationVo,bMainFactoryVerification);
        return toAjax(bMainFactoryVerificationService.saveOrUpdate(bMainFactoryVerification));
    }

    /**
     * 删除移动执法信息-企业核查
     */
    @ApiOperation(value = "删除移动执法信息-企业核查")
    @RequiresPermissions("talroad:verification:remove")
    @Log(title = "移动执法信息-企业核查", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bMainFactoryVerificationService.removeByIds(Arrays.asList(ids)));
    }
}
