package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainHazardousSolidWasteCr;
import cn.cecep.talroad.service.IBMainHazardousSolidWasteCrService;
import cn.cecep.talroad.vo.BMainHazardousSolidWasteCrVo;
import cn.cecep.talroad.vo.query.BMainHazardousSolidWasteCrQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固体废物环境管理要求Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-11*/
@Api(tags = "基本档案--固体废物环境管理要求")
@RestController
@RequestMapping("/bMainHazardousSolidWasteCr")
public class BMainHazardousSolidWasteCrController extends BasesController
{
    @Autowired
    private IBMainHazardousSolidWasteCrService bMainHazardousSolidWasteCrService;

    /**
     * 查询固体废物环境管理要求列表
     */
    @ApiOperation(value = "查询固体废物环境管理要求")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainHazardousSolidWasteCr>> list(BMainHazardousSolidWasteCrQuery bMainHazardousSolidWasteCr)
    {
        Page<BMainHazardousSolidWasteCr> page = new Page<>(bMainHazardousSolidWasteCr.getPageNum(), bMainHazardousSolidWasteCr.getPageSize());
        IPage<BMainHazardousSolidWasteCr> list = bMainHazardousSolidWasteCrService.page(page, BMainHazardousSolidWasteCrQuery.createLambdaQueryWrapper(bMainHazardousSolidWasteCr));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出固体废物环境管理要求列表
     */
    @ApiOperation(value = "导出固体废物环境管理要求")
    @Log(title = "固体废物环境管理要求", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainHazardousSolidWasteCr bMainHazardousSolidWasteCr)
    {
        List<BMainHazardousSolidWasteCr> list = bMainHazardousSolidWasteCrService.selectBMainHazardousSolidWasteCrList(bMainHazardousSolidWasteCr);
        ExcelUtil<BMainHazardousSolidWasteCr> util = new ExcelUtil<BMainHazardousSolidWasteCr>(BMainHazardousSolidWasteCr.class);
        util.exportExcel(response, list, "固体废物环境管理要求数据");
    }

    /**
     * 获取固体废物环境管理要求详细信息
     */
    @ApiOperation(value = "获取固体废物环境管理要求详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainHazardousSolidWasteCr> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bMainHazardousSolidWasteCrService.getById(id));
    }
    /**
     * 新增固体废物环境管理要求
     */
    @ApiOperation(value = "新增固体废物环境管理要求")
    @Log(title = "固体废物环境管理要求", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainHazardousSolidWasteCr bMainHazardousSolidWasteCr) {
        return toAjax(bMainHazardousSolidWasteCrService.save(bMainHazardousSolidWasteCr));
    }
    /**
     * 修改固体废物环境管理要求
     */
    @ApiOperation(value = "修改固体废物环境管理要求")
    @Log(title = "固体废物环境管理要求", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainHazardousSolidWasteCrVo bMainHazardousSolidWasteCrVo) {
        if(ObjectUtil.isEmpty(bMainHazardousSolidWasteCrVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainHazardousSolidWasteCrVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainHazardousSolidWasteCr bMainHazardousSolidWasteCr = bMainHazardousSolidWasteCrService.getById(bMainHazardousSolidWasteCrVo.getId());
        if(ObjectUtil.isEmpty(bMainHazardousSolidWasteCr)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainHazardousSolidWasteCrVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainHazardousSolidWasteCrVo,bMainHazardousSolidWasteCr);
        return toAjax(bMainHazardousSolidWasteCrService.saveOrUpdate(bMainHazardousSolidWasteCr));
    }

    /**
     * 删除固体废物环境管理要求
     */
    @ApiOperation(value = "删除固体废物环境管理要求")
    @RequiresPermissions("talroad:cr:remove")
    @Log(title = "固体废物环境管理要求", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bMainHazardousSolidWasteCrService.removeByIds(Arrays.asList(ids)));
    }
}
