package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainHazardousSolidWasteFa;
import cn.cecep.talroad.service.IBMainHazardousSolidWasteFaService;
import cn.cecep.talroad.vo.BMainHazardousSolidWasteFaVo;
import cn.cecep.talroad.vo.query.BMainHazardousSolidWasteFaQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 自行贮存和自行利用处置设施信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "基本档案--自行贮存和自行利用处置设施信息")
@RestController
@RequestMapping("/bMainHazardousSolidWasteFa")
public class BMainHazardousSolidWasteFaController extends BasesController
{
    @Autowired
    private IBMainHazardousSolidWasteFaService bMainHazardousSolidWasteFaService;

    /**
     * 查询自行贮存和自行利用处置设施信息列表
     */
    @ApiOperation(value = "查询自行贮存和自行利用处置设施信息")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainHazardousSolidWasteFa>> list(BMainHazardousSolidWasteFaQuery bMainHazardousSolidWasteFa)
    {
        Page<BMainHazardousSolidWasteFa> page = new Page<>(bMainHazardousSolidWasteFa.getPageNum(), bMainHazardousSolidWasteFa.getPageSize());
        IPage<BMainHazardousSolidWasteFa> list = bMainHazardousSolidWasteFaService.page(page, BMainHazardousSolidWasteFaQuery.createLambdaQueryWrapper(bMainHazardousSolidWasteFa));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出自行贮存和自行利用处置设施信息列表
     */
    @ApiOperation(value = "导出自行贮存和自行利用处置设施信息")
    @Log(title = "自行贮存和自行利用处置设施信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainHazardousSolidWasteFa bMainHazardousSolidWasteFa)
    {
        List<BMainHazardousSolidWasteFa> list = bMainHazardousSolidWasteFaService.selectBMainHazardousSolidWasteFaList(bMainHazardousSolidWasteFa);
        ExcelUtil<BMainHazardousSolidWasteFa> util = new ExcelUtil<BMainHazardousSolidWasteFa>(BMainHazardousSolidWasteFa.class);
        util.exportExcel(response, list, "自行贮存和自行利用处置设施信息数据");
    }

    /**
     * 获取自行贮存和自行利用处置设施信息详细信息
     */
    @ApiOperation(value = "获取自行贮存和自行利用处置设施信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainHazardousSolidWasteFa> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bMainHazardousSolidWasteFaService.getById(id));
    }
    /**
     * 新增自行贮存和自行利用处置设施信息
     */
    @ApiOperation(value = "新增自行贮存和自行利用处置设施信息")
    @Log(title = "自行贮存和自行利用处置设施信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainHazardousSolidWasteFa bMainHazardousSolidWasteFa) {
        return toAjax(bMainHazardousSolidWasteFaService.save(bMainHazardousSolidWasteFa));
    }
    /**
     * 修改自行贮存和自行利用处置设施信息
     */
    @ApiOperation(value = "修改自行贮存和自行利用处置设施信息")
    @Log(title = "自行贮存和自行利用处置设施信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainHazardousSolidWasteFaVo bMainHazardousSolidWasteFaVo) {
        if(ObjectUtil.isEmpty(bMainHazardousSolidWasteFaVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainHazardousSolidWasteFaVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainHazardousSolidWasteFa bMainHazardousSolidWasteFa = bMainHazardousSolidWasteFaService.getById(bMainHazardousSolidWasteFaVo.getId());
        if(ObjectUtil.isEmpty(bMainHazardousSolidWasteFa)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainHazardousSolidWasteFaVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainHazardousSolidWasteFaVo,bMainHazardousSolidWasteFa);
        return toAjax(bMainHazardousSolidWasteFaService.saveOrUpdate(bMainHazardousSolidWasteFa));
    }

    /**
     * 删除自行贮存和自行利用处置设施信息
     */
    @ApiOperation(value = "删除自行贮存和自行利用处置设施信息")
    @RequiresPermissions("talroad:fa:remove")
    @Log(title = "自行贮存和自行利用处置设施信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bMainHazardousSolidWasteFaService.removeByIds(Arrays.asList(ids)));
    }
}
