package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainHazardousSolidWasteLe;
import cn.cecep.talroad.service.IBMainHazardousSolidWasteLeService;
import cn.cecep.talroad.vo.BMainHazardousSolidWasteLeVo;
import cn.cecep.talroad.vo.query.BMainHazardousSolidWasteLeQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固体废物排放信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "基本档案--固体废物排放信息")
@RestController
@RequestMapping("/bMainHazardousSolidWasteLe")
public class BMainHazardousSolidWasteLeController extends BasesController
{
    @Autowired
    private IBMainHazardousSolidWasteLeService bMainHazardousSolidWasteLeService;

    /**
     * 查询固体废物排放信息列表
     */
    @ApiOperation(value = "查询固体废物排放信息")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainHazardousSolidWasteLe>> list(BMainHazardousSolidWasteLeQuery bMainHazardousSolidWasteLe)
    {
        Page<BMainHazardousSolidWasteLe> page = new Page<>(bMainHazardousSolidWasteLe.getPageNum(), bMainHazardousSolidWasteLe.getPageSize());
        IPage<BMainHazardousSolidWasteLe> list = bMainHazardousSolidWasteLeService.page(page, BMainHazardousSolidWasteLeQuery.createLambdaQueryWrapper(bMainHazardousSolidWasteLe));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出固体废物排放信息列表
     */
    @ApiOperation(value = "导出固体废物排放信息")
    @Log(title = "固体废物排放信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainHazardousSolidWasteLe bMainHazardousSolidWasteLe)
    {
        List<BMainHazardousSolidWasteLe> list = bMainHazardousSolidWasteLeService.selectBMainHazardousSolidWasteLeList(bMainHazardousSolidWasteLe);
        ExcelUtil<BMainHazardousSolidWasteLe> util = new ExcelUtil<BMainHazardousSolidWasteLe>(BMainHazardousSolidWasteLe.class);
        util.exportExcel(response, list, "固体废物排放信息数据");
    }

    /**
     * 获取固体废物排放信息详细信息
     */
    @ApiOperation(value = "获取固体废物排放信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainHazardousSolidWasteLe> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bMainHazardousSolidWasteLeService.getById(id));
    }
    /**
     * 新增固体废物排放信息
     */
    @ApiOperation(value = "新增固体废物排放信息")
    @Log(title = "固体废物排放信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainHazardousSolidWasteLe bMainHazardousSolidWasteLe) {
        return toAjax(bMainHazardousSolidWasteLeService.save(bMainHazardousSolidWasteLe));
    }
    /**
     * 修改固体废物排放信息
     */
    @ApiOperation(value = "修改固体废物排放信息")
    @Log(title = "固体废物排放信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainHazardousSolidWasteLeVo bMainHazardousSolidWasteLeVo) {
        if(ObjectUtil.isEmpty(bMainHazardousSolidWasteLeVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainHazardousSolidWasteLeVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainHazardousSolidWasteLe bMainHazardousSolidWasteLe = bMainHazardousSolidWasteLeService.getById(bMainHazardousSolidWasteLeVo.getId());
        if(ObjectUtil.isEmpty(bMainHazardousSolidWasteLe)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainHazardousSolidWasteLeVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainHazardousSolidWasteLeVo,bMainHazardousSolidWasteLe);
        return toAjax(bMainHazardousSolidWasteLeService.saveOrUpdate(bMainHazardousSolidWasteLe));
    }

    /**
     * 删除固体废物排放信息
     */
    @ApiOperation(value = "删除固体废物排放信息")
    @RequiresPermissions("talroad:le:remove")
    @Log(title = "固体废物排放信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bMainHazardousSolidWasteLeService.removeByIds(Arrays.asList(ids)));
    }
}
