package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainMobileEnforcement;
import cn.cecep.talroad.service.IBMainMobileEnforcementService;
import cn.cecep.talroad.vo.BMainMobileEnforcementVo;
import cn.cecep.talroad.vo.query.BMainMobileEnforcementQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 移动执法信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "业务--移动执法信息")
@RestController
@RequestMapping("/bMainMobileEnforcement")
public class BMainMobileEnforcementController extends BasesController
{
    @Autowired
    private IBMainMobileEnforcementService bMainMobileEnforcementService;

    /**
     * 查询移动执法信息列表
     */
    @ApiOperation(value = "查询移动执法信息")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainMobileEnforcement>> list(BMainMobileEnforcementQuery bMainMobileEnforcement)
    {
        return AjaxResults.success(bMainMobileEnforcementService.pullPage(bMainMobileEnforcement));
    }


    /**
     * 导出移动执法信息列表
     */
    @ApiOperation(value = "导出移动执法信息")
    @Log(title = "移动执法信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BMainMobileEnforcementQuery bMainMobileEnforcement)
    {
        List<BMainMobileEnforcement> list = bMainMobileEnforcementService.list(BMainMobileEnforcementQuery.createLambdaQueryWrapper(bMainMobileEnforcement));
        ExcelUtil<BMainMobileEnforcement> util = new ExcelUtil<BMainMobileEnforcement>(BMainMobileEnforcement.class);
        util.exportExcel(response, list, "移动执法信息数据");
    }

    /**
     * 获取移动执法信息详细信息
     */
    @ApiOperation(value = "获取移动执法信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainMobileEnforcement> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bMainMobileEnforcementService.getById(id));
    }
    /**
     * 新增移动执法信息
     */
    @ApiOperation(value = "新增移动执法信息")
    @Log(title = "移动执法信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainMobileEnforcement bMainMobileEnforcement) {
        return toAjax(bMainMobileEnforcementService.save(bMainMobileEnforcement));
    }
    /**
     * 修改移动执法信息
     */
    @ApiOperation(value = "修改移动执法信息")
    @Log(title = "移动执法信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainMobileEnforcementVo bMainMobileEnforcementVo) {
        if(ObjectUtil.isEmpty(bMainMobileEnforcementVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainMobileEnforcementVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainMobileEnforcement bMainMobileEnforcement = bMainMobileEnforcementService.getById(bMainMobileEnforcementVo.getId());
        if(ObjectUtil.isEmpty(bMainMobileEnforcement)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainMobileEnforcementVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainMobileEnforcementVo,bMainMobileEnforcement);
        return toAjax(bMainMobileEnforcementService.saveOrUpdate(bMainMobileEnforcement));
    }

    /**
     * 删除移动执法信息
     */
    @ApiOperation(value = "删除移动执法信息")
    @RequiresPermissions("talroad:enforcement:remove")
    @Log(title = "移动执法信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bMainMobileEnforcementService.removeByIds(Arrays.asList(ids)));
    }
}
