package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainNoiseEmissions;
import cn.cecep.talroad.service.IBMainNoiseEmissionsService;
import cn.cecep.talroad.vo.BMainNoiseEmissionsVo;
import cn.cecep.talroad.vo.query.BMainNoiseEmissionsQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 噪声排放信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "基本档案--噪声排放信息")
@RestController
@RequestMapping("/bMainNoiseEmissions")
public class BMainNoiseEmissionsController extends BasesController
{
    @Autowired
    private IBMainNoiseEmissionsService bMainNoiseEmissionsService;

    /**
     * 查询噪声排放信息列表
     */
    @ApiOperation(value = "查询噪声排放信息")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainNoiseEmissions>> list(BMainNoiseEmissionsQuery bMainNoiseEmissions)
    {
        Page<BMainNoiseEmissions> page = new Page<>(bMainNoiseEmissions.getPageNum(), bMainNoiseEmissions.getPageSize());
        IPage<BMainNoiseEmissions> list = bMainNoiseEmissionsService.page(page, BMainNoiseEmissionsQuery.createLambdaQueryWrapper(bMainNoiseEmissions));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出噪声排放信息列表
     */
    @ApiOperation(value = "导出噪声排放信息")
    @Log(title = "噪声排放信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainNoiseEmissions bMainNoiseEmissions)
    {
        List<BMainNoiseEmissions> list = bMainNoiseEmissionsService.selectBMainNoiseEmissionsList(bMainNoiseEmissions);
        ExcelUtil<BMainNoiseEmissions> util = new ExcelUtil<BMainNoiseEmissions>(BMainNoiseEmissions.class);
        util.exportExcel(response, list, "噪声排放信息数据");
    }

    /**
     * 获取噪声排放信息详细信息
     */
    @ApiOperation(value = "获取噪声排放信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainNoiseEmissions> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bMainNoiseEmissionsService.getById(id));
    }
    /**
     * 新增噪声排放信息
     */
    @ApiOperation(value = "新增噪声排放信息")
    @Log(title = "噪声排放信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainNoiseEmissions bMainNoiseEmissions) {
        return toAjax(bMainNoiseEmissionsService.save(bMainNoiseEmissions));
    }
    /**
     * 修改噪声排放信息
     */
    @ApiOperation(value = "修改噪声排放信息")
    @Log(title = "噪声排放信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainNoiseEmissionsVo bMainNoiseEmissionsVo) {
        if(ObjectUtil.isEmpty(bMainNoiseEmissionsVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainNoiseEmissionsVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainNoiseEmissions bMainNoiseEmissions = bMainNoiseEmissionsService.getById(bMainNoiseEmissionsVo.getId());
        if(ObjectUtil.isEmpty(bMainNoiseEmissions)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainNoiseEmissionsVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainNoiseEmissionsVo,bMainNoiseEmissions);
        return toAjax(bMainNoiseEmissionsService.saveOrUpdate(bMainNoiseEmissions));
    }

    /**
     * 删除噪声排放信息
     */
    @ApiOperation(value = "删除噪声排放信息")
    @RequiresPermissions("talroad:emissions:remove")
    @Log(title = "噪声排放信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bMainNoiseEmissionsService.removeByIds(Arrays.asList(ids)));
    }
}
