package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainPollDisEmlRecord;
import cn.cecep.talroad.service.IBMainPollDisEmlRecordService;
import cn.cecep.talroad.vo.BMainPollDisEmlRecordVo;
import cn.cecep.talroad.vo.query.BMainPollDisEmlRecordQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 排污许可环境管理台账Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-18*/
@Api(tags = "排污许可--排污许可环境管理台账")
@RestController
@RequestMapping("/BMainPollDisEmlRecord")
public class BMainPollDisEmlRecordController extends BasesController
{
    @Autowired
    private IBMainPollDisEmlRecordService bMainPollDisEmlRecordService;

    /**
     * 查询排污许可环境管理台账列表
     */
    @ApiOperation(value = "查询排污许可环境管理台账")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainPollDisEmlRecord>> list(BMainPollDisEmlRecordQuery bMainPollDisEmlRecord)
    {
        Page<BMainPollDisEmlRecord> page = new Page<>(bMainPollDisEmlRecord.getPageNum(), bMainPollDisEmlRecord.getPageSize());
        IPage<BMainPollDisEmlRecord> list = bMainPollDisEmlRecordService.page(page, BMainPollDisEmlRecordQuery.createLambdaQueryWrapper(bMainPollDisEmlRecord));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出排污许可环境管理台账列表
     */
    @ApiOperation(value = "导出排污许可环境管理台账")
    @Log(title = "排污许可环境管理台账", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainPollDisEmlRecord bMainPollDisEmlRecord)
    {
        List<BMainPollDisEmlRecord> list = bMainPollDisEmlRecordService.selectBMainPollDisEmlRecordList(bMainPollDisEmlRecord);
        ExcelUtil<BMainPollDisEmlRecord> util = new ExcelUtil<BMainPollDisEmlRecord>(BMainPollDisEmlRecord.class);
        util.exportExcel(response, list, "排污许可环境管理台账数据");
    }

    /**
     * 获取排污许可环境管理台账详细信息
     */
    @ApiOperation(value = "获取排污许可环境管理台账详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainPollDisEmlRecord> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(bMainPollDisEmlRecordService.getById(id));
    }
    /**
     * 新增排污许可环境管理台账
     */
    @ApiOperation(value = "新增排污许可环境管理台账")
    @Log(title = "排污许可环境管理台账", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainPollDisEmlRecord bMainPollDisEmlRecord) {
        return toAjax(bMainPollDisEmlRecordService.save(bMainPollDisEmlRecord));
    }
    /**
     * 修改排污许可环境管理台账
     */
    @ApiOperation(value = "修改排污许可环境管理台账")
    @Log(title = "排污许可环境管理台账", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainPollDisEmlRecordVo bMainPollDisEmlRecordVo) {
        if(ObjectUtil.isEmpty(bMainPollDisEmlRecordVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainPollDisEmlRecordVo.getEmlId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainPollDisEmlRecord bMainPollDisEmlRecord = bMainPollDisEmlRecordService.getById(bMainPollDisEmlRecordVo.getEmlId());
        if(ObjectUtil.isEmpty(bMainPollDisEmlRecord)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainPollDisEmlRecordVo.getEmlId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainPollDisEmlRecordVo,bMainPollDisEmlRecord);
        return toAjax(bMainPollDisEmlRecordService.saveOrUpdate(bMainPollDisEmlRecord));
    }

    /**
     * 删除排污许可环境管理台账
     */
    @ApiOperation(value = "删除排污许可环境管理台账")
    @RequiresPermissions("talroad:BMainPollDisEmlRecord:remove")
    @Log(title = "排污许可环境管理台账", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bMainPollDisEmlRecordService.removeByIds(Arrays.asList(ids)));
    }
}
