package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainPollDisEnforcement;
import cn.cecep.talroad.service.IBMainPollDisEnforcementService;
import cn.cecep.talroad.vo.BMainPollDisEnforcementVo;
import cn.cecep.talroad.vo.query.BMainPollDisEnforcementQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 排污许可执行守法报告Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-18*/
@Api(tags = "排污许可--排污许可执行守法报告")
@RestController
@RequestMapping("/BMainPollDisEnforcement")
public class BMainPollDisEnforcementController extends BasesController
{
    @Autowired
    private IBMainPollDisEnforcementService bMainPollDisEnforcementService;

    /**
     * 查询排污许可执行守法报告列表
     */
    @ApiOperation(value = "查询排污许可执行守法报告")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainPollDisEnforcement>> list(BMainPollDisEnforcementQuery bMainPollDisEnforcement)
    {
        Page<BMainPollDisEnforcement> page = new Page<>(bMainPollDisEnforcement.getPageNum(), bMainPollDisEnforcement.getPageSize());
        IPage<BMainPollDisEnforcement> list = bMainPollDisEnforcementService.page(page, BMainPollDisEnforcementQuery.createLambdaQueryWrapper(bMainPollDisEnforcement));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出排污许可执行守法报告列表
     */
    @ApiOperation(value = "导出排污许可执行守法报告")
    @Log(title = "排污许可执行守法报告", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainPollDisEnforcement bMainPollDisEnforcement)
    {
        List<BMainPollDisEnforcement> list = bMainPollDisEnforcementService.selectBMainPollDisEnforcementList(bMainPollDisEnforcement);
        ExcelUtil<BMainPollDisEnforcement> util = new ExcelUtil<BMainPollDisEnforcement>(BMainPollDisEnforcement.class);
        util.exportExcel(response, list, "排污许可执行守法报告数据");
    }

    /**
     * 获取排污许可执行守法报告详细信息
     */
    @ApiOperation(value = "获取排污许可执行守法报告详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainPollDisEnforcement> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(bMainPollDisEnforcementService.getById(id));
    }
    /**
     * 新增排污许可执行守法报告
     */
    @ApiOperation(value = "新增排污许可执行守法报告")
    @Log(title = "排污许可执行守法报告", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainPollDisEnforcement bMainPollDisEnforcement) {
        return toAjax(bMainPollDisEnforcementService.save(bMainPollDisEnforcement));
    }
    /**
     * 修改排污许可执行守法报告
     */
    @ApiOperation(value = "修改排污许可执行守法报告")
    @Log(title = "排污许可执行守法报告", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainPollDisEnforcementVo bMainPollDisEnforcementVo) {
        if(ObjectUtil.isEmpty(bMainPollDisEnforcementVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainPollDisEnforcementVo.getEnforcementId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainPollDisEnforcement bMainPollDisEnforcement = bMainPollDisEnforcementService.getById(bMainPollDisEnforcementVo.getEnforcementId());
        if(ObjectUtil.isEmpty(bMainPollDisEnforcement)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainPollDisEnforcementVo.getEnforcementId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainPollDisEnforcementVo,bMainPollDisEnforcement);
        return toAjax(bMainPollDisEnforcementService.saveOrUpdate(bMainPollDisEnforcement));
    }

    /**
     * 删除排污许可执行守法报告
     */
    @ApiOperation(value = "删除排污许可执行守法报告")
    @RequiresPermissions("talroad:BMainPollDisEnforcement:remove")
    @Log(title = "排污许可执行守法报告", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bMainPollDisEnforcementService.removeByIds(Arrays.asList(ids)));
    }
}
