package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainPollDisGasOout;
import cn.cecep.talroad.service.IBMainPollDisGasOoutService;
import cn.cecep.talroad.vo.BMainPollDisGasOoutVo;
import cn.cecep.talroad.vo.query.BMainPollDisGasOoutQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 排污许可大气一般排放口Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-18*/
@Api(tags = "排污许可--排污许可大气一般排放口")
@RestController
@RequestMapping("/BMainPollDisGasOout")
public class BMainPollDisGasOoutController extends BasesController
{
    @Autowired
    private IBMainPollDisGasOoutService bMainPollDisGasOoutService;

    /**
     * 查询排污许可大气一般排放口列表
     */
    @ApiOperation(value = "查询排污许可大气一般排放口")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainPollDisGasOout>> list(BMainPollDisGasOoutQuery bMainPollDisGasOout)
    {
        return AjaxResults.success(bMainPollDisGasOoutService.pullPage(bMainPollDisGasOout));
    }
    

    /**
     * 导出排污许可大气一般排放口列表
     */
    @ApiOperation(value = "导出排污许可大气一般排放口")
    @Log(title = "排污许可大气一般排放口", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainPollDisGasOout bMainPollDisGasOout)
    {
        List<BMainPollDisGasOout> list = bMainPollDisGasOoutService.selectBMainPollDisGasOoutList(bMainPollDisGasOout);
        ExcelUtil<BMainPollDisGasOout> util = new ExcelUtil<BMainPollDisGasOout>(BMainPollDisGasOout.class);
        util.exportExcel(response, list, "排污许可大气一般排放口数据");
    }

    /**
     * 获取排污许可大气一般排放口详细信息
     */
    @ApiOperation(value = "获取排污许可大气一般排放口详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainPollDisGasOout> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(bMainPollDisGasOoutService.getById(id));
    }
    /**
     * 新增排污许可大气一般排放口
     */
    @ApiOperation(value = "新增排污许可大气一般排放口")
    @Log(title = "排污许可大气一般排放口", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainPollDisGasOout bMainPollDisGasOout) {
        return toAjax(bMainPollDisGasOoutService.save(bMainPollDisGasOout));
    }
    /**
     * 修改排污许可大气一般排放口
     */
    @ApiOperation(value = "修改排污许可大气一般排放口")
    @Log(title = "排污许可大气一般排放口", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainPollDisGasOoutVo bMainPollDisGasOoutVo) {
        if(ObjectUtil.isEmpty(bMainPollDisGasOoutVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainPollDisGasOoutVo.getOoutId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainPollDisGasOout bMainPollDisGasOout = bMainPollDisGasOoutService.getById(bMainPollDisGasOoutVo.getOoutId());
        if(ObjectUtil.isEmpty(bMainPollDisGasOout)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainPollDisGasOoutVo.getOoutId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainPollDisGasOoutVo,bMainPollDisGasOout);
        return toAjax(bMainPollDisGasOoutService.saveOrUpdate(bMainPollDisGasOout));
    }

    /**
     * 删除排污许可大气一般排放口
     */
    @ApiOperation(value = "删除排污许可大气一般排放口")
    @RequiresPermissions("talroad:BMainPollDisGasOout:remove")
    @Log(title = "排污许可大气一般排放口", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bMainPollDisGasOoutService.removeByIds(Arrays.asList(ids)));
    }
}
