package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainPollDisGasSout;
import cn.cecep.talroad.service.IBMainPollDisGasSoutService;
import cn.cecep.talroad.vo.BMainPollDisGasSoutVo;
import cn.cecep.talroad.vo.query.BMainPollDisGasSoutQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;


/**
 * * 开发公司：中节能天融科技有限公司
 * 排污许可大气特殊情况Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-18*/
@Api(tags = "排污许可--排污许可大气特殊情况")
@RestController
@RequestMapping("/BMainPollDisGasSout")
public class BMainPollDisGasSoutController extends BasesController
{
    @Autowired
    private IBMainPollDisGasSoutService bMainPollDisGasSoutService;

    /**
     * 查询排污许可大气特殊情况列表
     */
    @ApiOperation(value = "查询排污许可大气特殊情况")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainPollDisGasSout>> list(@Valid BMainPollDisGasSoutQuery bMainPollDisGasSout)
    {
        Page<BMainPollDisGasSout> page = new Page<>(bMainPollDisGasSout.getPageNum(), bMainPollDisGasSout.getPageSize());
        IPage<BMainPollDisGasSout> list = bMainPollDisGasSoutService.page(page, BMainPollDisGasSoutQuery.createLambdaQueryWrapper(bMainPollDisGasSout));
        List<BMainPollDisGasSout> records = list.getRecords();
        if(CollectionUtils.isNotEmpty(records)){
            ArrayList<BMainPollDisGasSout> res = Lists.newArrayList();
            List<BMainPollDisGasSout> mainDrain = records.stream().filter(item -> item.getPollutantType().equals("0")).collect(Collectors.toList());
            if(CollectionUtils.isNotEmpty(mainDrain)){
                mainDrain.forEach(item->{
                    item.setVent("主要排放口");
                });
                res.addAll(mainDrain);
            }
            List<BMainPollDisGasSout> drain = records.stream().filter(item -> item.getPollutantType().equals("1")).collect(Collectors.toList());
            if(CollectionUtils.isNotEmpty(drain)){
                drain.forEach(item->{
                    item.setVent("一般排放口");
                });
                res.addAll(drain);
            }
            list.setRecords(res);
            List<BMainPollDisGasSout> wzzs = records.stream().filter(item -> item.getPollutantType().equals("2")).collect(Collectors.toList());
            if(CollectionUtils.isNotEmpty(wzzs)){
                wzzs.forEach(item->{
                    item.setVent("无组织排放口");
                });
                res.addAll(wzzs);
            }
            List<BMainPollDisGasSout> amounts = records.stream().filter(item -> item.getPollutantType().equals("3")).collect(Collectors.toList());
            if(CollectionUtils.isNotEmpty(amounts)){
                amounts.forEach(item->{
                    item.setVent("全厂合计");
                });
                res.addAll(amounts);
            }
            list.setRecords(res);
        }
        return AjaxResults.success(list);
    }
    

    /**
     * 导出排污许可大气特殊情况列表
     */
    @ApiOperation(value = "导出排污许可大气特殊情况")
    @Log(title = "排污许可大气特殊情况", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainPollDisGasSout bMainPollDisGasSout)
    {
        List<BMainPollDisGasSout> list = bMainPollDisGasSoutService.list(new QueryWrapper<>());
        ExcelUtil<BMainPollDisGasSout> util = new ExcelUtil<BMainPollDisGasSout>(BMainPollDisGasSout.class);
        util.exportExcel(response, list, "排污许可大气特殊情况数据");
    }

    /**
     * 获取排污许可大气特殊情况详细信息
     */
    @ApiOperation(value = "获取排污许可大气特殊情况详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainPollDisGasSout> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(bMainPollDisGasSoutService.getById(id));
    }
    /**
     * 新增排污许可大气特殊情况
     */
    @ApiOperation(value = "新增排污许可大气特殊情况")
    @Log(title = "排污许可大气特殊情况", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainPollDisGasSout bMainPollDisGasSout) {
        return toAjax(bMainPollDisGasSoutService.save(bMainPollDisGasSout));
    }
    /**
     * 修改排污许可大气特殊情况
     */
    @ApiOperation(value = "修改排污许可大气特殊情况")
    @Log(title = "排污许可大气特殊情况", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainPollDisGasSoutVo bMainPollDisGasSoutVo) {
        if(ObjectUtil.isEmpty(bMainPollDisGasSoutVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainPollDisGasSoutVo.getSoutId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainPollDisGasSout bMainPollDisGasSout = bMainPollDisGasSoutService.getById(bMainPollDisGasSoutVo.getSoutId());
        if(ObjectUtil.isEmpty(bMainPollDisGasSout)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainPollDisGasSoutVo.getSoutId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainPollDisGasSoutVo,bMainPollDisGasSout);
        return toAjax(bMainPollDisGasSoutService.saveOrUpdate(bMainPollDisGasSout));
    }

    /**
     * 删除排污许可大气特殊情况
     */
    @ApiOperation(value = "删除排污许可大气特殊情况")
    @RequiresPermissions("talroad:BMainPollDisGasSout:remove")
    @Log(title = "排污许可大气特殊情况", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bMainPollDisGasSoutService.removeByIds(Arrays.asList(ids)));
    }
}
