package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainPollDisGasTout;
import cn.cecep.talroad.service.IBMainPollDisGasToutService;
import cn.cecep.talroad.vo.BMainPollDisGasToutVo;
import cn.cecep.talroad.vo.query.BMainPollDisGasToutQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 排污许可大气排放总计Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-18*/
@Api(tags = "排污许可--排污许可大气排放总计")
@RestController
@RequestMapping("/BMainPollDisGasTout")
public class BMainPollDisGasToutController extends BasesController
{
    @Autowired
    private IBMainPollDisGasToutService bMainPollDisGasToutService;

    /**
     * 查询排污许可大气排放总计列表
     */
    @ApiOperation(value = "查询排污许可大气排放总计")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainPollDisGasTout>> list(BMainPollDisGasToutQuery bMainPollDisGasTout)
    {
        return AjaxResults.success(bMainPollDisGasToutService.pullPage(bMainPollDisGasTout));
    }
    

    /**
     * 导出排污许可大气排放总计列表
     */
    @ApiOperation(value = "导出排污许可大气排放总计")
    @Log(title = "排污许可大气排放总计", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainPollDisGasTout bMainPollDisGasTout)
    {
        List<BMainPollDisGasTout> list = bMainPollDisGasToutService.selectBMainPollDisGasToutList(bMainPollDisGasTout);
        ExcelUtil<BMainPollDisGasTout> util = new ExcelUtil<BMainPollDisGasTout>(BMainPollDisGasTout.class);
        util.exportExcel(response, list, "排污许可大气排放总计数据");
    }

    /**
     * 获取排污许可大气排放总计详细信息
     */
    @ApiOperation(value = "获取排污许可大气排放总计详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainPollDisGasTout> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(bMainPollDisGasToutService.getById(id));
    }
    /**
     * 新增排污许可大气排放总计
     */
    @ApiOperation(value = "新增排污许可大气排放总计")
    @Log(title = "排污许可大气排放总计", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainPollDisGasTout bMainPollDisGasTout) {
        return toAjax(bMainPollDisGasToutService.save(bMainPollDisGasTout));
    }
    /**
     * 修改排污许可大气排放总计
     */
    @ApiOperation(value = "修改排污许可大气排放总计")
    @Log(title = "排污许可大气排放总计", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainPollDisGasToutVo bMainPollDisGasToutVo) {
        if(ObjectUtil.isEmpty(bMainPollDisGasToutVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainPollDisGasToutVo.getToutId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainPollDisGasTout bMainPollDisGasTout = bMainPollDisGasToutService.getById(bMainPollDisGasToutVo.getToutId());
        if(ObjectUtil.isEmpty(bMainPollDisGasTout)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainPollDisGasToutVo.getToutId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainPollDisGasToutVo,bMainPollDisGasTout);
        return toAjax(bMainPollDisGasToutService.saveOrUpdate(bMainPollDisGasTout));
    }

    /**
     * 删除排污许可大气排放总计
     */
    @ApiOperation(value = "删除排污许可大气排放总计")
    @RequiresPermissions("talroad:BMainPollDisGasTout:remove")
    @Log(title = "排污许可大气排放总计", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bMainPollDisGasToutService.removeByIds(Arrays.asList(ids)));
    }
}
