package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainPollDisInformation;
import cn.cecep.talroad.service.IBMainPollDisInformationService;
import cn.cecep.talroad.vo.BMainPollDisInformationVo;
import cn.cecep.talroad.vo.query.BMainPollDisInformationQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 排污许可信息公开Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-18*/
@Api(tags = "排污许可--排污许可信息公开")
@RestController
@RequestMapping("/BMainPollDisInformation")
public class BMainPollDisInformationController extends BasesController
{
    @Autowired
    private IBMainPollDisInformationService bMainPollDisInformationService;

    /**
     * 查询排污许可信息公开列表
     */
    @ApiOperation(value = "查询排污许可信息公开")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainPollDisInformation>> list(BMainPollDisInformationQuery bMainPollDisInformation)
    {
        Page<BMainPollDisInformation> page = new Page<>(bMainPollDisInformation.getPageNum(), bMainPollDisInformation.getPageSize());
        IPage<BMainPollDisInformation> list = bMainPollDisInformationService.page(page, BMainPollDisInformationQuery.createLambdaQueryWrapper(bMainPollDisInformation));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出排污许可信息公开列表
     */
    @ApiOperation(value = "导出排污许可信息公开")
    @Log(title = "排污许可信息公开", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainPollDisInformation bMainPollDisInformation)
    {
        List<BMainPollDisInformation> list = bMainPollDisInformationService.selectBMainPollDisInformationList(bMainPollDisInformation);
        ExcelUtil<BMainPollDisInformation> util = new ExcelUtil<BMainPollDisInformation>(BMainPollDisInformation.class);
        util.exportExcel(response, list, "排污许可信息公开数据");
    }

    /**
     * 获取排污许可信息公开详细信息
     */
    @ApiOperation(value = "获取排污许可信息公开详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainPollDisInformation> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(bMainPollDisInformationService.getById(id));
    }
    /**
     * 新增排污许可信息公开
     */
    @ApiOperation(value = "新增排污许可信息公开")
    @Log(title = "排污许可信息公开", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainPollDisInformation bMainPollDisInformation) {
        return toAjax(bMainPollDisInformationService.save(bMainPollDisInformation));
    }
    /**
     * 修改排污许可信息公开
     */
    @ApiOperation(value = "修改排污许可信息公开")
    @Log(title = "排污许可信息公开", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainPollDisInformationVo bMainPollDisInformationVo) {
        if(ObjectUtil.isEmpty(bMainPollDisInformationVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainPollDisInformationVo.getInformationId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainPollDisInformation bMainPollDisInformation = bMainPollDisInformationService.getById(bMainPollDisInformationVo.getInformationId());
        if(ObjectUtil.isEmpty(bMainPollDisInformation)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainPollDisInformationVo.getInformationId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainPollDisInformationVo,bMainPollDisInformation);
        return toAjax(bMainPollDisInformationService.saveOrUpdate(bMainPollDisInformation));
    }

    /**
     * 删除排污许可信息公开
     */
    @ApiOperation(value = "删除排污许可信息公开")
    @RequiresPermissions("talroad:BMainPollDisInformation:remove")
    @Log(title = "排污许可信息公开", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bMainPollDisInformationService.removeByIds(Arrays.asList(ids)));
    }
}
