package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainPollDisNoise;
import cn.cecep.talroad.service.IBMainPollDisNoiseService;
import cn.cecep.talroad.vo.BMainPollDisNoiseVo;
import cn.cecep.talroad.vo.query.BMainPollDisNoiseQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 排污许可其他许可噪声排放Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-18*/
@Api(tags = "排污许可--排污许可其他许可噪声排放")
@RestController
@RequestMapping("/BMainPollDisNoise")
public class BMainPollDisNoiseController extends BasesController
{
    @Autowired
    private IBMainPollDisNoiseService bMainPollDisNoiseService;

    /**
     * 查询排污许可其他许可噪声排放列表
     */
    @ApiOperation(value = "查询排污许可其他许可噪声排放")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainPollDisNoise>> list(BMainPollDisNoiseQuery bMainPollDisNoise)
    {
        Page<BMainPollDisNoise> page = new Page<>(bMainPollDisNoise.getPageNum(), bMainPollDisNoise.getPageSize());
        IPage<BMainPollDisNoise> list = bMainPollDisNoiseService.page(page, BMainPollDisNoiseQuery.createLambdaQueryWrapper(bMainPollDisNoise));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出排污许可其他许可噪声排放列表
     */
    @ApiOperation(value = "导出排污许可其他许可噪声排放")
    @Log(title = "排污许可其他许可噪声排放", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainPollDisNoise bMainPollDisNoise)
    {
        List<BMainPollDisNoise> list = bMainPollDisNoiseService.selectBMainPollDisNoiseList(bMainPollDisNoise);
        ExcelUtil<BMainPollDisNoise> util = new ExcelUtil<BMainPollDisNoise>(BMainPollDisNoise.class);
        util.exportExcel(response, list, "排污许可其他许可噪声排放数据");
    }

    /**
     * 获取排污许可其他许可噪声排放详细信息
     */
    @ApiOperation(value = "获取排污许可其他许可噪声排放详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainPollDisNoise> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(bMainPollDisNoiseService.getById(id));
    }
    /**
     * 新增排污许可其他许可噪声排放
     */
    @ApiOperation(value = "新增排污许可其他许可噪声排放")
    @Log(title = "排污许可其他许可噪声排放", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainPollDisNoise bMainPollDisNoise) {
        return toAjax(bMainPollDisNoiseService.save(bMainPollDisNoise));
    }
    /**
     * 修改排污许可其他许可噪声排放
     */
    @ApiOperation(value = "修改排污许可其他许可噪声排放")
    @Log(title = "排污许可其他许可噪声排放", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainPollDisNoiseVo bMainPollDisNoiseVo) {
        if(ObjectUtil.isEmpty(bMainPollDisNoiseVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainPollDisNoiseVo.getNoiseId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainPollDisNoise bMainPollDisNoise = bMainPollDisNoiseService.getById(bMainPollDisNoiseVo.getNoiseId());
        if(ObjectUtil.isEmpty(bMainPollDisNoise)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainPollDisNoiseVo.getNoiseId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainPollDisNoiseVo,bMainPollDisNoise);
        return toAjax(bMainPollDisNoiseService.saveOrUpdate(bMainPollDisNoise));
    }

    /**
     * 删除排污许可其他许可噪声排放
     */
    @ApiOperation(value = "删除排污许可其他许可噪声排放")
    @RequiresPermissions("talroad:BMainPollDisNoise:remove")
    @Log(title = "排污许可其他许可噪声排放", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bMainPollDisNoiseService.removeByIds(Arrays.asList(ids)));
    }
}
