package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainPollDisNoiseRegulation;
import cn.cecep.talroad.service.IBMainPollDisNoiseRegulationService;
import cn.cecep.talroad.vo.BMainPollDisNoiseRegulationVo;
import cn.cecep.talroad.vo.query.BMainPollDisNoiseRegulationQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 排污许可其他许可改正规定Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-18*/
@Api(tags = "排污许可--排污许可其他许可改正规定")
@RestController
@RequestMapping("/BMainPollDisNoiseRegulation")
public class BMainPollDisNoiseRegulationController extends BasesController
{
    @Autowired
    private IBMainPollDisNoiseRegulationService bMainPollDisNoiseRegulationService;

    /**
     * 查询排污许可其他许可改正规定列表
     */
    @ApiOperation(value = "查询排污许可其他许可改正规定")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainPollDisNoiseRegulation>> list(BMainPollDisNoiseRegulationQuery bMainPollDisNoiseRegulation)
    {
        Page<BMainPollDisNoiseRegulation> page = new Page<>(bMainPollDisNoiseRegulation.getPageNum(), bMainPollDisNoiseRegulation.getPageSize());
        IPage<BMainPollDisNoiseRegulation> list = bMainPollDisNoiseRegulationService.page(page, BMainPollDisNoiseRegulationQuery.createLambdaQueryWrapper(bMainPollDisNoiseRegulation));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出排污许可其他许可改正规定列表
     */
    @ApiOperation(value = "导出排污许可其他许可改正规定")
    @Log(title = "排污许可其他许可改正规定", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainPollDisNoiseRegulation bMainPollDisNoiseRegulation)
    {
        List<BMainPollDisNoiseRegulation> list = bMainPollDisNoiseRegulationService.selectBMainPollDisNoiseRegulationList(bMainPollDisNoiseRegulation);
        ExcelUtil<BMainPollDisNoiseRegulation> util = new ExcelUtil<BMainPollDisNoiseRegulation>(BMainPollDisNoiseRegulation.class);
        util.exportExcel(response, list, "排污许可其他许可改正规定数据");
    }

    /**
     * 获取排污许可其他许可改正规定详细信息
     */
    @ApiOperation(value = "获取排污许可其他许可改正规定详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainPollDisNoiseRegulation> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(bMainPollDisNoiseRegulationService.getById(id));
    }
    /**
     * 新增排污许可其他许可改正规定
     */
    @ApiOperation(value = "新增排污许可其他许可改正规定")
    @Log(title = "排污许可其他许可改正规定", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainPollDisNoiseRegulation bMainPollDisNoiseRegulation) {
        return toAjax(bMainPollDisNoiseRegulationService.save(bMainPollDisNoiseRegulation));
    }
    /**
     * 修改排污许可其他许可改正规定
     */
    @ApiOperation(value = "修改排污许可其他许可改正规定")
    @Log(title = "排污许可其他许可改正规定", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainPollDisNoiseRegulationVo bMainPollDisNoiseRegulationVo) {
        if(ObjectUtil.isEmpty(bMainPollDisNoiseRegulationVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainPollDisNoiseRegulationVo.getRegulationId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainPollDisNoiseRegulation bMainPollDisNoiseRegulation = bMainPollDisNoiseRegulationService.getById(bMainPollDisNoiseRegulationVo.getRegulationId());
        if(ObjectUtil.isEmpty(bMainPollDisNoiseRegulation)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainPollDisNoiseRegulationVo.getRegulationId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainPollDisNoiseRegulationVo,bMainPollDisNoiseRegulation);
        return toAjax(bMainPollDisNoiseRegulationService.saveOrUpdate(bMainPollDisNoiseRegulation));
    }

    /**
     * 删除排污许可其他许可改正规定
     */
    @ApiOperation(value = "删除排污许可其他许可改正规定")
    @RequiresPermissions("talroad:BMainPollDisNoiseRegulation:remove")
    @Log(title = "排污许可其他许可改正规定", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bMainPollDisNoiseRegulationService.removeByIds(Arrays.asList(ids)));
    }
}
