package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainPollDisNoiseRequire;
import cn.cecep.talroad.service.IBMainPollDisNoiseRequireService;
import cn.cecep.talroad.vo.BMainPollDisNoiseRequireVo;
import cn.cecep.talroad.vo.query.BMainPollDisNoiseRequireQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 排污许可环境管理要求Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-18*/
@Api(tags = "排污许可--排污许可环境管理要求")
@RestController
@RequestMapping("/BMainPollDisNoiseRequire")
public class BMainPollDisNoiseRequireController extends BasesController
{
    @Autowired
    private IBMainPollDisNoiseRequireService bMainPollDisNoiseRequireService;

    /**
     * 查询排污许可环境管理要求列表
     */
    @ApiOperation(value = "查询排污许可环境管理要求")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainPollDisNoiseRequire>> list(BMainPollDisNoiseRequireQuery bMainPollDisNoiseRequire)
    {
        Page<BMainPollDisNoiseRequire> page = new Page<>(bMainPollDisNoiseRequire.getPageNum(), bMainPollDisNoiseRequire.getPageSize());
        IPage<BMainPollDisNoiseRequire> list = bMainPollDisNoiseRequireService.page(page, BMainPollDisNoiseRequireQuery.createLambdaQueryWrapper(bMainPollDisNoiseRequire));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出排污许可环境管理要求列表
     */
    @ApiOperation(value = "导出排污许可环境管理要求")
    @Log(title = "排污许可环境管理要求", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainPollDisNoiseRequire bMainPollDisNoiseRequire)
    {
        List<BMainPollDisNoiseRequire> list = bMainPollDisNoiseRequireService.selectBMainPollDisNoiseRequireList(bMainPollDisNoiseRequire);
        ExcelUtil<BMainPollDisNoiseRequire> util = new ExcelUtil<BMainPollDisNoiseRequire>(BMainPollDisNoiseRequire.class);
        util.exportExcel(response, list, "排污许可环境管理要求数据");
    }

    /**
     * 获取排污许可环境管理要求详细信息
     */
    @ApiOperation(value = "获取排污许可环境管理要求详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainPollDisNoiseRequire> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(bMainPollDisNoiseRequireService.getById(id));
    }
    /**
     * 新增排污许可环境管理要求
     */
    @ApiOperation(value = "新增排污许可环境管理要求")
    @Log(title = "排污许可环境管理要求", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainPollDisNoiseRequire bMainPollDisNoiseRequire) {
        return toAjax(bMainPollDisNoiseRequireService.save(bMainPollDisNoiseRequire));
    }
    /**
     * 修改排污许可环境管理要求
     */
    @ApiOperation(value = "修改排污许可环境管理要求")
    @Log(title = "排污许可环境管理要求", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainPollDisNoiseRequireVo bMainPollDisNoiseRequireVo) {
        if(ObjectUtil.isEmpty(bMainPollDisNoiseRequireVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainPollDisNoiseRequireVo.getRequireId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainPollDisNoiseRequire bMainPollDisNoiseRequire = bMainPollDisNoiseRequireService.getById(bMainPollDisNoiseRequireVo.getRequireId());
        if(ObjectUtil.isEmpty(bMainPollDisNoiseRequire)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainPollDisNoiseRequireVo.getRequireId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainPollDisNoiseRequireVo,bMainPollDisNoiseRequire);
        return toAjax(bMainPollDisNoiseRequireService.saveOrUpdate(bMainPollDisNoiseRequire));
    }

    /**
     * 删除排污许可环境管理要求
     */
    @ApiOperation(value = "删除排污许可环境管理要求")
    @RequiresPermissions("talroad:BMainPollDisNoiseRequire:remove")
    @Log(title = "排污许可环境管理要求", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bMainPollDisNoiseRequireService.removeByIds(Arrays.asList(ids)));
    }
}
