package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainPollDisSelfMon;
import cn.cecep.talroad.service.IBMainPollDisSelfMonService;
import cn.cecep.talroad.vo.BMainPollDisSelfMonVo;
import cn.cecep.talroad.vo.query.BMainPollDisSelfMonQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 排污许可自行监测要求Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-18*/
@Api(tags = "排污许可--排污许可自行监测要求")
@RestController
@RequestMapping("/BMainPollDisSelfMon")
public class BMainPollDisSelfMonController extends BasesController
{
    @Autowired
    private IBMainPollDisSelfMonService bMainPollDisSelfMonService;

    /**
     * 查询排污许可自行监测要求列表
     */
    @ApiOperation(value = "查询排污许可自行监测要求")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainPollDisSelfMon>> list(BMainPollDisSelfMonQuery bMainPollDisSelfMon)
    {
        Page<BMainPollDisSelfMon> page = new Page<>(bMainPollDisSelfMon.getPageNum(), bMainPollDisSelfMon.getPageSize());
        IPage<BMainPollDisSelfMon> list = bMainPollDisSelfMonService.page(page, BMainPollDisSelfMonQuery.createLambdaQueryWrapper(bMainPollDisSelfMon));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出排污许可自行监测要求列表
     */
    @ApiOperation(value = "导出排污许可自行监测要求")
    @Log(title = "排污许可自行监测要求", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainPollDisSelfMon bMainPollDisSelfMon)
    {
        List<BMainPollDisSelfMon> list = bMainPollDisSelfMonService.selectBMainPollDisSelfMonList(bMainPollDisSelfMon);
        ExcelUtil<BMainPollDisSelfMon> util = new ExcelUtil<BMainPollDisSelfMon>(BMainPollDisSelfMon.class);
        util.exportExcel(response, list, "排污许可自行监测要求数据");
    }

    /**
     * 获取排污许可自行监测要求详细信息
     */
    @ApiOperation(value = "获取排污许可自行监测要求详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainPollDisSelfMon> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(bMainPollDisSelfMonService.getById(id));
    }
    /**
     * 新增排污许可自行监测要求
     */
    @ApiOperation(value = "新增排污许可自行监测要求")
    @Log(title = "排污许可自行监测要求", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainPollDisSelfMon bMainPollDisSelfMon) {
        return toAjax(bMainPollDisSelfMonService.save(bMainPollDisSelfMon));
    }
    /**
     * 修改排污许可自行监测要求
     */
    @ApiOperation(value = "修改排污许可自行监测要求")
    @Log(title = "排污许可自行监测要求", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainPollDisSelfMonVo bMainPollDisSelfMonVo) {
        if(ObjectUtil.isEmpty(bMainPollDisSelfMonVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainPollDisSelfMonVo.getMonId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainPollDisSelfMon bMainPollDisSelfMon = bMainPollDisSelfMonService.getById(bMainPollDisSelfMonVo.getMonId());
        if(ObjectUtil.isEmpty(bMainPollDisSelfMon)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainPollDisSelfMonVo.getMonId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainPollDisSelfMonVo,bMainPollDisSelfMon);
        return toAjax(bMainPollDisSelfMonService.saveOrUpdate(bMainPollDisSelfMon));
    }

    /**
     * 删除排污许可自行监测要求
     */
    @ApiOperation(value = "删除排污许可自行监测要求")
    @RequiresPermissions("talroad:BMainPollDisSelfMon:remove")
    @Log(title = "排污许可自行监测要求", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bMainPollDisSelfMonService.removeByIds(Arrays.asList(ids)));
    }
}
