package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainPollDisWaterMout;
import cn.cecep.talroad.service.IBMainPollDisWaterMoutService;
import cn.cecep.talroad.vo.BMainPollDisWaterMoutVo;
import cn.cecep.talroad.vo.query.BMainPollDisWaterMoutQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;


/**
 * * 开发公司：中节能天融科技有限公司
 * 排污许可水污染物主排放口Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-18*/
@Api(tags = "排污许可--排污许可水污染物主排放口")
@RestController
@RequestMapping("/BMainPollDisWaterMout")
public class BMainPollDisWaterMoutController extends BasesController
{
    @Autowired
    private IBMainPollDisWaterMoutService bMainPollDisWaterMoutService;

    /**
     * 查询排污许可水污染物主排放口列表
     */
    @ApiOperation(value = "查询排污许可水污染物主排放口")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainPollDisWaterMout>> list(BMainPollDisWaterMoutQuery bMainPollDisWaterMout)
    {
        Page<BMainPollDisWaterMout> page = new Page<>(bMainPollDisWaterMout.getPageNum(), bMainPollDisWaterMout.getPageSize());
        IPage<BMainPollDisWaterMout> list = bMainPollDisWaterMoutService.page(page, BMainPollDisWaterMoutQuery.createLambdaQueryWrapper(bMainPollDisWaterMout));
        List<BMainPollDisWaterMout> records = list.getRecords();
        if(CollectionUtils.isNotEmpty(records)){
            List<BMainPollDisWaterMout> mainDrain = records.stream().filter(item -> item.getOutletUseTotal().equals("1")).collect(Collectors.toList());
            if(CollectionUtils.isNotEmpty(mainDrain)){
                mainDrain.forEach(item->{
                    item.setPollutantsMgRate(item.getPollutantsType());
                    item.setPollutantsType("");
                    item.setVent("主要排放口合计");
                });
            }
            List<BMainPollDisWaterMout> drain = records.stream().filter(item -> item.getOutletUseTotal().equals("0")).collect(Collectors.toList());
            if(CollectionUtils.isNotEmpty(drain)){
                drain.sort(Comparator.comparing(BMainPollDisWaterMout::getOutletCode, Comparator.naturalOrder()));
            }
            ArrayList<BMainPollDisWaterMout> res = Lists.newArrayList();
            res.addAll(drain);
            res.addAll(mainDrain);
            list.setRecords(res);
        }
        return AjaxResults.success(list);
    }
    

    /**
     * 导出排污许可水污染物主排放口列表
     */
    @ApiOperation(value = "导出排污许可水污染物主排放口")
    @Log(title = "排污许可水污染物主排放口", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainPollDisWaterMout bMainPollDisWaterMout)
    {
        List<BMainPollDisWaterMout> list = bMainPollDisWaterMoutService.selectBMainPollDisWaterMoutList(bMainPollDisWaterMout);
        ExcelUtil<BMainPollDisWaterMout> util = new ExcelUtil<BMainPollDisWaterMout>(BMainPollDisWaterMout.class);
        util.exportExcel(response, list, "排污许可水污染物主排放口数据");
    }

    /**
     * 获取排污许可水污染物主排放口详细信息
     */
    @ApiOperation(value = "获取排污许可水污染物主排放口详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainPollDisWaterMout> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(bMainPollDisWaterMoutService.getById(id));
    }
    /**
     * 新增排污许可水污染物主排放口
     */
    @ApiOperation(value = "新增排污许可水污染物主排放口")
    @Log(title = "排污许可水污染物主排放口", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainPollDisWaterMout bMainPollDisWaterMout) {
        return toAjax(bMainPollDisWaterMoutService.save(bMainPollDisWaterMout));
    }
    /**
     * 修改排污许可水污染物主排放口
     */
    @ApiOperation(value = "修改排污许可水污染物主排放口")
    @Log(title = "排污许可水污染物主排放口", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainPollDisWaterMoutVo bMainPollDisWaterMoutVo) {
        if(ObjectUtil.isEmpty(bMainPollDisWaterMoutVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainPollDisWaterMoutVo.getMoutId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainPollDisWaterMout bMainPollDisWaterMout = bMainPollDisWaterMoutService.getById(bMainPollDisWaterMoutVo.getMoutId());
        if(ObjectUtil.isEmpty(bMainPollDisWaterMout)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainPollDisWaterMoutVo.getMoutId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainPollDisWaterMoutVo,bMainPollDisWaterMout);
        return toAjax(bMainPollDisWaterMoutService.saveOrUpdate(bMainPollDisWaterMout));
    }

    /**
     * 删除排污许可水污染物主排放口
     */
    @ApiOperation(value = "删除排污许可水污染物主排放口")
    @RequiresPermissions("talroad:BMainPollDisWaterMout:remove")
    @Log(title = "排污许可水污染物主排放口", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bMainPollDisWaterMoutService.removeByIds(Arrays.asList(ids)));
    }
}
