package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainPollDisWaterOout;
import cn.cecep.talroad.service.IBMainPollDisWaterOoutService;
import cn.cecep.talroad.vo.BMainPollDisWaterOoutVo;
import cn.cecep.talroad.vo.query.BMainPollDisWaterOoutQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;


/**
 * * 开发公司：中节能天融科技有限公司
 * 排污许可水污染物一般排放口Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-18*/
@Api(tags = "排污许可--排污许可水污染物一般排放口")
@RestController
@RequestMapping("/BMainPollDisWaterOout")
public class BMainPollDisWaterOoutController extends BasesController
{
    @Autowired
    private IBMainPollDisWaterOoutService bMainPollDisWaterOoutService;

    /**
     * 查询排污许可水污染物一般排放口列表
     */
    @ApiOperation(value = "查询排污许可水污染物一般排放口")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainPollDisWaterOout>> list(BMainPollDisWaterOoutQuery bMainPollDisWaterOout)
    {
        Page<BMainPollDisWaterOout> page = new Page<>(bMainPollDisWaterOout.getPageNum(), bMainPollDisWaterOout.getPageSize());
        IPage<BMainPollDisWaterOout> list = bMainPollDisWaterOoutService.page(page, BMainPollDisWaterOoutQuery.createLambdaQueryWrapper(bMainPollDisWaterOout));
        List<BMainPollDisWaterOout> records = list.getRecords();
        if(CollectionUtils.isNotEmpty(records)){
            List<BMainPollDisWaterOout> mainDrain = records.stream().filter(item -> item.getOutletUseTotal().equals("1")).collect(Collectors.toList());
            if(CollectionUtils.isNotEmpty(mainDrain)){
                mainDrain.forEach(item->{
                    item.setPollutantsMgRate(item.getPollutantsType());
                    item.setPollutantsType("");
                    item.setVent("一般排放口合计");
                });
            }
            List<BMainPollDisWaterOout> drain = records.stream().filter(item -> item.getOutletUseTotal().equals("0")).collect(Collectors.toList());
            if(CollectionUtils.isNotEmpty(drain)){
                drain.sort(Comparator.comparing(BMainPollDisWaterOout::getOutletCode, Comparator.naturalOrder()));
            }
            ArrayList<BMainPollDisWaterOout> res = Lists.newArrayList();
            res.addAll(drain);
            res.addAll(mainDrain);
            list.setRecords(res);
        }
        return AjaxResults.success(list);
    }
    

    /**
     * 导出排污许可水污染物一般排放口列表
     */
    @ApiOperation(value = "导出排污许可水污染物一般排放口")
    @Log(title = "排污许可水污染物一般排放口", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainPollDisWaterOout bMainPollDisWaterOout)
    {
        List<BMainPollDisWaterOout> list = bMainPollDisWaterOoutService.selectBMainPollDisWaterOoutList(bMainPollDisWaterOout);
        ExcelUtil<BMainPollDisWaterOout> util = new ExcelUtil<BMainPollDisWaterOout>(BMainPollDisWaterOout.class);
        util.exportExcel(response, list, "排污许可水污染物一般排放口数据");
    }

    /**
     * 获取排污许可水污染物一般排放口详细信息
     */
    @ApiOperation(value = "获取排污许可水污染物一般排放口详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainPollDisWaterOout> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(bMainPollDisWaterOoutService.getById(id));
    }
    /**
     * 新增排污许可水污染物一般排放口
     */
    @ApiOperation(value = "新增排污许可水污染物一般排放口")
    @Log(title = "排污许可水污染物一般排放口", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainPollDisWaterOout bMainPollDisWaterOout) {
        return toAjax(bMainPollDisWaterOoutService.save(bMainPollDisWaterOout));
    }
    /**
     * 修改排污许可水污染物一般排放口
     */
    @ApiOperation(value = "修改排污许可水污染物一般排放口")
    @Log(title = "排污许可水污染物一般排放口", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainPollDisWaterOoutVo bMainPollDisWaterOoutVo) {
        if(ObjectUtil.isEmpty(bMainPollDisWaterOoutVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainPollDisWaterOoutVo.getOoutId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainPollDisWaterOout bMainPollDisWaterOout = bMainPollDisWaterOoutService.getById(bMainPollDisWaterOoutVo.getOoutId());
        if(ObjectUtil.isEmpty(bMainPollDisWaterOout)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainPollDisWaterOoutVo.getOoutId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainPollDisWaterOoutVo,bMainPollDisWaterOout);
        return toAjax(bMainPollDisWaterOoutService.saveOrUpdate(bMainPollDisWaterOout));
    }

    /**
     * 删除排污许可水污染物一般排放口
     */
    @ApiOperation(value = "删除排污许可水污染物一般排放口")
    @RequiresPermissions("talroad:BMainPollDisWaterOout:remove")
    @Log(title = "排污许可水污染物一般排放口", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bMainPollDisWaterOoutService.removeByIds(Arrays.asList(ids)));
    }
}
