package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainPollDisWaterSout;
import cn.cecep.talroad.service.IBMainPollDisWaterSoutService;
import cn.cecep.talroad.vo.BMainPollDisWaterSoutVo;
import cn.cecep.talroad.vo.query.BMainPollDisWaterSoutQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 排污许可水污染物特殊情况Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-18*/
@Api(tags = "排污许可--排污许可水污染物特殊情况")
@RestController
@RequestMapping("/BMainPollDisWaterSout")
public class BMainPollDisWaterSoutController extends BasesController
{
    @Autowired
    private IBMainPollDisWaterSoutService bMainPollDisWaterSoutService;

    /**
     * 查询排污许可水污染物特殊情况列表
     */
    @ApiOperation(value = "查询排污许可水污染物特殊情况")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainPollDisWaterSout>> list(BMainPollDisWaterSoutQuery bMainPollDisWaterSout)
    {
        Page<BMainPollDisWaterSout> page = new Page<>(bMainPollDisWaterSout.getPageNum(), bMainPollDisWaterSout.getPageSize());
        IPage<BMainPollDisWaterSout> list = bMainPollDisWaterSoutService.page(page, BMainPollDisWaterSoutQuery.createLambdaQueryWrapper(bMainPollDisWaterSout));

        if(CollectionUtils.isNotEmpty(list.getRecords())){
            list.getRecords().sort(Comparator.comparing(BMainPollDisWaterSout::getOutletCode, Comparator.naturalOrder()));
        }
        return AjaxResults.success(list);
    }
    

    /**
     * 导出排污许可水污染物特殊情况列表
     */
    @ApiOperation(value = "导出排污许可水污染物特殊情况")
    @Log(title = "排污许可水污染物特殊情况", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainPollDisWaterSout bMainPollDisWaterSout)
    {
        List<BMainPollDisWaterSout> list = bMainPollDisWaterSoutService.selectBMainPollDisWaterSoutList(bMainPollDisWaterSout);
        ExcelUtil<BMainPollDisWaterSout> util = new ExcelUtil<BMainPollDisWaterSout>(BMainPollDisWaterSout.class);
        util.exportExcel(response, list, "排污许可水污染物特殊情况数据");
    }

    /**
     * 获取排污许可水污染物特殊情况详细信息
     */
    @ApiOperation(value = "获取排污许可水污染物特殊情况详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainPollDisWaterSout> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(bMainPollDisWaterSoutService.getById(id));
    }
    /**
     * 新增排污许可水污染物特殊情况
     */
    @ApiOperation(value = "新增排污许可水污染物特殊情况")
    @Log(title = "排污许可水污染物特殊情况", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainPollDisWaterSout bMainPollDisWaterSout) {
        return toAjax(bMainPollDisWaterSoutService.save(bMainPollDisWaterSout));
    }
    /**
     * 修改排污许可水污染物特殊情况
     */
    @ApiOperation(value = "修改排污许可水污染物特殊情况")
    @Log(title = "排污许可水污染物特殊情况", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainPollDisWaterSoutVo bMainPollDisWaterSoutVo) {
        if(ObjectUtil.isEmpty(bMainPollDisWaterSoutVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainPollDisWaterSoutVo.getSoutId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainPollDisWaterSout bMainPollDisWaterSout = bMainPollDisWaterSoutService.getById(bMainPollDisWaterSoutVo.getSoutId());
        if(ObjectUtil.isEmpty(bMainPollDisWaterSout)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainPollDisWaterSoutVo.getSoutId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainPollDisWaterSoutVo,bMainPollDisWaterSout);
        return toAjax(bMainPollDisWaterSoutService.saveOrUpdate(bMainPollDisWaterSout));
    }

    /**
     * 删除排污许可水污染物特殊情况
     */
    @ApiOperation(value = "删除排污许可水污染物特殊情况")
    @RequiresPermissions("talroad:BMainPollDisWaterSout:remove")
    @Log(title = "排污许可水污染物特殊情况", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bMainPollDisWaterSoutService.removeByIds(Arrays.asList(ids)));
    }
}
