package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainPollDisWaterTout;
import cn.cecep.talroad.service.IBMainPollDisWaterToutService;
import cn.cecep.talroad.vo.BMainPollDisWaterToutVo;
import cn.cecep.talroad.vo.query.BMainPollDisWaterToutQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 排污许可水污染物排放总计Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-18*/
@Api(tags = "排污许可--排污许可水污染物排放总计")
@RestController
@RequestMapping("/BMainPollDisWaterTout")
public class BMainPollDisWaterToutController extends BasesController
{
    @Autowired
    private IBMainPollDisWaterToutService bMainPollDisWaterToutService;

    /**
     * 查询排污许可水污染物排放总计列表
     */
    @ApiOperation(value = "查询排污许可水污染物排放总计")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainPollDisWaterTout>> list(BMainPollDisWaterToutQuery bMainPollDisWaterTout)
    {
        return AjaxResults.success(bMainPollDisWaterToutService.pullPage(bMainPollDisWaterTout));
    }
    

    /**
     * 导出排污许可水污染物排放总计列表
     */
    @ApiOperation(value = "导出排污许可水污染物排放总计")
    @Log(title = "排污许可水污染物排放总计", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BMainPollDisWaterTout bMainPollDisWaterTout)
    {
        List<BMainPollDisWaterTout> list = bMainPollDisWaterToutService.selectBMainPollDisWaterToutList(bMainPollDisWaterTout);
        ExcelUtil<BMainPollDisWaterTout> util = new ExcelUtil<BMainPollDisWaterTout>(BMainPollDisWaterTout.class);
        util.exportExcel(response, list, "排污许可水污染物排放总计数据");
    }

    /**
     * 获取排污许可水污染物排放总计详细信息
     */
    @ApiOperation(value = "获取排污许可水污染物排放总计详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainPollDisWaterTout> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(bMainPollDisWaterToutService.getById(id));
    }
    /**
     * 新增排污许可水污染物排放总计
     */
    @ApiOperation(value = "新增排污许可水污染物排放总计")
    @Log(title = "排污许可水污染物排放总计", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainPollDisWaterTout bMainPollDisWaterTout) {
        return toAjax(bMainPollDisWaterToutService.save(bMainPollDisWaterTout));
    }
    /**
     * 修改排污许可水污染物排放总计
     */
    @ApiOperation(value = "修改排污许可水污染物排放总计")
    @Log(title = "排污许可水污染物排放总计", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainPollDisWaterToutVo bMainPollDisWaterToutVo) {
        if(ObjectUtil.isEmpty(bMainPollDisWaterToutVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainPollDisWaterToutVo.getToutId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainPollDisWaterTout bMainPollDisWaterTout = bMainPollDisWaterToutService.getById(bMainPollDisWaterToutVo.getToutId());
        if(ObjectUtil.isEmpty(bMainPollDisWaterTout)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainPollDisWaterToutVo.getToutId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainPollDisWaterToutVo,bMainPollDisWaterTout);
        return toAjax(bMainPollDisWaterToutService.saveOrUpdate(bMainPollDisWaterTout));
    }

    /**
     * 删除排污许可水污染物排放总计
     */
    @ApiOperation(value = "删除排污许可水污染物排放总计")
    @RequiresPermissions("talroad:BMainPollDisWaterTout:remove")
    @Log(title = "排污许可水污染物排放总计", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bMainPollDisWaterToutService.removeByIds(Arrays.asList(ids)));
    }
}
