package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainSolidHazardous;
import cn.cecep.talroad.service.IBMainSolidHazardousService;
import cn.cecep.talroad.vo.BMainSolidHazardousVo;
import cn.cecep.talroad.vo.query.BMainSolidHazardousQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固危废贮存信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-18*/
@Api(tags = "产治排跟踪--固危废贮存信息")
@RestController
@RequestMapping("/BMainSolidHazardous")
public class BMainSolidHazardousController extends BasesController
{
    @Autowired
    private IBMainSolidHazardousService bMainSolidHazardousService;

    /**
     * 查询固危废贮存信息列表
     */
    @ApiOperation(value = "查询固危废贮存信息")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainSolidHazardous>> list(BMainSolidHazardousQuery bMainSolidHazardous)
    {
        Page<BMainSolidHazardous> page = new Page<>(bMainSolidHazardous.getPageNum(), bMainSolidHazardous.getPageSize());
        IPage<BMainSolidHazardous> list = bMainSolidHazardousService.page(page, BMainSolidHazardousQuery.createLambdaQueryWrapper(bMainSolidHazardous));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出固危废贮存信息列表
     */
    @ApiOperation(value = "导出固危废贮存信息")
    @Log(title = "固危废贮存信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BMainSolidHazardous bMainSolidHazardous)
    {
        List<BMainSolidHazardous> list = bMainSolidHazardousService.selectBMainSolidHazardousList(bMainSolidHazardous);
        ExcelUtil<BMainSolidHazardous> util = new ExcelUtil<BMainSolidHazardous>(BMainSolidHazardous.class);
        util.exportExcel(response, list, "固危废贮存信息数据");
    }

    /**
     * 获取固危废贮存信息详细信息
     */
    @ApiOperation(value = "获取固危废贮存信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainSolidHazardous> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bMainSolidHazardousService.getById(id));
    }
    /**
     * 新增固危废贮存信息
     */
    @ApiOperation(value = "新增固危废贮存信息")
    @Log(title = "固危废贮存信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainSolidHazardous bMainSolidHazardous) {
        return toAjax(bMainSolidHazardousService.save(bMainSolidHazardous));
    }
    /**
     * 修改固危废贮存信息
     */
    @ApiOperation(value = "修改固危废贮存信息")
    @Log(title = "固危废贮存信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainSolidHazardousVo bMainSolidHazardousVo) {
        if(ObjectUtil.isEmpty(bMainSolidHazardousVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainSolidHazardousVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainSolidHazardous bMainSolidHazardous = bMainSolidHazardousService.getById(bMainSolidHazardousVo.getId());
        if(ObjectUtil.isEmpty(bMainSolidHazardous)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainSolidHazardousVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainSolidHazardousVo,bMainSolidHazardous);
        return toAjax(bMainSolidHazardousService.saveOrUpdate(bMainSolidHazardous));
    }

    /**
     * 删除固危废贮存信息
     */
    @ApiOperation(value = "删除固危废贮存信息")
    @RequiresPermissions("talroad:BMainSolidHazardous:remove")
    @Log(title = "固危废贮存信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bMainSolidHazardousService.removeByIds(Arrays.asList(ids)));
    }
}
