package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BMainSolidWasteOutput;
import cn.cecep.talroad.service.IBMainSolidWasteOutputService;
import cn.cecep.talroad.vo.BMainSolidWasteOutputVo;
import cn.cecep.talroad.vo.EChartsVO;
import cn.cecep.talroad.vo.query.BMainSolidWasteOutputQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固废产生与处理Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "基本档案--固废产生与处理")
@RestController
@RequestMapping("/bMainSolidWasteOutput")
public class BMainSolidWasteOutputController extends BasesController
{
    @Autowired
    private IBMainSolidWasteOutputService bMainSolidWasteOutputService;

    /**
     * 查询固废产生与处理列表
     */
    @ApiOperation(value = "查询固废产生与处理")
    @GetMapping("/list")
    public AjaxResults<IPage<BMainSolidWasteOutput>> list(BMainSolidWasteOutputQuery bMainSolidWasteOutput)
    {
        Page<BMainSolidWasteOutput> page = new Page<>(bMainSolidWasteOutput.getPageNum(), bMainSolidWasteOutput.getPageSize());
        IPage<BMainSolidWasteOutput> list = bMainSolidWasteOutputService.page(page, BMainSolidWasteOutputQuery.createLambdaQueryWrapper(bMainSolidWasteOutput));
        return AjaxResults.success(list);
    }

    @ApiOperation(value = "查询固废产生与处理曲线图数据")
    @GetMapping("/charData")
    public AjaxResults<EChartsVO> getCharData(BMainSolidWasteOutputQuery bMainSolidWasteOutput)
    {
        EChartsVO charData = bMainSolidWasteOutputService.getCharData(bMainSolidWasteOutput);
        return AjaxResults.success(charData);
    }

    /**
     * 导出固废产生与处理列表
     */
    @ApiOperation(value = "导出固废产生与处理")
    @Log(title = "固废产生与处理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BMainSolidWasteOutputQuery bMainSolidWasteOutput)
    {
        List<BMainSolidWasteOutput> list = bMainSolidWasteOutputService.list(BMainSolidWasteOutputQuery.createLambdaQueryWrapper(bMainSolidWasteOutput));
        ExcelUtil<BMainSolidWasteOutput> util = new ExcelUtil<BMainSolidWasteOutput>(BMainSolidWasteOutput.class);
        util.exportExcel(response, list, "固废产生与处理数据");
    }

    /**
     * 获取固废产生与处理详细信息
     */
    @ApiOperation(value = "获取固废产生与处理详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BMainSolidWasteOutput> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bMainSolidWasteOutputService.getById(id));
    }
    /**
     * 新增固废产生与处理
     */
    @ApiOperation(value = "新增固废产生与处理")
    @Log(title = "固废产生与处理", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BMainSolidWasteOutput bMainSolidWasteOutput) {
        return toAjax(bMainSolidWasteOutputService.save(bMainSolidWasteOutput));
    }
    /**
     * 修改固废产生与处理
     */
    @ApiOperation(value = "修改固废产生与处理")
    @Log(title = "固废产生与处理", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BMainSolidWasteOutputVo bMainSolidWasteOutputVo) {
        if(ObjectUtil.isEmpty(bMainSolidWasteOutputVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bMainSolidWasteOutputVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BMainSolidWasteOutput bMainSolidWasteOutput = bMainSolidWasteOutputService.getById(bMainSolidWasteOutputVo.getId());
        if(ObjectUtil.isEmpty(bMainSolidWasteOutput)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bMainSolidWasteOutputVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bMainSolidWasteOutputVo,bMainSolidWasteOutput);
        return toAjax(bMainSolidWasteOutputService.saveOrUpdate(bMainSolidWasteOutput));
    }

    /**
     * 删除固废产生与处理
     */
    @ApiOperation(value = "删除固废产生与处理")
    @RequiresPermissions("talroad:output:remove")
    @Log(title = "固废产生与处理", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(bMainSolidWasteOutputService.removeByIds(Arrays.asList(ids)));
    }
}
