package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgGasEmissionQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgGasEmission;
import cn.cecep.talroad.vo.BStatTableZxbgGasEmissionVo;
import cn.cecep.talroad.service.IBStatTableZxbgGasEmissionService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--有组织废气污染物排放浓度监测数据Controller 控制层
 *
 * @author szhpt
 * @date 2023-08-01*/
@Api(tags = "固定源产排治--执行报告--有组织废气污染物排放浓度监测数据")
@RestController
@RequestMapping("/execution/gas/emission")
public class BStatTableZxbgGasEmissionController extends BasesController
{
    @Autowired
    private IBStatTableZxbgGasEmissionService bStatTableZxbgGasEmissionService;

    /**
     * 查询固定源产排治--执行报告--有组织废气污染物排放浓度监测数据列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--有组织废气污染物排放浓度监测数据")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgGasEmission>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgGasEmissionService.selectBStatTableZxbgGasEmissionListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--有组织废气污染物排放浓度监测数据列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--有组织废气污染物排放浓度监测数据")
    @Log(title = "固定源产排治--执行报告--有组织废气污染物排放浓度监测数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgGasEmission>> iPageAjaxResults = bStatTableZxbgGasEmissionService.selectBStatTableZxbgGasEmissionListBy(query);
        ExcelUtil<BStatTableZxbgGasEmission> util = new ExcelUtil<BStatTableZxbgGasEmission>(BStatTableZxbgGasEmission.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "固定源产排治--执行报告--有组织废气污染物排放浓度监测数据数据");
    }

    /**
     * 获取固定源产排治--执行报告--有组织废气污染物排放浓度监测数据详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--有组织废气污染物排放浓度监测数据详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgGasEmission> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgGasEmissionService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--有组织废气污染物排放浓度监测数据
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--有组织废气污染物排放浓度监测数据")
    @Log(title = "固定源产排治--执行报告--有组织废气污染物排放浓度监测数据", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgGasEmission bStatTableZxbgGasEmission) {
        return toAjax(bStatTableZxbgGasEmissionService.save(bStatTableZxbgGasEmission));
    }
    /**
     * 修改固定源产排治--执行报告--有组织废气污染物排放浓度监测数据
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--有组织废气污染物排放浓度监测数据")
    @Log(title = "固定源产排治--执行报告--有组织废气污染物排放浓度监测数据", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgGasEmissionVo bStatTableZxbgGasEmissionVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgGasEmissionVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgGasEmissionVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgGasEmission bStatTableZxbgGasEmission = bStatTableZxbgGasEmissionService.getById(bStatTableZxbgGasEmissionVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgGasEmission)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgGasEmissionVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgGasEmissionVo,bStatTableZxbgGasEmission);
        return toAjax(bStatTableZxbgGasEmissionService.saveOrUpdate(bStatTableZxbgGasEmission));
    }

    /**
     * 删除固定源产排治--执行报告--有组织废气污染物排放浓度监测数据
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--有组织废气污染物排放浓度监测数据")
    @RequiresPermissions("talroad:emission:remove")
    @Log(title = "固定源产排治--执行报告--有组织废气污染物排放浓度监测数据", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgGasEmissionService.removeByIds(Arrays.asList(ids)));
    }
}
