package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.domain.BStatTableZxbgGasEmission;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgGasEmissionSpeedQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgGasEmissionSpeed;
import cn.cecep.talroad.vo.BStatTableZxbgGasEmissionSpeedVo;
import cn.cecep.talroad.service.IBStatTableZxbgGasEmissionSpeedService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--有组织废气污染物排放速率监测Controller 控制层
 *
 * @author szhpt
 * @date 2023-08-01*/
@Api(tags = "固定源产排治--执行报告--有组织废气污染物排放速率监测")
@RestController
@RequestMapping("/execution/report/speed")
public class BStatTableZxbgGasEmissionSpeedController extends BasesController
{
    @Autowired
    private IBStatTableZxbgGasEmissionSpeedService bStatTableZxbgGasEmissionSpeedService;

    /**
     * 查询固定源产排治--执行报告--有组织废气污染物排放速率监测列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--有组织废气污染物排放速率监测")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgGasEmissionSpeed>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgGasEmissionSpeedService.selectBStatTableZxbgGasEmissionSpeedListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--有组织废气污染物排放速率监测列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--有组织废气污染物排放速率监测")
    @Log(title = "固定源产排治--执行报告--有组织废气污染物排放速率监测", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response,@RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgGasEmissionSpeed>> iPageAjaxResults = bStatTableZxbgGasEmissionSpeedService.selectBStatTableZxbgGasEmissionSpeedListBy(query);
        ExcelUtil<BStatTableZxbgGasEmissionSpeed> util = new ExcelUtil<BStatTableZxbgGasEmissionSpeed>(BStatTableZxbgGasEmissionSpeed.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "固定源产排治--执行报告--有组织废气污染物排放速率监测数据");
    }

    /**
     * 获取固定源产排治--执行报告--有组织废气污染物排放速率监测详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--有组织废气污染物排放速率监测详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgGasEmissionSpeed> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgGasEmissionSpeedService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--有组织废气污染物排放速率监测
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--有组织废气污染物排放速率监测")
    @Log(title = "固定源产排治--执行报告--有组织废气污染物排放速率监测", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgGasEmissionSpeed bStatTableZxbgGasEmissionSpeed) {
        return toAjax(bStatTableZxbgGasEmissionSpeedService.save(bStatTableZxbgGasEmissionSpeed));
    }
    /**
     * 修改固定源产排治--执行报告--有组织废气污染物排放速率监测
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--有组织废气污染物排放速率监测")
    @Log(title = "固定源产排治--执行报告--有组织废气污染物排放速率监测", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgGasEmissionSpeedVo bStatTableZxbgGasEmissionSpeedVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgGasEmissionSpeedVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgGasEmissionSpeedVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgGasEmissionSpeed bStatTableZxbgGasEmissionSpeed = bStatTableZxbgGasEmissionSpeedService.getById(bStatTableZxbgGasEmissionSpeedVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgGasEmissionSpeed)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgGasEmissionSpeedVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgGasEmissionSpeedVo,bStatTableZxbgGasEmissionSpeed);
        return toAjax(bStatTableZxbgGasEmissionSpeedService.saveOrUpdate(bStatTableZxbgGasEmissionSpeed));
    }

    /**
     * 删除固定源产排治--执行报告--有组织废气污染物排放速率监测
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--有组织废气污染物排放速率监测")
    @RequiresPermissions("talroad:speed:remove")
    @Log(title = "固定源产排治--执行报告--有组织废气污染物排放速率监测", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgGasEmissionSpeedService.removeByIds(Arrays.asList(ids)));
    }
}
