package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgGasEmissionStatisticsQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgGasEmissionStatistics;
import cn.cecep.talroad.vo.BStatTableZxbgGasEmissionStatisticsVo;
import cn.cecep.talroad.service.IBStatTableZxbgGasEmissionStatisticsService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计Controller 控制层
 *
 * @author szhpt
 * @date 2023-08-01*/
@Api(tags = "固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计")
@RestController
@RequestMapping("/execution/report/statistics")
public class BStatTableZxbgGasEmissionStatisticsController extends BasesController
{
    @Autowired
    private IBStatTableZxbgGasEmissionStatisticsService bStatTableZxbgGasEmissionStatisticsService;

    /**
     * 查询固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgGasEmissionStatistics>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgGasEmissionStatisticsService.selectBStatTableZxbgGasEmissionStatisticsListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计")
    @Log(title = "固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgGasEmissionStatistics>> iPageAjaxResults = bStatTableZxbgGasEmissionStatisticsService.selectBStatTableZxbgGasEmissionStatisticsListBy(query);
        ExcelUtil<BStatTableZxbgGasEmissionStatistics> util = new ExcelUtil<BStatTableZxbgGasEmissionStatistics>(BStatTableZxbgGasEmissionStatistics.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计数据");
    }

    /**
     * 获取固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgGasEmissionStatistics> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgGasEmissionStatisticsService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计")
    @Log(title = "固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgGasEmissionStatistics bStatTableZxbgGasEmissionStatistics) {
        return toAjax(bStatTableZxbgGasEmissionStatisticsService.save(bStatTableZxbgGasEmissionStatistics));
    }
    /**
     * 修改固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计")
    @Log(title = "固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgGasEmissionStatisticsVo bStatTableZxbgGasEmissionStatisticsVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgGasEmissionStatisticsVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgGasEmissionStatisticsVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgGasEmissionStatistics bStatTableZxbgGasEmissionStatistics = bStatTableZxbgGasEmissionStatisticsService.getById(bStatTableZxbgGasEmissionStatisticsVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgGasEmissionStatistics)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgGasEmissionStatisticsVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgGasEmissionStatisticsVo,bStatTableZxbgGasEmissionStatistics);
        return toAjax(bStatTableZxbgGasEmissionStatisticsService.saveOrUpdate(bStatTableZxbgGasEmissionStatistics));
    }

    /**
     * 删除固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计")
    @RequiresPermissions("talroad:statistics:remove")
    @Log(title = "固定源产排治--执行报告--无组织废气污染物排放浓度监测数据统计", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgGasEmissionStatisticsService.removeByIds(Arrays.asList(ids)));
    }
}
