package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.domain.BStatTableZxbgGasEmissionStatistics;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgGtProductionQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgGtProduction;
import cn.cecep.talroad.vo.BStatTableZxbgGtProductionVo;
import cn.cecep.talroad.service.IBStatTableZxbgGtProductionService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--钢铁-生产单元运行情况Controller 控制层
 *
 * @author szhpt
 * @date 2023-08-01*/
@Api(tags = "固定源产排治--执行报告--钢铁-生产单元运行情况")
@RestController
@RequestMapping("/execution/report/production")
public class BStatTableZxbgGtProductionController extends BasesController
{
    @Autowired
    private IBStatTableZxbgGtProductionService bStatTableZxbgGtProductionService;

    /**
     * 查询固定源产排治--执行报告--钢铁-生产单元运行情况列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--钢铁-生产单元运行情况")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgGtProduction>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgGtProductionService.selectBStatTableZxbgGtProductionListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--钢铁-生产单元运行情况列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--钢铁-生产单元运行情况")
    @Log(title = "固定源产排治--执行报告--钢铁-生产单元运行情况", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgGtProduction>> iPageAjaxResults = bStatTableZxbgGtProductionService.selectBStatTableZxbgGtProductionListBy(query);
        ExcelUtil<BStatTableZxbgGtProduction> util = new ExcelUtil<BStatTableZxbgGtProduction>(BStatTableZxbgGtProduction.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "固定源产排治--执行报告--钢铁-生产单元运行情况数据");
    }

    /**
     * 获取固定源产排治--执行报告--钢铁-生产单元运行情况详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--钢铁-生产单元运行情况详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgGtProduction> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgGtProductionService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--钢铁-生产单元运行情况
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--钢铁-生产单元运行情况")
    @Log(title = "固定源产排治--执行报告--钢铁-生产单元运行情况", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgGtProduction bStatTableZxbgGtProduction) {
        return toAjax(bStatTableZxbgGtProductionService.save(bStatTableZxbgGtProduction));
    }
    /**
     * 修改固定源产排治--执行报告--钢铁-生产单元运行情况
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--钢铁-生产单元运行情况")
    @Log(title = "固定源产排治--执行报告--钢铁-生产单元运行情况", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgGtProductionVo bStatTableZxbgGtProductionVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgGtProductionVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgGtProductionVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgGtProduction bStatTableZxbgGtProduction = bStatTableZxbgGtProductionService.getById(bStatTableZxbgGtProductionVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgGtProduction)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgGtProductionVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgGtProductionVo,bStatTableZxbgGtProduction);
        return toAjax(bStatTableZxbgGtProductionService.saveOrUpdate(bStatTableZxbgGtProduction));
    }

    /**
     * 删除固定源产排治--执行报告--钢铁-生产单元运行情况
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--钢铁-生产单元运行情况")
    @RequiresPermissions("talroad:production:remove")
    @Log(title = "固定源产排治--执行报告--钢铁-生产单元运行情况", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgGtProductionService.removeByIds(Arrays.asList(ids)));
    }
}
