package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.domain.BStatTableZxbgGasEmission;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgHdFuelAnalysisQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgHdFuelAnalysis;
import cn.cecep.talroad.vo.BStatTableZxbgHdFuelAnalysisVo;
import cn.cecep.talroad.service.IBStatTableZxbgHdFuelAnalysisService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--火电燃料分析Controller 控制层
 *
 * @author szhpt
 * @date 2023-08-01*/
@Api(tags = "固定源产排治--执行报告--火电燃料分析")
@RestController
@RequestMapping("/execution/report/hd/analysis")
public class BStatTableZxbgHdFuelAnalysisController extends BasesController
{
    @Autowired
    private IBStatTableZxbgHdFuelAnalysisService bStatTableZxbgHdFuelAnalysisService;

    /**
     * 查询固定源产排治--执行报告--火电燃料分析列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--火电燃料分析")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgHdFuelAnalysis>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgHdFuelAnalysisService.selectBStatTableZxbgHdFuelAnalysisListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--火电燃料分析列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--火电燃料分析")
    @Log(title = "固定源产排治--执行报告--火电燃料分析", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgHdFuelAnalysis>> iPageAjaxResults = bStatTableZxbgHdFuelAnalysisService.selectBStatTableZxbgHdFuelAnalysisListBy(query);
        ExcelUtil<BStatTableZxbgHdFuelAnalysis> util = new ExcelUtil<BStatTableZxbgHdFuelAnalysis>(BStatTableZxbgHdFuelAnalysis.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "固定源产排治--执行报告--火电燃料分析数据");
    }

    /**
     * 获取固定源产排治--执行报告--火电燃料分析详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--火电燃料分析详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgHdFuelAnalysis> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgHdFuelAnalysisService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--火电燃料分析
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--火电燃料分析")
    @Log(title = "固定源产排治--执行报告--火电燃料分析", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgHdFuelAnalysis bStatTableZxbgHdFuelAnalysis) {
        return toAjax(bStatTableZxbgHdFuelAnalysisService.save(bStatTableZxbgHdFuelAnalysis));
    }
    /**
     * 修改固定源产排治--执行报告--火电燃料分析
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--火电燃料分析")
    @Log(title = "固定源产排治--执行报告--火电燃料分析", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgHdFuelAnalysisVo bStatTableZxbgHdFuelAnalysisVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgHdFuelAnalysisVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgHdFuelAnalysisVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgHdFuelAnalysis bStatTableZxbgHdFuelAnalysis = bStatTableZxbgHdFuelAnalysisService.getById(bStatTableZxbgHdFuelAnalysisVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgHdFuelAnalysis)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgHdFuelAnalysisVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgHdFuelAnalysisVo,bStatTableZxbgHdFuelAnalysis);
        return toAjax(bStatTableZxbgHdFuelAnalysisService.saveOrUpdate(bStatTableZxbgHdFuelAnalysis));
    }

    /**
     * 删除固定源产排治--执行报告--火电燃料分析
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--火电燃料分析")
    @RequiresPermissions("talroad:analysis:remove")
    @Log(title = "固定源产排治--执行报告--火电燃料分析", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgHdFuelAnalysisService.removeByIds(Arrays.asList(ids)));
    }
}
