package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.domain.BStatTableZxbgGasEmission;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgHdGtBasicQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgHdGtBasic;
import cn.cecep.talroad.vo.BStatTableZxbgHdGtBasicVo;
import cn.cecep.talroad.service.IBStatTableZxbgHdGtBasicService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--火电、钢铁基本信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-08-01*/
@Api(tags = "固定源产排治--执行报告--火电、钢铁基本信息")
@RestController
@RequestMapping("/execution/report/basic")
public class BStatTableZxbgHdGtBasicController extends BasesController
{
    @Autowired
    private IBStatTableZxbgHdGtBasicService bStatTableZxbgHdGtBasicService;

    /**
     * 查询固定源产排治--执行报告--火电、钢铁基本信息列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--火电、钢铁基本信息")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgHdGtBasic>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgHdGtBasicService.selectBStatTableZxbgHdGtBasicListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--火电、钢铁基本信息列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--火电、钢铁基本信息")
    @Log(title = "固定源产排治--执行报告--火电、钢铁基本信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgHdGtBasic>> iPageAjaxResults = bStatTableZxbgHdGtBasicService.selectBStatTableZxbgHdGtBasicListBy(query);
        ExcelUtil<BStatTableZxbgHdGtBasic> util = new ExcelUtil<BStatTableZxbgHdGtBasic>(BStatTableZxbgHdGtBasic.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "固定源产排治--执行报告--火电、钢铁基本信息数据");
    }

    /**
     * 获取固定源产排治--执行报告--火电、钢铁基本信息详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--火电、钢铁基本信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgHdGtBasic> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgHdGtBasicService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--火电、钢铁基本信息
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--火电、钢铁基本信息")
    @Log(title = "固定源产排治--执行报告--火电、钢铁基本信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgHdGtBasic bStatTableZxbgHdGtBasic) {
        return toAjax(bStatTableZxbgHdGtBasicService.save(bStatTableZxbgHdGtBasic));
    }
    /**
     * 修改固定源产排治--执行报告--火电、钢铁基本信息
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--火电、钢铁基本信息")
    @Log(title = "固定源产排治--执行报告--火电、钢铁基本信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgHdGtBasicVo bStatTableZxbgHdGtBasicVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgHdGtBasicVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgHdGtBasicVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgHdGtBasic bStatTableZxbgHdGtBasic = bStatTableZxbgHdGtBasicService.getById(bStatTableZxbgHdGtBasicVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgHdGtBasic)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgHdGtBasicVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgHdGtBasicVo,bStatTableZxbgHdGtBasic);
        return toAjax(bStatTableZxbgHdGtBasicService.saveOrUpdate(bStatTableZxbgHdGtBasic));
    }

    /**
     * 删除固定源产排治--执行报告--火电、钢铁基本信息
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--火电、钢铁基本信息")
    @RequiresPermissions("talroad:basic:remove")
    @Log(title = "固定源产排治--执行报告--火电、钢铁基本信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgHdGtBasicService.removeByIds(Arrays.asList(ids)));
    }
}
