package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.domain.BStatTableZxbgGasEmissionStatistics;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgHdGtFuelQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgHdGtFuel;
import cn.cecep.talroad.vo.BStatTableZxbgHdGtFuelVo;
import cn.cecep.talroad.service.IBStatTableZxbgHdGtFuelService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--火电、钢铁燃料信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-08-01*/
@Api(tags = "固定源产排治--执行报告--火电、钢铁燃料信息")
@RestController
@RequestMapping("/execution/report/hd/fuel")
public class BStatTableZxbgHdGtFuelController extends BasesController
{
    @Autowired
    private IBStatTableZxbgHdGtFuelService bStatTableZxbgHdGtFuelService;

    /**
     * 查询固定源产排治--执行报告--火电、钢铁燃料信息列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--火电、钢铁燃料信息")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgHdGtFuel>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgHdGtFuelService.selectBStatTableZxbgHdGtFuelListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--火电、钢铁燃料信息列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--火电、钢铁燃料信息")
    @Log(title = "固定源产排治--执行报告--火电、钢铁燃料信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgHdGtFuel>> iPageAjaxResults = bStatTableZxbgHdGtFuelService.selectBStatTableZxbgHdGtFuelListBy(query);
        ExcelUtil<BStatTableZxbgHdGtFuel> util = new ExcelUtil<BStatTableZxbgHdGtFuel>(BStatTableZxbgHdGtFuel.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "固定源产排治--执行报告--火电、钢铁燃料信息数据");
    }

    /**
     * 获取固定源产排治--执行报告--火电、钢铁燃料信息详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--火电、钢铁燃料信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgHdGtFuel> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgHdGtFuelService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--火电、钢铁燃料信息
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--火电、钢铁燃料信息")
    @Log(title = "固定源产排治--执行报告--火电、钢铁燃料信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgHdGtFuel bStatTableZxbgHdGtFuel) {
        return toAjax(bStatTableZxbgHdGtFuelService.save(bStatTableZxbgHdGtFuel));
    }
    /**
     * 修改固定源产排治--执行报告--火电、钢铁燃料信息
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--火电、钢铁燃料信息")
    @Log(title = "固定源产排治--执行报告--火电、钢铁燃料信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgHdGtFuelVo bStatTableZxbgHdGtFuelVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgHdGtFuelVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgHdGtFuelVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgHdGtFuel bStatTableZxbgHdGtFuel = bStatTableZxbgHdGtFuelService.getById(bStatTableZxbgHdGtFuelVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgHdGtFuel)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgHdGtFuelVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgHdGtFuelVo,bStatTableZxbgHdGtFuel);
        return toAjax(bStatTableZxbgHdGtFuelService.saveOrUpdate(bStatTableZxbgHdGtFuel));
    }

    /**
     * 删除固定源产排治--执行报告--火电、钢铁燃料信息
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--火电、钢铁燃料信息")
    @RequiresPermissions("talroad:fuel:remove")
    @Log(title = "固定源产排治--执行报告--火电、钢铁燃料信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgHdGtFuelService.removeByIds(Arrays.asList(ids)));
    }
}
