package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.vo.query.BStatTableZxbgQuery;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgJbxxInfoQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgJbxxInfo;
import cn.cecep.talroad.vo.BStatTableZxbgJbxxInfoVo;
import cn.cecep.talroad.service.IBStatTableZxbgJbxxInfoService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--企业基本信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-07-27*/
@Api(tags = "固定源产排治--执行报告--企业基本信息")
@RestController
@RequestMapping("/execution/report/info")
public class BStatTableZxbgJbxxInfoController extends BasesController
{
    @Autowired
    private IBStatTableZxbgJbxxInfoService bStatTableZxbgJbxxInfoService;

    /**
     * 查询固定源产排治--执行报告--企业基本信息列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--企业基本信息")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgJbxxInfo>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgJbxxInfoService.selectBStatTableZxbgJbxxInfoListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--企业基本信息列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--企业基本信息",produces = "application/octet-stream")
    @Log(title = "固定源产排治--执行报告--企业基本信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgJbxxInfo>> iPageAjaxResults = bStatTableZxbgJbxxInfoService.selectBStatTableZxbgJbxxInfoListBy(query);
        ExcelUtil<BStatTableZxbgJbxxInfo> util = new ExcelUtil<BStatTableZxbgJbxxInfo>(BStatTableZxbgJbxxInfo.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "执行报告--企业基本信息数据");
    }

    /**
     * 获取固定源产排治--执行报告--企业基本信息详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--企业基本信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgJbxxInfo> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgJbxxInfoService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--企业基本信息
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--企业基本信息")
    @Log(title = "固定源产排治--执行报告--企业基本信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgJbxxInfo bStatTableZxbgJbxxInfo) {
        return toAjax(bStatTableZxbgJbxxInfoService.save(bStatTableZxbgJbxxInfo));
    }
    /**
     * 修改固定源产排治--执行报告--企业基本信息
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--企业基本信息")
    @Log(title = "固定源产排治--执行报告--企业基本信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgJbxxInfoVo bStatTableZxbgJbxxInfoVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgJbxxInfoVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgJbxxInfoVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgJbxxInfo bStatTableZxbgJbxxInfo = bStatTableZxbgJbxxInfoService.getById(bStatTableZxbgJbxxInfoVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgJbxxInfo)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgJbxxInfoVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgJbxxInfoVo,bStatTableZxbgJbxxInfo);
        return toAjax(bStatTableZxbgJbxxInfoService.saveOrUpdate(bStatTableZxbgJbxxInfo));
    }

    /**
     * 删除固定源产排治--执行报告--企业基本信息
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--企业基本信息")
    @RequiresPermissions("talroad:info:remove")
    @Log(title = "固定源产排治--执行报告--企业基本信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgJbxxInfoService.removeByIds(Arrays.asList(ids)));
    }
}
