package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.domain.BStatTableZxbgPflAir;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgPflAirOther;
import cn.cecep.talroad.vo.BStatTableZxbgPflAirOtherVo;
import cn.cecep.talroad.service.IBStatTableZxbgPflAirOtherService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--废气其他排放总计Controller
 *
 * @author szhpt
 * @date 2023-07-27*/
@Api(tags = "固定源产排治--执行报告--废气其他排放总计")
@RestController
@RequestMapping("/execution/report/other")
public class BStatTableZxbgPflAirOtherController extends BasesController
{
    @Autowired
    private IBStatTableZxbgPflAirOtherService bStatTableZxbgPflAirOtherService;

    /**
     * 查询固定源产排治--执行报告--废气其他排放总计列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--废气其他排放总计")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgPflAirOther>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgPflAirOtherService.selectBStatTableZxbgPflAirOtherListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--废气其他排放总计列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--废气其他排放总计",produces = "application/octet-stream")
    @Log(title = "固定源产排治--执行报告--废气其他排放总计", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgPflAirOther>> iPageAjaxResults = bStatTableZxbgPflAirOtherService.selectBStatTableZxbgPflAirOtherListBy(query);
        ExcelUtil<BStatTableZxbgPflAirOther> util = new ExcelUtil<BStatTableZxbgPflAirOther>(BStatTableZxbgPflAirOther.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "执行报告--废气其他排放总计数据");
    }

    /**
     * 获取固定源产排治--执行报告--废气其他排放总计详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--废气其他排放总计详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgPflAirOther> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgPflAirOtherService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--废气其他排放总计
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--废气其他排放总计")
    @Log(title = "固定源产排治--执行报告--废气其他排放总计", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgPflAirOther bStatTableZxbgPflAirOther) {
        return toAjax(bStatTableZxbgPflAirOtherService.save(bStatTableZxbgPflAirOther));
    }
    /**
     * 修改固定源产排治--执行报告--废气其他排放总计
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--废气其他排放总计")
    @Log(title = "固定源产排治--执行报告--废气其他排放总计", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgPflAirOtherVo bStatTableZxbgPflAirOtherVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgPflAirOtherVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgPflAirOtherVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgPflAirOther bStatTableZxbgPflAirOther = bStatTableZxbgPflAirOtherService.getById(bStatTableZxbgPflAirOtherVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgPflAirOther)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgPflAirOtherVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgPflAirOtherVo,bStatTableZxbgPflAirOther);
        return toAjax(bStatTableZxbgPflAirOtherService.saveOrUpdate(bStatTableZxbgPflAirOther));
    }

    /**
     * 删除固定源产排治--执行报告--废气其他排放总计
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--废气其他排放总计")
    @RequiresPermissions("talroad:other:remove")
    @Log(title = "固定源产排治--执行报告--废气其他排放总计", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgPflAirOtherService.removeByIds(Arrays.asList(ids)));
    }
}
