package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.domain.BStatTableZxbgPflAirOther;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgPflAirOverproofQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgPflAirOverproof;
import cn.cecep.talroad.vo.BStatTableZxbgPflAirOverproofVo;
import cn.cecep.talroad.service.IBStatTableZxbgPflAirOverproofService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--废气超标排放信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-07-28*/
@Api(tags = "固定源产排治--执行报告--废气超标排放信息")
@RestController
@RequestMapping("/execution/gas/overproof")
public class BStatTableZxbgPflAirOverproofController extends BasesController
{
    @Autowired
    private IBStatTableZxbgPflAirOverproofService bStatTableZxbgPflAirOverproofService;

    /**
     * 查询固定源产排治--执行报告--废气超标排放信息列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--废气超标排放信息")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgPflAirOverproof>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgPflAirOverproofService.selectBStatTableZxbgPflAirOverproofListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--废气超标排放信息列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--废气超标排放信息",produces = "application/octet-stream")
    @Log(title = "固定源产排治--执行报告--废气超标排放信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response,  @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgPflAirOverproof>> iPageAjaxResults = bStatTableZxbgPflAirOverproofService.selectBStatTableZxbgPflAirOverproofListBy(query);
        ExcelUtil<BStatTableZxbgPflAirOverproof> util = new ExcelUtil<BStatTableZxbgPflAirOverproof>(BStatTableZxbgPflAirOverproof.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "固定源产排治--执行报告--废气超标排放信息数据");
    }

    /**
     * 获取固定源产排治--执行报告--废气超标排放信息详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--废气超标排放信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgPflAirOverproof> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgPflAirOverproofService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--废气超标排放信息
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--废气超标排放信息")
    @Log(title = "固定源产排治--执行报告--废气超标排放信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgPflAirOverproof bStatTableZxbgPflAirOverproof) {
        return toAjax(bStatTableZxbgPflAirOverproofService.save(bStatTableZxbgPflAirOverproof));
    }
    /**
     * 修改固定源产排治--执行报告--废气超标排放信息
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--废气超标排放信息")
    @Log(title = "固定源产排治--执行报告--废气超标排放信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgPflAirOverproofVo bStatTableZxbgPflAirOverproofVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgPflAirOverproofVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgPflAirOverproofVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgPflAirOverproof bStatTableZxbgPflAirOverproof = bStatTableZxbgPflAirOverproofService.getById(bStatTableZxbgPflAirOverproofVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgPflAirOverproof)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgPflAirOverproofVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgPflAirOverproofVo,bStatTableZxbgPflAirOverproof);
        return toAjax(bStatTableZxbgPflAirOverproofService.saveOrUpdate(bStatTableZxbgPflAirOverproof));
    }

    /**
     * 删除固定源产排治--执行报告--废气超标排放信息
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--废气超标排放信息")
    @RequiresPermissions("talroad:overproof:remove")
    @Log(title = "固定源产排治--执行报告--废气超标排放信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgPflAirOverproofService.removeByIds(Arrays.asList(ids)));
    }
}
