package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.domain.BStatTableZxbgPflAirOther;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgPflWaterQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgPflWater;
import cn.cecep.talroad.vo.BStatTableZxbgPflWaterVo;
import cn.cecep.talroad.service.IBStatTableZxbgPflWaterService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--废水实际排放量Controller 控制层
 *
 * @author szhpt
 * @date 2023-07-28*/
@Api(tags = "固定源产排治--执行报告--废水实际排放量")
@RestController
@RequestMapping("/execution/report/water")
public class BStatTableZxbgPflWaterController extends BasesController
{
    @Autowired
    private IBStatTableZxbgPflWaterService bStatTableZxbgPflWaterService;

    /**
     * 查询固定源产排治--执行报告--废水实际排放量列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--废水实际排放量")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgPflWater>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgPflWaterService.selectBStatTableZxbgPflWaterListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--废水实际排放量列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--废水实际排放量",produces = "application/octet-stream")
    @Log(title = "固定源产排治--执行报告--废水实际排放量", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgPflWater>> iPageAjaxResults = bStatTableZxbgPflWaterService.selectBStatTableZxbgPflWaterListBy(query);
        ExcelUtil<BStatTableZxbgPflWater> util = new ExcelUtil<BStatTableZxbgPflWater>(BStatTableZxbgPflWater.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "固定源产排治--执行报告--废水实际排放量数据");
    }

    /**
     * 获取固定源产排治--执行报告--废水实际排放量详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--废水实际排放量详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgPflWater> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgPflWaterService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--废水实际排放量
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--废水实际排放量")
    @Log(title = "固定源产排治--执行报告--废水实际排放量", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgPflWater bStatTableZxbgPflWater) {
        return toAjax(bStatTableZxbgPflWaterService.save(bStatTableZxbgPflWater));
    }
    /**
     * 修改固定源产排治--执行报告--废水实际排放量
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--废水实际排放量")
    @Log(title = "固定源产排治--执行报告--废水实际排放量", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgPflWaterVo bStatTableZxbgPflWaterVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgPflWaterVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgPflWaterVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgPflWater bStatTableZxbgPflWater = bStatTableZxbgPflWaterService.getById(bStatTableZxbgPflWaterVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgPflWater)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgPflWaterVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgPflWaterVo,bStatTableZxbgPflWater);
        return toAjax(bStatTableZxbgPflWaterService.saveOrUpdate(bStatTableZxbgPflWater));
    }

    /**
     * 删除固定源产排治--执行报告--废水实际排放量
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--废水实际排放量")
    @RequiresPermissions("talroad:water:remove")
    @Log(title = "固定源产排治--执行报告--废水实际排放量", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgPflWaterService.removeByIds(Arrays.asList(ids)));
    }
}
