package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.domain.BStatTableZxbgPflAirOther;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgPflWaterVoerproofQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgPflWaterVoerproof;
import cn.cecep.talroad.vo.BStatTableZxbgPflWaterVoerproofVo;
import cn.cecep.talroad.service.IBStatTableZxbgPflWaterVoerproofService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--废水超标排放信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-07-28*/
@Api(tags = "固定源产排治--执行报告--废水超标排放信息")
@RestController
@RequestMapping("/execution/report/voerproof")
public class BStatTableZxbgPflWaterVoerproofController extends BasesController
{
    @Autowired
    private IBStatTableZxbgPflWaterVoerproofService bStatTableZxbgPflWaterVoerproofService;

    /**
     * 查询固定源产排治--执行报告--废水超标排放信息列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--废水超标排放信息")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgPflWaterVoerproof>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgPflWaterVoerproofService.selectBStatTableZxbgPflWaterVoerproofListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--废水超标排放信息列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--废水超标排放信息",produces = "application/octet-stream")
    @Log(title = "固定源产排治--执行报告--废水超标排放信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgPflWaterVoerproof>> iPageAjaxResults = bStatTableZxbgPflWaterVoerproofService.selectBStatTableZxbgPflWaterVoerproofListBy(query);
        ExcelUtil<BStatTableZxbgPflWaterVoerproof> util = new ExcelUtil<BStatTableZxbgPflWaterVoerproof>(BStatTableZxbgPflWaterVoerproof.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "固定源产排治--执行报告--废水超标排放信息数据");
    }

    /**
     * 获取固定源产排治--执行报告--废水超标排放信息详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--废水超标排放信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgPflWaterVoerproof> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgPflWaterVoerproofService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--废水超标排放信息
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--废水超标排放信息")
    @Log(title = "固定源产排治--执行报告--废水超标排放信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgPflWaterVoerproof bStatTableZxbgPflWaterVoerproof) {
        return toAjax(bStatTableZxbgPflWaterVoerproofService.save(bStatTableZxbgPflWaterVoerproof));
    }
    /**
     * 修改固定源产排治--执行报告--废水超标排放信息
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--废水超标排放信息")
    @Log(title = "固定源产排治--执行报告--废水超标排放信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgPflWaterVoerproofVo bStatTableZxbgPflWaterVoerproofVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgPflWaterVoerproofVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgPflWaterVoerproofVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgPflWaterVoerproof bStatTableZxbgPflWaterVoerproof = bStatTableZxbgPflWaterVoerproofService.getById(bStatTableZxbgPflWaterVoerproofVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgPflWaterVoerproof)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgPflWaterVoerproofVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgPflWaterVoerproofVo,bStatTableZxbgPflWaterVoerproof);
        return toAjax(bStatTableZxbgPflWaterVoerproofService.saveOrUpdate(bStatTableZxbgPflWaterVoerproof));
    }

    /**
     * 删除固定源产排治--执行报告--废水超标排放信息
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--废水超标排放信息")
    @RequiresPermissions("talroad:voerproof:remove")
    @Log(title = "固定源产排治--执行报告--废水超标排放信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgPflWaterVoerproofService.removeByIds(Arrays.asList(ids)));
    }
}
