package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.domain.BStatTableZxbgPflAirOther;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgPflallWaterQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgPflallWater;
import cn.cecep.talroad.vo.BStatTableZxbgPflallWaterVo;
import cn.cecep.talroad.service.IBStatTableZxbgPflallWaterService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--总计废水实际排放量Controller 控制层
 *
 * @author szhpt
 * @date 2023-07-28*/
@Api(tags = "固定源产排治--执行报告--总计废水实际排放量")
@RestController
@RequestMapping("/execution/report/allwater")
public class BStatTableZxbgPflallWaterController extends BasesController
{
    @Autowired
    private IBStatTableZxbgPflallWaterService bStatTableZxbgPflallWaterService;

    /**
     * 查询固定源产排治--执行报告--总计废水实际排放量列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--总计废水实际排放量")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgPflallWater>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgPflallWaterService.selectBStatTableZxbgPflallWaterListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--总计废水实际排放量列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--总计废水实际排放量",produces = "application/octet-stream")
    @Log(title = "固定源产排治--执行报告--总计废水实际排放量", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response,  @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgPflallWater>> iPageAjaxResults = bStatTableZxbgPflallWaterService.selectBStatTableZxbgPflallWaterListBy(query);
        ExcelUtil<BStatTableZxbgPflallWater> util = new ExcelUtil<BStatTableZxbgPflallWater>(BStatTableZxbgPflallWater.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "固定源产排治--执行报告--总计废水实际排放量数据");
    }

    /**
     * 获取固定源产排治--执行报告--总计废水实际排放量详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--总计废水实际排放量详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgPflallWater> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgPflallWaterService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--总计废水实际排放量
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--总计废水实际排放量")
    @Log(title = "固定源产排治--执行报告--总计废水实际排放量", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgPflallWater bStatTableZxbgPflallWater) {
        return toAjax(bStatTableZxbgPflallWaterService.save(bStatTableZxbgPflallWater));
    }
    /**
     * 修改固定源产排治--执行报告--总计废水实际排放量
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--总计废水实际排放量")
    @Log(title = "固定源产排治--执行报告--总计废水实际排放量", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgPflallWaterVo bStatTableZxbgPflallWaterVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgPflallWaterVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgPflallWaterVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgPflallWater bStatTableZxbgPflallWater = bStatTableZxbgPflallWaterService.getById(bStatTableZxbgPflallWaterVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgPflallWater)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgPflallWaterVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgPflallWaterVo,bStatTableZxbgPflallWater);
        return toAjax(bStatTableZxbgPflallWaterService.saveOrUpdate(bStatTableZxbgPflallWater));
    }

    /**
     * 删除固定源产排治--执行报告--总计废水实际排放量
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--总计废水实际排放量")
    @RequiresPermissions("talroad:water:remove")
    @Log(title = "固定源产排治--执行报告--总计废水实际排放量", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgPflallWaterService.removeByIds(Arrays.asList(ids)));
    }
}
