package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.domain.BStatTableZxbgGasEmissionStatistics;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgSnProductionQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgSnProduction;
import cn.cecep.talroad.vo.BStatTableZxbgSnProductionVo;
import cn.cecep.talroad.service.IBStatTableZxbgSnProductionService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--水泥-生产单元运行情况Controller 控制层
 *
 * @author szhpt
 * @date 2023-08-01*/
@Api(tags = "固定源产排治--执行报告--水泥-生产单元运行情况")
@RestController
@RequestMapping("/execution/report/sn/production")
public class BStatTableZxbgSnProductionController extends BasesController
{
    @Autowired
    private IBStatTableZxbgSnProductionService bStatTableZxbgSnProductionService;

    /**
     * 查询固定源产排治--执行报告--水泥-生产单元运行情况列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--水泥-生产单元运行情况")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgSnProduction>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgSnProductionService.selectBStatTableZxbgSnProductionListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--水泥-生产单元运行情况列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--水泥-生产单元运行情况")
    @Log(title = "固定源产排治--执行报告--水泥-生产单元运行情况", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgSnProduction>> iPageAjaxResults= bStatTableZxbgSnProductionService.selectBStatTableZxbgSnProductionListBy(query);
        ExcelUtil<BStatTableZxbgSnProduction> util = new ExcelUtil<BStatTableZxbgSnProduction>(BStatTableZxbgSnProduction.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "固定源产排治--执行报告--水泥-生产单元运行情况数据");
    }

    /**
     * 获取固定源产排治--执行报告--水泥-生产单元运行情况详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--水泥-生产单元运行情况详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgSnProduction> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgSnProductionService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--水泥-生产单元运行情况
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--水泥-生产单元运行情况")
    @Log(title = "固定源产排治--执行报告--水泥-生产单元运行情况", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgSnProduction bStatTableZxbgSnProduction) {
        return toAjax(bStatTableZxbgSnProductionService.save(bStatTableZxbgSnProduction));
    }
    /**
     * 修改固定源产排治--执行报告--水泥-生产单元运行情况
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--水泥-生产单元运行情况")
    @Log(title = "固定源产排治--执行报告--水泥-生产单元运行情况", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgSnProductionVo bStatTableZxbgSnProductionVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgSnProductionVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgSnProductionVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgSnProduction bStatTableZxbgSnProduction = bStatTableZxbgSnProductionService.getById(bStatTableZxbgSnProductionVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgSnProduction)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgSnProductionVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgSnProductionVo,bStatTableZxbgSnProduction);
        return toAjax(bStatTableZxbgSnProductionService.saveOrUpdate(bStatTableZxbgSnProduction));
    }

    /**
     * 删除固定源产排治--执行报告--水泥-生产单元运行情况
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--水泥-生产单元运行情况")
    @RequiresPermissions("talroad:production:remove")
    @Log(title = "固定源产排治--执行报告--水泥-生产单元运行情况", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgSnProductionService.removeByIds(Arrays.asList(ids)));
    }
}
