package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.domain.BStatTableZxbgPflAir;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgWaterEmissionQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgWaterEmission;
import cn.cecep.talroad.vo.BStatTableZxbgWaterEmissionVo;
import cn.cecep.talroad.service.IBStatTableZxbgWaterEmissionService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--废水污染物排放浓度监测数据Controller 控制层
 *
 * @author szhpt
 * @date 2023-08-01*/
@Api(tags = "固定源产排治--执行报告--废水污染物排放浓度监测数据")
@RestController
@RequestMapping("/execution/water/emission")
public class BStatTableZxbgWaterEmissionController extends BasesController
{
    @Autowired
    private IBStatTableZxbgWaterEmissionService bStatTableZxbgWaterEmissionService;

    /**
     * 查询固定源产排治--执行报告--废水污染物排放浓度监测数据列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--废水污染物排放浓度监测数据")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgWaterEmission>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgWaterEmissionService.selectBStatTableZxbgWaterEmissionListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--废水污染物排放浓度监测数据列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--废水污染物排放浓度监测数据")
    @Log(title = "固定源产排治--执行报告--废水污染物排放浓度监测数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgWaterEmission>> iPageAjaxResults = bStatTableZxbgWaterEmissionService.selectBStatTableZxbgWaterEmissionListBy(query);
        ExcelUtil<BStatTableZxbgWaterEmission> util = new ExcelUtil<BStatTableZxbgWaterEmission>(BStatTableZxbgWaterEmission.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "固定源产排治--执行报告--废水污染物排放浓度监测数据数据");
    }

    /**
     * 获取固定源产排治--执行报告--废水污染物排放浓度监测数据详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--废水污染物排放浓度监测数据详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgWaterEmission> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgWaterEmissionService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--废水污染物排放浓度监测数据
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--废水污染物排放浓度监测数据")
    @Log(title = "固定源产排治--执行报告--废水污染物排放浓度监测数据", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgWaterEmission bStatTableZxbgWaterEmission) {
        return toAjax(bStatTableZxbgWaterEmissionService.save(bStatTableZxbgWaterEmission));
    }
    /**
     * 修改固定源产排治--执行报告--废水污染物排放浓度监测数据
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--废水污染物排放浓度监测数据")
    @Log(title = "固定源产排治--执行报告--废水污染物排放浓度监测数据", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgWaterEmissionVo bStatTableZxbgWaterEmissionVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgWaterEmissionVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgWaterEmissionVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgWaterEmission bStatTableZxbgWaterEmission = bStatTableZxbgWaterEmissionService.getById(bStatTableZxbgWaterEmissionVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgWaterEmission)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgWaterEmissionVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgWaterEmissionVo,bStatTableZxbgWaterEmission);
        return toAjax(bStatTableZxbgWaterEmissionService.saveOrUpdate(bStatTableZxbgWaterEmission));
    }

    /**
     * 删除固定源产排治--执行报告--废水污染物排放浓度监测数据
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--废水污染物排放浓度监测数据")
    @RequiresPermissions("talroad:emission:remove")
    @Log(title = "固定源产排治--执行报告--废水污染物排放浓度监测数据", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgWaterEmissionService.removeByIds(Arrays.asList(ids)));
    }
}
