package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgWrzlAbnormalitiesQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgWrzlAbnormalities;
import cn.cecep.talroad.vo.BStatTableZxbgWrzlAbnormalitiesVo;
import cn.cecep.talroad.service.IBStatTableZxbgWrzlAbnormalitiesService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--污染治理设施异常情况Controller 控制层
 *
 * @author szhpt
 * @date 2023-07-31*/
@Api(tags = "固定源产排治--执行报告--污染治理设施异常情况")
@RestController
@RequestMapping("/execution/report/abnormalities")
public class BStatTableZxbgWrzlAbnormalitiesController extends BasesController
{
    @Autowired
    private IBStatTableZxbgWrzlAbnormalitiesService bStatTableZxbgWrzlAbnormalitiesService;

    /**
     * 查询固定源产排治--执行报告--污染治理设施异常情况列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--污染治理设施异常情况")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgWrzlAbnormalities>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgWrzlAbnormalitiesService.selectBStatTableZxbgWrzlAbnormalitiesListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--污染治理设施异常情况列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--污染治理设施异常情况")
    @Log(title = "固定源产排治--执行报告--污染治理设施异常情况", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgWrzlAbnormalities>> iPageAjaxResults = bStatTableZxbgWrzlAbnormalitiesService.selectBStatTableZxbgWrzlAbnormalitiesListBy(query);
        ExcelUtil<BStatTableZxbgWrzlAbnormalities> util = new ExcelUtil<BStatTableZxbgWrzlAbnormalities>(BStatTableZxbgWrzlAbnormalities.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "固定源产排治--执行报告--污染治理设施异常情况数据");
    }

    /**
     * 获取固定源产排治--执行报告--污染治理设施异常情况详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--污染治理设施异常情况详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgWrzlAbnormalities> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgWrzlAbnormalitiesService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--污染治理设施异常情况
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--污染治理设施异常情况")
    @Log(title = "固定源产排治--执行报告--污染治理设施异常情况", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgWrzlAbnormalities bStatTableZxbgWrzlAbnormalities) {
        return toAjax(bStatTableZxbgWrzlAbnormalitiesService.save(bStatTableZxbgWrzlAbnormalities));
    }
    /**
     * 修改固定源产排治--执行报告--污染治理设施异常情况
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--污染治理设施异常情况")
    @Log(title = "固定源产排治--执行报告--污染治理设施异常情况", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgWrzlAbnormalitiesVo bStatTableZxbgWrzlAbnormalitiesVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgWrzlAbnormalitiesVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgWrzlAbnormalitiesVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgWrzlAbnormalities bStatTableZxbgWrzlAbnormalities = bStatTableZxbgWrzlAbnormalitiesService.getById(bStatTableZxbgWrzlAbnormalitiesVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgWrzlAbnormalities)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgWrzlAbnormalitiesVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgWrzlAbnormalitiesVo,bStatTableZxbgWrzlAbnormalities);
        return toAjax(bStatTableZxbgWrzlAbnormalitiesService.saveOrUpdate(bStatTableZxbgWrzlAbnormalities));
    }

    /**
     * 删除固定源产排治--执行报告--污染治理设施异常情况
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--污染治理设施异常情况")
    @RequiresPermissions("talroad:abnormalities:remove")
    @Log(title = "固定源产排治--执行报告--污染治理设施异常情况", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgWrzlAbnormalitiesService.removeByIds(Arrays.asList(ids)));
    }
}
