package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.vo.query.BStatTablepPwxkWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.BStatTableZxbgWrzlTotalQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.BStatTableZxbgWrzlTotal;
import cn.cecep.talroad.vo.BStatTableZxbgWrzlTotalVo;
import cn.cecep.talroad.service.IBStatTableZxbgWrzlTotalService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固定源产排治--执行报告--污染治理设施正常情况汇总Controller 控制层
 *
 * @author szhpt
 * @date 2023-07-31*/
@Api(tags = "固定源产排治--执行报告--污染治理设施正常情况汇总")
@RestController
@RequestMapping("/execution/report/facilities")
public class BStatTableZxbgWrzlTotalController extends BasesController
{
    @Autowired
    private IBStatTableZxbgWrzlTotalService bStatTableZxbgWrzlTotalService;

    /**
     * 查询固定源产排治--执行报告--污染治理设施正常情况汇总列表
     */
    @ApiOperation(value = "查询固定源产排治--执行报告--污染治理设施正常情况汇总")
    @PostMapping("/list")
    public AjaxResults<IPage<BStatTableZxbgWrzlTotal>> list(@RequestBody BStatTablepPwxkWaterQuery query)
    {
        return bStatTableZxbgWrzlTotalService.selectBStatTableZxbgWrzlTotalListBy(query);
    }
    

    /**
     * 导出固定源产排治--执行报告--污染治理设施正常情况汇总列表
     */
    @ApiOperation(value = "导出固定源产排治--执行报告--污染治理设施正常情况汇总")
    @Log(title = "固定源产排治--执行报告--污染治理设施正常情况汇总", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BStatTablepPwxkWaterQuery query)
    {
        AjaxResults<IPage<BStatTableZxbgWrzlTotal>> iPageAjaxResults = bStatTableZxbgWrzlTotalService.selectBStatTableZxbgWrzlTotalListBy(query);
        ExcelUtil<BStatTableZxbgWrzlTotal> util = new ExcelUtil<BStatTableZxbgWrzlTotal>(BStatTableZxbgWrzlTotal.class);
        util.setBol(query.getBol());
        util.exportExcel(response, iPageAjaxResults.getData().getRecords(), "固定源产排治--执行报告--污染治理设施正常情况汇总数据");
    }

    /**
     * 获取固定源产排治--执行报告--污染治理设施正常情况汇总详细信息
     */
    @ApiOperation(value = "获取固定源产排治--执行报告--污染治理设施正常情况汇总详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<BStatTableZxbgWrzlTotal> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(bStatTableZxbgWrzlTotalService.getById(id));
    }
    /**
     * 新增固定源产排治--执行报告--污染治理设施正常情况汇总
     */
    @ApiOperation(value = "新增固定源产排治--执行报告--污染治理设施正常情况汇总")
    @Log(title = "固定源产排治--执行报告--污染治理设施正常情况汇总", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BStatTableZxbgWrzlTotal bStatTableZxbgWrzlTotal) {
        return toAjax(bStatTableZxbgWrzlTotalService.save(bStatTableZxbgWrzlTotal));
    }
    /**
     * 修改固定源产排治--执行报告--污染治理设施正常情况汇总
     */
    @ApiOperation(value = "修改固定源产排治--执行报告--污染治理设施正常情况汇总")
    @Log(title = "固定源产排治--执行报告--污染治理设施正常情况汇总", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BStatTableZxbgWrzlTotalVo bStatTableZxbgWrzlTotalVo) {
        if(ObjectUtil.isEmpty(bStatTableZxbgWrzlTotalVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(bStatTableZxbgWrzlTotalVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        BStatTableZxbgWrzlTotal bStatTableZxbgWrzlTotal = bStatTableZxbgWrzlTotalService.getById(bStatTableZxbgWrzlTotalVo.getId());
        if(ObjectUtil.isEmpty(bStatTableZxbgWrzlTotal)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", bStatTableZxbgWrzlTotalVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(bStatTableZxbgWrzlTotalVo,bStatTableZxbgWrzlTotal);
        return toAjax(bStatTableZxbgWrzlTotalService.saveOrUpdate(bStatTableZxbgWrzlTotal));
    }

    /**
     * 删除固定源产排治--执行报告--污染治理设施正常情况汇总
     */
    @ApiOperation(value = "删除固定源产排治--执行报告--污染治理设施正常情况汇总")
    @RequiresPermissions("talroad:total:remove")
    @Log(title = "固定源产排治--执行报告--污染治理设施正常情况汇总", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(bStatTableZxbgWrzlTotalService.removeByIds(Arrays.asList(ids)));
    }
}
