package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BasDevToElec;
import cn.cecep.talroad.service.IBasDevToElecService;
import cn.cecep.talroad.vo.BasDevToElecVo;
import cn.cecep.talroad.vo.query.BasDevToElecQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 有组织-分表计电设备对应Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控--有组织-分表计电设备对应")
@RestController
@RequestMapping("/basDevToElec")
public class BasDevToElecController extends BasesController
{
    @Autowired
    private IBasDevToElecService basDevToElecService;

    /**
     * 查询有组织-分表计电设备对应列表
     */
    @ApiOperation(value = "查询有组织-分表计电设备对应")
    @GetMapping("/list")
    public AjaxResults<IPage<BasDevToElec>> list(BasDevToElecQuery basDevToElec)
    {
        Page<BasDevToElec> page = new Page<>(basDevToElec.getPageNum(), basDevToElec.getPageSize());
        IPage<BasDevToElec> list = basDevToElecService.page(page, BasDevToElecQuery.createLambdaQueryWrapper(basDevToElec));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出有组织-分表计电设备对应列表
     */
    @ApiOperation(value = "导出有组织-分表计电设备对应")
    @Log(title = "有组织-分表计电设备对应", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BasDevToElec basDevToElec)
    {
        List<BasDevToElec> list = basDevToElecService.selectBasDevToElecList(basDevToElec);
        ExcelUtil<BasDevToElec> util = new ExcelUtil<BasDevToElec>(BasDevToElec.class);
        util.exportExcel(response, list, "有组织-分表计电设备对应数据");
    }

    /**
     * 获取有组织-分表计电设备对应详细信息
     */
    @ApiOperation(value = "获取有组织-分表计电设备对应详细信息")
    @PostMapping(value = "/{deviceId}")
    public AjaxResults<BasDevToElec> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(basDevToElecService.getById(id));
    }
    /**
     * 新增有组织-分表计电设备对应
     */
    @ApiOperation(value = "新增有组织-分表计电设备对应")
    @Log(title = "有组织-分表计电设备对应", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BasDevToElec basDevToElec) {
        return toAjax(basDevToElecService.save(basDevToElec));
    }
    /**
     * 修改有组织-分表计电设备对应
     */
    @ApiOperation(value = "修改有组织-分表计电设备对应")
    @Log(title = "有组织-分表计电设备对应", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BasDevToElecVo basDevToElecVo) {
        if(ObjectUtil.isEmpty(basDevToElecVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(basDevToElecVo.getDeviceId())){
            throw new InvalidParameterException("参数id为空");
        }
        BasDevToElec basDevToElec = basDevToElecService.getById(basDevToElecVo.getDeviceId());
        if(ObjectUtil.isEmpty(basDevToElec)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", basDevToElecVo.getDeviceId()));
        }
        //赋值
        BeanUtil.copyProperties(basDevToElecVo,basDevToElec);
        return toAjax(basDevToElecService.saveOrUpdate(basDevToElec));
    }

    /**
     * 删除有组织-分表计电设备对应
     */
    @ApiOperation(value = "删除有组织-分表计电设备对应")
    @RequiresPermissions("talroad:elec:remove")
    @Log(title = "有组织-分表计电设备对应", businessType = BusinessType.DELETE)
    @PostMapping("/{deviceIds}")
    public AjaxResults<Boolean> remove(@PathVariable String[] deviceIds) {
        return toAjax(basDevToElecService.removeByIds(Arrays.asList(deviceIds)));
    }
}
