package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.BasElecDevice;
import cn.cecep.talroad.service.IBasElecDeviceService;
import cn.cecep.talroad.vo.BasElecDeviceVo;
import cn.cecep.talroad.vo.query.BasElecDeviceQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 分表计电设备信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控--分表计电设备信息")
@RestController
@RequestMapping("/basElecDevice")
public class BasElecDeviceController extends BasesController
{
    @Autowired
    private IBasElecDeviceService basElecDeviceService;

    /**
     * 查询分表计电设备信息列表
     */
    @ApiOperation(value = "查询分表计电设备信息")
    @GetMapping("/list")
    public AjaxResults<IPage<BasElecDevice>> list(BasElecDeviceQuery basElecDevice)
    {
        Page<BasElecDevice> page = new Page<>(basElecDevice.getPageNum(), basElecDevice.getPageSize());
        IPage<BasElecDevice> list = basElecDeviceService.page(page, BasElecDeviceQuery.createLambdaQueryWrapper(basElecDevice));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出分表计电设备信息列表
     */
    @ApiOperation(value = "导出分表计电设备信息")
    @Log(title = "分表计电设备信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BasElecDevice basElecDevice)
    {
        List<BasElecDevice> list = basElecDeviceService.selectBasElecDeviceList(basElecDevice);
        ExcelUtil<BasElecDevice> util = new ExcelUtil<BasElecDevice>(BasElecDevice.class);
        util.exportExcel(response, list, "分表计电设备信息数据");
    }

    /**
     * 获取分表计电设备信息详细信息
     */
    @ApiOperation(value = "获取分表计电设备信息详细信息")
    @PostMapping(value = "/{elecDeviceId}")
    public AjaxResults<BasElecDevice> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(basElecDeviceService.getById(id));
    }
    /**
     * 新增分表计电设备信息
     */
    @ApiOperation(value = "新增分表计电设备信息")
    @Log(title = "分表计电设备信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody BasElecDevice basElecDevice) {
        return toAjax(basElecDeviceService.save(basElecDevice));
    }
    /**
     * 修改分表计电设备信息
     */
    @ApiOperation(value = "修改分表计电设备信息")
    @Log(title = "分表计电设备信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody BasElecDeviceVo basElecDeviceVo) {
        if(ObjectUtil.isEmpty(basElecDeviceVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(basElecDeviceVo.getElecDeviceId())){
            throw new InvalidParameterException("参数id为空");
        }
        BasElecDevice basElecDevice = basElecDeviceService.getById(basElecDeviceVo.getElecDeviceId());
        if(ObjectUtil.isEmpty(basElecDevice)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", basElecDeviceVo.getElecDeviceId()));
        }
        //赋值
        BeanUtil.copyProperties(basElecDeviceVo,basElecDevice);
        return toAjax(basElecDeviceService.saveOrUpdate(basElecDevice));
    }

    /**
     * 删除分表计电设备信息
     */
    @ApiOperation(value = "删除分表计电设备信息")
    @RequiresPermissions("talroad:device:remove")
    @Log(title = "分表计电设备信息", businessType = BusinessType.DELETE)
    @PostMapping("/{elecDeviceIds}")
    public AjaxResults<Boolean> remove(@PathVariable String[] elecDeviceIds) {
        return toAjax(basElecDeviceService.removeByIds(Arrays.asList(elecDeviceIds)));
    }
}
