package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.MonDevBlOther;
import cn.cecep.talroad.service.IMonDevBlOtherService;
import cn.cecep.talroad.vo.MonDevBlOtherVo;
import cn.cecep.talroad.vo.query.MonDevBlOtherQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 设备数据-玻璃-其它时间 保存设备的小时以上的数据Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控--工况数据--设备数据-玻璃-其它时间 保存设备的小时以上的数据")
@RestController
@RequestMapping("/monDevBlOther")
public class MonDevBlOtherController extends BasesController
{
    @Autowired
    private IMonDevBlOtherService monDevBlOtherService;

    /**
     * 查询设备数据-玻璃-其它时间 保存设备的小时以上的数据列表
     */
    @ApiOperation(value = "查询设备数据-玻璃-其它时间 保存设备的小时以上的数据")
    @GetMapping("/list")
    public AjaxResults<IPage<MonDevBlOther>> list(MonDevBlOtherQuery monDevBlOther)
    {
        Page<MonDevBlOther> page = new Page<>(monDevBlOther.getPageNum(), monDevBlOther.getPageSize());
        IPage<MonDevBlOther> list = monDevBlOtherService.page(page, MonDevBlOtherQuery.createLambdaQueryWrapper(monDevBlOther));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出设备数据-玻璃-其它时间 保存设备的小时以上的数据列表
     */
    @ApiOperation(value = "导出设备数据-玻璃-其它时间 保存设备的小时以上的数据")
    @Log(title = "设备数据-玻璃-其它时间 保存设备的小时以上的数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, MonDevBlOther monDevBlOther)
    {
        List<MonDevBlOther> list = monDevBlOtherService.selectMonDevBlOtherList(monDevBlOther);
        ExcelUtil<MonDevBlOther> util = new ExcelUtil<MonDevBlOther>(MonDevBlOther.class);
        util.exportExcel(response, list, "设备数据-玻璃-其它时间 保存设备的小时以上的数据数据");
    }

    /**
     * 获取设备数据-玻璃-其它时间 保存设备的小时以上的数据详细信息
     */
    @ApiOperation(value = "获取设备数据-玻璃-其它时间 保存设备的小时以上的数据详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<MonDevBlOther> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(monDevBlOtherService.getById(id));
    }
    /**
     * 新增设备数据-玻璃-其它时间 保存设备的小时以上的数据
     */
    @ApiOperation(value = "新增设备数据-玻璃-其它时间 保存设备的小时以上的数据")
    @Log(title = "设备数据-玻璃-其它时间 保存设备的小时以上的数据", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody MonDevBlOther monDevBlOther) {
        return toAjax(monDevBlOtherService.save(monDevBlOther));
    }
    /**
     * 修改设备数据-玻璃-其它时间 保存设备的小时以上的数据
     */
    @ApiOperation(value = "修改设备数据-玻璃-其它时间 保存设备的小时以上的数据")
    @Log(title = "设备数据-玻璃-其它时间 保存设备的小时以上的数据", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody MonDevBlOtherVo monDevBlOtherVo) {
        if(ObjectUtil.isEmpty(monDevBlOtherVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(monDevBlOtherVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        MonDevBlOther monDevBlOther = monDevBlOtherService.getById(monDevBlOtherVo.getDataId());
        if(ObjectUtil.isEmpty(monDevBlOther)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", monDevBlOtherVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(monDevBlOtherVo,monDevBlOther);
        return toAjax(monDevBlOtherService.saveOrUpdate(monDevBlOther));
    }

    /**
     * 删除设备数据-玻璃-其它时间 保存设备的小时以上的数据
     */
    @ApiOperation(value = "删除设备数据-玻璃-其它时间 保存设备的小时以上的数据")
    @RequiresPermissions("talroad:other:remove")
    @Log(title = "设备数据-玻璃-其它时间 保存设备的小时以上的数据", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] dataIds) {
        return toAjax(monDevBlOtherService.removeByIds(Arrays.asList(dataIds)));
    }
}
