package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.MonDevDlOther;
import cn.cecep.talroad.service.IMonDevDlOtherService;
import cn.cecep.talroad.vo.MonDevDlOtherVo;
import cn.cecep.talroad.vo.query.MonDevDlOtherQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 设备数据-电力-其它时间 保存设备的小时以上的数据Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控--工况数据--设备数据-电力-其它时间 保存设备的小时以上的数据")
@RestController
@RequestMapping("/monDevDlOther")
public class MonDevDlOtherController extends BasesController
{
    @Autowired
    private IMonDevDlOtherService monDevDlOtherService;

    /**
     * 查询设备数据-电力-其它时间 保存设备的小时以上的数据列表
     */
    @ApiOperation(value = "查询设备数据-电力-其它时间 保存设备的小时以上的数据")
    @GetMapping("/list")
    public AjaxResults<IPage<MonDevDlOther>> list(MonDevDlOtherQuery monDevDlOther)
    {
        Page<MonDevDlOther> page = new Page<>(monDevDlOther.getPageNum(), monDevDlOther.getPageSize());
        IPage<MonDevDlOther> list = monDevDlOtherService.page(page, MonDevDlOtherQuery.createLambdaQueryWrapper(monDevDlOther));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出设备数据-电力-其它时间 保存设备的小时以上的数据列表
     */
    @ApiOperation(value = "导出设备数据-电力-其它时间 保存设备的小时以上的数据")
    @Log(title = "设备数据-电力-其它时间 保存设备的小时以上的数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, MonDevDlOther monDevDlOther)
    {
        List<MonDevDlOther> list = monDevDlOtherService.selectMonDevDlOtherList(monDevDlOther);
        ExcelUtil<MonDevDlOther> util = new ExcelUtil<MonDevDlOther>(MonDevDlOther.class);
        util.exportExcel(response, list, "设备数据-电力-其它时间 保存设备的小时以上的数据数据");
    }

    /**
     * 获取设备数据-电力-其它时间 保存设备的小时以上的数据详细信息
     */
    @ApiOperation(value = "获取设备数据-电力-其它时间 保存设备的小时以上的数据详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<MonDevDlOther> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(monDevDlOtherService.getById(id));
    }
    /**
     * 新增设备数据-电力-其它时间 保存设备的小时以上的数据
     */
    @ApiOperation(value = "新增设备数据-电力-其它时间 保存设备的小时以上的数据")
    @Log(title = "设备数据-电力-其它时间 保存设备的小时以上的数据", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody MonDevDlOther monDevDlOther) {
        return toAjax(monDevDlOtherService.save(monDevDlOther));
    }
    /**
     * 修改设备数据-电力-其它时间 保存设备的小时以上的数据
     */
    @ApiOperation(value = "修改设备数据-电力-其它时间 保存设备的小时以上的数据")
    @Log(title = "设备数据-电力-其它时间 保存设备的小时以上的数据", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody MonDevDlOtherVo monDevDlOtherVo) {
        if(ObjectUtil.isEmpty(monDevDlOtherVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(monDevDlOtherVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        MonDevDlOther monDevDlOther = monDevDlOtherService.getById(monDevDlOtherVo.getDataId());
        if(ObjectUtil.isEmpty(monDevDlOther)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", monDevDlOtherVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(monDevDlOtherVo,monDevDlOther);
        return toAjax(monDevDlOtherService.saveOrUpdate(monDevDlOther));
    }

    /**
     * 删除设备数据-电力-其它时间 保存设备的小时以上的数据
     */
    @ApiOperation(value = "删除设备数据-电力-其它时间 保存设备的小时以上的数据")
    @RequiresPermissions("talroad:other:remove")
    @Log(title = "设备数据-电力-其它时间 保存设备的小时以上的数据", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] dataIds) {
        return toAjax(monDevDlOtherService.removeByIds(Arrays.asList(dataIds)));
    }
}
