package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.MonDevJhReal;
import cn.cecep.talroad.service.IMonDevJhRealService;
import cn.cecep.talroad.vo.MonDevJhRealVo;
import cn.cecep.talroad.vo.query.MonDevJhRealQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 设备数据-焦化-实时 保存设备的实时数据Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控--工况数据--设备数据-焦化-实时 保存设备的实时数据")
@RestController
@RequestMapping("/monDevJhReal")
public class MonDevJhRealController extends BasesController
{
    @Autowired
    private IMonDevJhRealService monDevJhRealService;

    /**
     * 查询设备数据-焦化-实时 保存设备的实时数据列表
     */
    @ApiOperation(value = "查询设备数据-焦化-实时 保存设备的实时数据")
    @GetMapping("/list")
    public AjaxResults<IPage<MonDevJhReal>> list(MonDevJhRealQuery monDevJhReal)
    {
        Page<MonDevJhReal> page = new Page<>(monDevJhReal.getPageNum(), monDevJhReal.getPageSize());
        IPage<MonDevJhReal> list = monDevJhRealService.page(page, MonDevJhRealQuery.createLambdaQueryWrapper(monDevJhReal));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出设备数据-焦化-实时 保存设备的实时数据列表
     */
    @ApiOperation(value = "导出设备数据-焦化-实时 保存设备的实时数据")
    @Log(title = "设备数据-焦化-实时 保存设备的实时数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, MonDevJhReal monDevJhReal)
    {
        List<MonDevJhReal> list = monDevJhRealService.selectMonDevJhRealList(monDevJhReal);
        ExcelUtil<MonDevJhReal> util = new ExcelUtil<MonDevJhReal>(MonDevJhReal.class);
        util.exportExcel(response, list, "设备数据-焦化-实时 保存设备的实时数据数据");
    }

    /**
     * 获取设备数据-焦化-实时 保存设备的实时数据详细信息
     */
    @ApiOperation(value = "获取设备数据-焦化-实时 保存设备的实时数据详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<MonDevJhReal> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(monDevJhRealService.getById(id));
    }
    /**
     * 新增设备数据-焦化-实时 保存设备的实时数据
     */
    @ApiOperation(value = "新增设备数据-焦化-实时 保存设备的实时数据")
    @Log(title = "设备数据-焦化-实时 保存设备的实时数据", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody MonDevJhReal monDevJhReal) {
        return toAjax(monDevJhRealService.save(monDevJhReal));
    }
    /**
     * 修改设备数据-焦化-实时 保存设备的实时数据
     */
    @ApiOperation(value = "修改设备数据-焦化-实时 保存设备的实时数据")
    @Log(title = "设备数据-焦化-实时 保存设备的实时数据", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody MonDevJhRealVo monDevJhRealVo) {
        if(ObjectUtil.isEmpty(monDevJhRealVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(monDevJhRealVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        MonDevJhReal monDevJhReal = monDevJhRealService.getById(monDevJhRealVo.getDataId());
        if(ObjectUtil.isEmpty(monDevJhReal)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", monDevJhRealVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(monDevJhRealVo,monDevJhReal);
        return toAjax(monDevJhRealService.saveOrUpdate(monDevJhReal));
    }

    /**
     * 删除设备数据-焦化-实时 保存设备的实时数据
     */
    @ApiOperation(value = "删除设备数据-焦化-实时 保存设备的实时数据")
    @RequiresPermissions("talroad:real:remove")
    @Log(title = "设备数据-焦化-实时 保存设备的实时数据", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] dataIds) {
        return toAjax(monDevJhRealService.removeByIds(Arrays.asList(dataIds)));
    }
}
