package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.MonDevOther;
import cn.cecep.talroad.service.IMonDevOtherService;
import cn.cecep.talroad.vo.MonDevOtherVo;
import cn.cecep.talroad.vo.query.MonDevOtherQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 设备数据(有组织)-其它时间 保存设备的小时以上的数据Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控--工况数据--设备数据(有组织)-其它时间 保存设备的小时以上的数据")
@RestController
@RequestMapping("/monDevOther")
public class MonDevOtherController extends BasesController
{
    @Autowired
    private IMonDevOtherService monDevOtherService;

    /**
     * 查询设备数据(有组织)-其它时间 保存设备的小时以上的数据列表
     */
    @ApiOperation(value = "查询设备数据(有组织)-其它时间 保存设备的小时以上的数据")
    @GetMapping("/list")
    public AjaxResults<IPage<MonDevOther>> list(MonDevOtherQuery monDevOther)
    {
        Page<MonDevOther> page = new Page<>(monDevOther.getPageNum(), monDevOther.getPageSize());
        IPage<MonDevOther> list = monDevOtherService.page(page, MonDevOtherQuery.createLambdaQueryWrapper(monDevOther));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出设备数据(有组织)-其它时间 保存设备的小时以上的数据列表
     */
    @ApiOperation(value = "导出设备数据(有组织)-其它时间 保存设备的小时以上的数据")
    @Log(title = "设备数据(有组织)-其它时间 保存设备的小时以上的数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, MonDevOther monDevOther)
    {
        List<MonDevOther> list = monDevOtherService.selectMonDevOtherList(monDevOther);
        ExcelUtil<MonDevOther> util = new ExcelUtil<MonDevOther>(MonDevOther.class);
        util.exportExcel(response, list, "设备数据(有组织)-其它时间 保存设备的小时以上的数据数据");
    }

    /**
     * 获取设备数据(有组织)-其它时间 保存设备的小时以上的数据详细信息
     */
    @ApiOperation(value = "获取设备数据(有组织)-其它时间 保存设备的小时以上的数据详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<MonDevOther> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(monDevOtherService.getById(id));
    }
    /**
     * 新增设备数据(有组织)-其它时间 保存设备的小时以上的数据
     */
    @ApiOperation(value = "新增设备数据(有组织)-其它时间 保存设备的小时以上的数据")
    @Log(title = "设备数据(有组织)-其它时间 保存设备的小时以上的数据", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody MonDevOther monDevOther) {
        return toAjax(monDevOtherService.save(monDevOther));
    }
    /**
     * 修改设备数据(有组织)-其它时间 保存设备的小时以上的数据
     */
    @ApiOperation(value = "修改设备数据(有组织)-其它时间 保存设备的小时以上的数据")
    @Log(title = "设备数据(有组织)-其它时间 保存设备的小时以上的数据", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody MonDevOtherVo monDevOtherVo) {
        if(ObjectUtil.isEmpty(monDevOtherVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(monDevOtherVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        MonDevOther monDevOther = monDevOtherService.getById(monDevOtherVo.getDataId());
        if(ObjectUtil.isEmpty(monDevOther)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", monDevOtherVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(monDevOtherVo,monDevOther);
        return toAjax(monDevOtherService.saveOrUpdate(monDevOther));
    }

    /**
     * 删除设备数据(有组织)-其它时间 保存设备的小时以上的数据
     */
    @ApiOperation(value = "删除设备数据(有组织)-其它时间 保存设备的小时以上的数据")
    @RequiresPermissions("talroad:other:remove")
    @Log(title = "设备数据(有组织)-其它时间 保存设备的小时以上的数据", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] dataIds) {
        return toAjax(monDevOtherService.removeByIds(Arrays.asList(dataIds)));
    }
}
