package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.MonDevReal;
import cn.cecep.talroad.service.IMonDevRealService;
import cn.cecep.talroad.vo.MonDevRealVo;
import cn.cecep.talroad.vo.query.MonDevRealQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控--工况数据--设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数")
@RestController
@RequestMapping("/monDevReal")
public class MonDevRealController extends BasesController
{
    @Autowired
    private IMonDevRealService monDevRealService;

    /**
     * 查询设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数列表
     */
    @ApiOperation(value = "查询设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数")
    @GetMapping("/list")
    public AjaxResults<IPage<MonDevReal>> list(MonDevRealQuery monDevReal)
    {
        Page<MonDevReal> page = new Page<>(monDevReal.getPageNum(), monDevReal.getPageSize());
        IPage<MonDevReal> list = monDevRealService.page(page, MonDevRealQuery.createLambdaQueryWrapper(monDevReal));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数列表
     */
    @ApiOperation(value = "导出设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数")
    @Log(title = "设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, MonDevReal monDevReal)
    {
        List<MonDevReal> list = monDevRealService.selectMonDevRealList(monDevReal);
        ExcelUtil<MonDevReal> util = new ExcelUtil<MonDevReal>(MonDevReal.class);
        util.exportExcel(response, list, "设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数数据");
    }

    /**
     * 获取设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数详细信息
     */
    @ApiOperation(value = "获取设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<MonDevReal> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(monDevRealService.getById(id));
    }
    /**
     * 新增设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数
     */
    @ApiOperation(value = "新增设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数")
    @Log(title = "设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody MonDevReal monDevReal) {
        return toAjax(monDevRealService.save(monDevReal));
    }
    /**
     * 修改设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数
     */
    @ApiOperation(value = "修改设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数")
    @Log(title = "设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody MonDevRealVo monDevRealVo) {
        if(ObjectUtil.isEmpty(monDevRealVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(monDevRealVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        MonDevReal monDevReal = monDevRealService.getById(monDevRealVo.getDataId());
        if(ObjectUtil.isEmpty(monDevReal)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", monDevRealVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(monDevRealVo,monDevReal);
        return toAjax(monDevRealService.saveOrUpdate(monDevReal));
    }

    /**
     * 删除设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数
     */
    @ApiOperation(value = "删除设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数")
    @RequiresPermissions("talroad:real:remove")
    @Log(title = "设备数据(有组织)-实时 保存有组织设备的实时数据。p1~pn字段对应各设备的生产过程类参数", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] dataIds) {
        return toAjax(monDevRealService.removeByIds(Arrays.asList(dataIds)));
    }
}
