package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.MonDevTcHour;
import cn.cecep.talroad.service.IMonDevTcHourService;
import cn.cecep.talroad.vo.MonDevTcHourVo;
import cn.cecep.talroad.vo.query.MonDevTcHourQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 设备数据-陶瓷-小时 保存设备的小时数据Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控--工况数据--设备数据-陶瓷-小时 保存设备的小时数据")
@RestController
@RequestMapping("/monDevTcHour")
public class MonDevTcHourController extends BasesController
{
    @Autowired
    private IMonDevTcHourService monDevTcHourService;

    /**
     * 查询设备数据-陶瓷-小时 保存设备的小时数据列表
     */
    @ApiOperation(value = "查询设备数据-陶瓷-小时 保存设备的小时数据")
    @GetMapping("/list")
    public AjaxResults<IPage<MonDevTcHour>> list(MonDevTcHourQuery monDevTcHour)
    {
        Page<MonDevTcHour> page = new Page<>(monDevTcHour.getPageNum(), monDevTcHour.getPageSize());
        IPage<MonDevTcHour> list = monDevTcHourService.page(page, MonDevTcHourQuery.createLambdaQueryWrapper(monDevTcHour));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出设备数据-陶瓷-小时 保存设备的小时数据列表
     */
    @ApiOperation(value = "导出设备数据-陶瓷-小时 保存设备的小时数据")
    @Log(title = "设备数据-陶瓷-小时 保存设备的小时数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, MonDevTcHour monDevTcHour)
    {
        List<MonDevTcHour> list = monDevTcHourService.selectMonDevTcHourList(monDevTcHour);
        ExcelUtil<MonDevTcHour> util = new ExcelUtil<MonDevTcHour>(MonDevTcHour.class);
        util.exportExcel(response, list, "设备数据-陶瓷-小时 保存设备的小时数据数据");
    }

    /**
     * 获取设备数据-陶瓷-小时 保存设备的小时数据详细信息
     */
    @ApiOperation(value = "获取设备数据-陶瓷-小时 保存设备的小时数据详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<MonDevTcHour> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(monDevTcHourService.getById(id));
    }
    /**
     * 新增设备数据-陶瓷-小时 保存设备的小时数据
     */
    @ApiOperation(value = "新增设备数据-陶瓷-小时 保存设备的小时数据")
    @Log(title = "设备数据-陶瓷-小时 保存设备的小时数据", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody MonDevTcHour monDevTcHour) {
        return toAjax(monDevTcHourService.save(monDevTcHour));
    }
    /**
     * 修改设备数据-陶瓷-小时 保存设备的小时数据
     */
    @ApiOperation(value = "修改设备数据-陶瓷-小时 保存设备的小时数据")
    @Log(title = "设备数据-陶瓷-小时 保存设备的小时数据", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody MonDevTcHourVo monDevTcHourVo) {
        if(ObjectUtil.isEmpty(monDevTcHourVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(monDevTcHourVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        MonDevTcHour monDevTcHour = monDevTcHourService.getById(monDevTcHourVo.getDataId());
        if(ObjectUtil.isEmpty(monDevTcHour)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", monDevTcHourVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(monDevTcHourVo,monDevTcHour);
        return toAjax(monDevTcHourService.saveOrUpdate(monDevTcHour));
    }

    /**
     * 删除设备数据-陶瓷-小时 保存设备的小时数据
     */
    @ApiOperation(value = "删除设备数据-陶瓷-小时 保存设备的小时数据")
    @RequiresPermissions("talroad:hour:remove")
    @Log(title = "设备数据-陶瓷-小时 保存设备的小时数据", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] dataIds) {
        return toAjax(monDevTcHourService.removeByIds(Arrays.asList(dataIds)));
    }
}
