package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.MonDevTcOther;
import cn.cecep.talroad.service.IMonDevTcOtherService;
import cn.cecep.talroad.vo.MonDevTcOtherVo;
import cn.cecep.talroad.vo.query.MonDevTcOtherQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 设备数据-陶瓷-其它时间 保存设备的小时以上的数据Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控--工况数据--设备数据-陶瓷-其它时间 保存设备的小时以上的数据")
@RestController
@RequestMapping("/monDevTcOther")
public class MonDevTcOtherController extends BasesController
{
    @Autowired
    private IMonDevTcOtherService monDevTcOtherService;

    /**
     * 查询设备数据-陶瓷-其它时间 保存设备的小时以上的数据列表
     */
    @ApiOperation(value = "查询设备数据-陶瓷-其它时间 保存设备的小时以上的数据")
    @GetMapping("/list")
    public AjaxResults<IPage<MonDevTcOther>> list(MonDevTcOtherQuery monDevTcOther)
    {
        Page<MonDevTcOther> page = new Page<>(monDevTcOther.getPageNum(), monDevTcOther.getPageSize());
        IPage<MonDevTcOther> list = monDevTcOtherService.page(page, MonDevTcOtherQuery.createLambdaQueryWrapper(monDevTcOther));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出设备数据-陶瓷-其它时间 保存设备的小时以上的数据列表
     */
    @ApiOperation(value = "导出设备数据-陶瓷-其它时间 保存设备的小时以上的数据")
    @Log(title = "设备数据-陶瓷-其它时间 保存设备的小时以上的数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, MonDevTcOther monDevTcOther)
    {
        List<MonDevTcOther> list = monDevTcOtherService.selectMonDevTcOtherList(monDevTcOther);
        ExcelUtil<MonDevTcOther> util = new ExcelUtil<MonDevTcOther>(MonDevTcOther.class);
        util.exportExcel(response, list, "设备数据-陶瓷-其它时间 保存设备的小时以上的数据数据");
    }

    /**
     * 获取设备数据-陶瓷-其它时间 保存设备的小时以上的数据详细信息
     */
    @ApiOperation(value = "获取设备数据-陶瓷-其它时间 保存设备的小时以上的数据详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<MonDevTcOther> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(monDevTcOtherService.getById(id));
    }
    /**
     * 新增设备数据-陶瓷-其它时间 保存设备的小时以上的数据
     */
    @ApiOperation(value = "新增设备数据-陶瓷-其它时间 保存设备的小时以上的数据")
    @Log(title = "设备数据-陶瓷-其它时间 保存设备的小时以上的数据", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody MonDevTcOther monDevTcOther) {
        return toAjax(monDevTcOtherService.save(monDevTcOther));
    }
    /**
     * 修改设备数据-陶瓷-其它时间 保存设备的小时以上的数据
     */
    @ApiOperation(value = "修改设备数据-陶瓷-其它时间 保存设备的小时以上的数据")
    @Log(title = "设备数据-陶瓷-其它时间 保存设备的小时以上的数据", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody MonDevTcOtherVo monDevTcOtherVo) {
        if(ObjectUtil.isEmpty(monDevTcOtherVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(monDevTcOtherVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        MonDevTcOther monDevTcOther = monDevTcOtherService.getById(monDevTcOtherVo.getDataId());
        if(ObjectUtil.isEmpty(monDevTcOther)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", monDevTcOtherVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(monDevTcOtherVo,monDevTcOther);
        return toAjax(monDevTcOtherService.saveOrUpdate(monDevTcOther));
    }

    /**
     * 删除设备数据-陶瓷-其它时间 保存设备的小时以上的数据
     */
    @ApiOperation(value = "删除设备数据-陶瓷-其它时间 保存设备的小时以上的数据")
    @RequiresPermissions("talroad:other:remove")
    @Log(title = "设备数据-陶瓷-其它时间 保存设备的小时以上的数据", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] dataIds) {
        return toAjax(monDevTcOtherService.removeByIds(Arrays.asList(dataIds)));
    }
}
