package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.MonDevTcReal;
import cn.cecep.talroad.service.IMonDevTcRealService;
import cn.cecep.talroad.vo.MonDevTcRealVo;
import cn.cecep.talroad.vo.query.MonDevTcRealQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 设备数据-陶瓷-实时 保存设备的实时数据Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控--工况数据--设备数据-陶瓷-实时 保存设备的实时数据")
@RestController
@RequestMapping("/monDevTcReal")
public class MonDevTcRealController extends BasesController
{
    @Autowired
    private IMonDevTcRealService monDevTcRealService;

    /**
     * 查询设备数据-陶瓷-实时 保存设备的实时数据列表
     */
    @ApiOperation(value = "查询设备数据-陶瓷-实时 保存设备的实时数据")
    @GetMapping("/list")
    public AjaxResults<IPage<MonDevTcReal>> list(MonDevTcRealQuery monDevTcReal)
    {
        Page<MonDevTcReal> page = new Page<>(monDevTcReal.getPageNum(), monDevTcReal.getPageSize());
        IPage<MonDevTcReal> list = monDevTcRealService.page(page, MonDevTcRealQuery.createLambdaQueryWrapper(monDevTcReal));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出设备数据-陶瓷-实时 保存设备的实时数据列表
     */
    @ApiOperation(value = "导出设备数据-陶瓷-实时 保存设备的实时数据")
    @Log(title = "设备数据-陶瓷-实时 保存设备的实时数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, MonDevTcReal monDevTcReal)
    {
        List<MonDevTcReal> list = monDevTcRealService.selectMonDevTcRealList(monDevTcReal);
        ExcelUtil<MonDevTcReal> util = new ExcelUtil<MonDevTcReal>(MonDevTcReal.class);
        util.exportExcel(response, list, "设备数据-陶瓷-实时 保存设备的实时数据数据");
    }

    /**
     * 获取设备数据-陶瓷-实时 保存设备的实时数据详细信息
     */
    @ApiOperation(value = "获取设备数据-陶瓷-实时 保存设备的实时数据详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<MonDevTcReal> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(monDevTcRealService.getById(id));
    }
    /**
     * 新增设备数据-陶瓷-实时 保存设备的实时数据
     */
    @ApiOperation(value = "新增设备数据-陶瓷-实时 保存设备的实时数据")
    @Log(title = "设备数据-陶瓷-实时 保存设备的实时数据", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody MonDevTcReal monDevTcReal) {
        return toAjax(monDevTcRealService.save(monDevTcReal));
    }
    /**
     * 修改设备数据-陶瓷-实时 保存设备的实时数据
     */
    @ApiOperation(value = "修改设备数据-陶瓷-实时 保存设备的实时数据")
    @Log(title = "设备数据-陶瓷-实时 保存设备的实时数据", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody MonDevTcRealVo monDevTcRealVo) {
        if(ObjectUtil.isEmpty(monDevTcRealVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(monDevTcRealVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        MonDevTcReal monDevTcReal = monDevTcRealService.getById(monDevTcRealVo.getDataId());
        if(ObjectUtil.isEmpty(monDevTcReal)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", monDevTcRealVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(monDevTcRealVo,monDevTcReal);
        return toAjax(monDevTcRealService.saveOrUpdate(monDevTcReal));
    }

    /**
     * 删除设备数据-陶瓷-实时 保存设备的实时数据
     */
    @ApiOperation(value = "删除设备数据-陶瓷-实时 保存设备的实时数据")
    @RequiresPermissions("talroad:real:remove")
    @Log(title = "设备数据-陶瓷-实时 保存设备的实时数据", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] dataIds) {
        return toAjax(monDevTcRealService.removeByIds(Arrays.asList(dataIds)));
    }
}
