package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.MonElecEvent;
import cn.cecep.talroad.service.IMonElecEventService;
import cn.cecep.talroad.vo.MonElecEventVo;
import cn.cecep.talroad.vo.query.MonElecEventQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 有组织-分表计电设备对应 两类设备为多对多的关系Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控-有组织-分表计电设备对应 两类设备为多对多的关系")
@RestController
@RequestMapping("/monElecEvent")
public class MonElecEventController extends BasesController
{
    @Autowired
    private IMonElecEventService monElecEventService;

    /**
     * 查询有组织-分表计电设备对应 两类设备为多对多的关系列表
     */
    @ApiOperation(value = "查询有组织-分表计电设备对应 两类设备为多对多的关系")
    @GetMapping("/list")
    public AjaxResults<IPage<MonElecEvent>> list(MonElecEventQuery monElecEvent)
    {
        Page<MonElecEvent> page = new Page<>(monElecEvent.getPageNum(), monElecEvent.getPageSize());
        IPage<MonElecEvent> list = monElecEventService.page(page, MonElecEventQuery.createLambdaQueryWrapper(monElecEvent));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出有组织-分表计电设备对应 两类设备为多对多的关系列表
     */
    @ApiOperation(value = "导出有组织-分表计电设备对应 两类设备为多对多的关系")
    @Log(title = "有组织-分表计电设备对应 两类设备为多对多的关系", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, MonElecEvent monElecEvent)
    {
        List<MonElecEvent> list = monElecEventService.selectMonElecEventList(monElecEvent);
        ExcelUtil<MonElecEvent> util = new ExcelUtil<MonElecEvent>(MonElecEvent.class);
        util.exportExcel(response, list, "有组织-分表计电设备对应 两类设备为多对多的关系数据");
    }

    /**
     * 获取有组织-分表计电设备对应 两类设备为多对多的关系详细信息
     */
    @ApiOperation(value = "获取有组织-分表计电设备对应 两类设备为多对多的关系详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<MonElecEvent> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(monElecEventService.getById(id));
    }
    /**
     * 新增有组织-分表计电设备对应 两类设备为多对多的关系
     */
    @ApiOperation(value = "新增有组织-分表计电设备对应 两类设备为多对多的关系")
    @Log(title = "有组织-分表计电设备对应 两类设备为多对多的关系", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody MonElecEvent monElecEvent) {
        return toAjax(monElecEventService.save(monElecEvent));
    }
    /**
     * 修改有组织-分表计电设备对应 两类设备为多对多的关系
     */
    @ApiOperation(value = "修改有组织-分表计电设备对应 两类设备为多对多的关系")
    @Log(title = "有组织-分表计电设备对应 两类设备为多对多的关系", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody MonElecEventVo monElecEventVo) {
        if(ObjectUtil.isEmpty(monElecEventVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(monElecEventVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        MonElecEvent monElecEvent = monElecEventService.getById(monElecEventVo.getDataId());
        if(ObjectUtil.isEmpty(monElecEvent)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", monElecEventVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(monElecEventVo,monElecEvent);
        return toAjax(monElecEventService.saveOrUpdate(monElecEvent));
    }

    /**
     * 删除有组织-分表计电设备对应 两类设备为多对多的关系
     */
    @ApiOperation(value = "删除有组织-分表计电设备对应 两类设备为多对多的关系")
    @RequiresPermissions("talroad:event:remove")
    @Log(title = "有组织-分表计电设备对应 两类设备为多对多的关系", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] dataIds) {
        return toAjax(monElecEventService.removeByIds(Arrays.asList(dataIds)));
    }
}
