package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.OdsMeshingWeiStationHourIQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.OdsMeshingWeiStationHourI;
import cn.cecep.talroad.vo.OdsMeshingWeiStationHourIVo;
import cn.cecep.talroad.service.IOdsMeshingWeiStationHourIService;


/**
 * * 开发公司：中节能天融科技有限公司
 * ods层网格化微站站点小时增量Controller 控制层
 *
 * @author szhpt
 * @date 2023-06-10*/
@Api(tags = "ods层网格化微站站点小时增量")
@RestController
@RequestMapping("/meshingWeiStation")
public class OdsMeshingWeiStationHourIController extends BasesController
{
    @Autowired
    private IOdsMeshingWeiStationHourIService odsMeshingWeiStationHourIService;

    /**
     * 查询ods层网格化微站站点小时增量列表
     */
    @ApiOperation(value = "查询ods层网格化微站站点小时增量")
    @GetMapping("/list")
    public AjaxResults<IPage<OdsMeshingWeiStationHourI>> list(OdsMeshingWeiStationHourIQuery odsMeshingWeiStationHourI)
    {
        Page<OdsMeshingWeiStationHourI> page = new Page<>(odsMeshingWeiStationHourI.getPageNum(), odsMeshingWeiStationHourI.getPageSize());
        IPage<OdsMeshingWeiStationHourI> list = odsMeshingWeiStationHourIService.page(page, OdsMeshingWeiStationHourIQuery.createLambdaQueryWrapper(odsMeshingWeiStationHourI));
        return AjaxResults.success(list);
    }


    /**
     * 导出ods层网格化微站站点小时增量列表
     */
    @ApiOperation(value = "导出ods层网格化微站站点小时增量")
    @Log(title = "ods层网格化微站站点小时增量", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, OdsMeshingWeiStationHourI odsMeshingWeiStationHourI)
    {
        List<OdsMeshingWeiStationHourI> list = odsMeshingWeiStationHourIService.selectOdsMeshingWeiStationHourIList(odsMeshingWeiStationHourI);
        ExcelUtil<OdsMeshingWeiStationHourI> util = new ExcelUtil<OdsMeshingWeiStationHourI>(OdsMeshingWeiStationHourI.class);
        util.exportExcel(response, list, "ods层网格化微站站点小时增量数据");
    }

    /**
     * 获取ods层网格化微站站点小时增量详细信息
     */
    @ApiOperation(value = "获取ods层网格化微站站点小时增量详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<OdsMeshingWeiStationHourI> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(odsMeshingWeiStationHourIService.getById(id));
    }
    /**
     * 新增ods层网格化微站站点小时增量
     */
    @ApiOperation(value = "新增ods层网格化微站站点小时增量")
    @Log(title = "ods层网格化微站站点小时增量", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody OdsMeshingWeiStationHourI odsMeshingWeiStationHourI) {
        return toAjax(odsMeshingWeiStationHourIService.save(odsMeshingWeiStationHourI));
    }
    /**
     * 修改ods层网格化微站站点小时增量
     */
    @ApiOperation(value = "修改ods层网格化微站站点小时增量")
    @Log(title = "ods层网格化微站站点小时增量", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody OdsMeshingWeiStationHourIVo odsMeshingWeiStationHourIVo) {
        if(ObjectUtil.isEmpty(odsMeshingWeiStationHourIVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(odsMeshingWeiStationHourIVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        OdsMeshingWeiStationHourI odsMeshingWeiStationHourI = odsMeshingWeiStationHourIService.getById(odsMeshingWeiStationHourIVo.getId());
        if(ObjectUtil.isEmpty(odsMeshingWeiStationHourI)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", odsMeshingWeiStationHourIVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(odsMeshingWeiStationHourIVo,odsMeshingWeiStationHourI);
        return toAjax(odsMeshingWeiStationHourIService.saveOrUpdate(odsMeshingWeiStationHourI));
    }

    /**
     * 删除ods层网格化微站站点小时增量
     */
    @ApiOperation(value = "删除ods层网格化微站站点小时增量")
    @RequiresPermissions("talroad:i:remove")
    @Log(title = "ods层网格化微站站点小时增量", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(odsMeshingWeiStationHourIService.removeByIds(Arrays.asList(ids)));
    }
}
