package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.OdsOnlineOlPsWhitelistAQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.OdsOnlineOlPsWhitelistA;
import cn.cecep.talroad.vo.OdsOnlineOlPsWhitelistAVo;
import cn.cecep.talroad.service.IOdsOnlineOlPsWhitelistAService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 白名单信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-06-10*/
@Api(tags = "白名单信息")
@RestController
@RequestMapping("/onlineWhiteList")
public class OdsOnlineOlPsWhitelistAController extends BasesController
{
    @Autowired
    private IOdsOnlineOlPsWhitelistAService odsOnlineOlPsWhitelistAService;

    /**
     * 查询白名单信息列表
     */
    @ApiOperation(value = "查询白名单信息")
    @GetMapping("/list")
    public AjaxResults<IPage<OdsOnlineOlPsWhitelistA>> list(OdsOnlineOlPsWhitelistAQuery odsOnlineOlPsWhitelistA)
    {
        Page<OdsOnlineOlPsWhitelistA> page = new Page<>(odsOnlineOlPsWhitelistA.getPageNum(), odsOnlineOlPsWhitelistA.getPageSize());
        IPage<OdsOnlineOlPsWhitelistA> list = odsOnlineOlPsWhitelistAService.page(page, OdsOnlineOlPsWhitelistAQuery.createLambdaQueryWrapper(odsOnlineOlPsWhitelistA));
        return AjaxResults.success(list);
    }


    /**
     * 导出白名单信息列表
     */
    @ApiOperation(value = "导出白名单信息")
    @Log(title = "白名单信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, OdsOnlineOlPsWhitelistA odsOnlineOlPsWhitelistA)
    {
        List<OdsOnlineOlPsWhitelistA> list = odsOnlineOlPsWhitelistAService.selectOdsOnlineOlPsWhitelistAList(odsOnlineOlPsWhitelistA);
        ExcelUtil<OdsOnlineOlPsWhitelistA> util = new ExcelUtil<OdsOnlineOlPsWhitelistA>(OdsOnlineOlPsWhitelistA.class);
        util.exportExcel(response, list, "白名单信息数据");
    }

    /**
     * 获取白名单信息详细信息
     */
    @ApiOperation(value = "获取白名单信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<OdsOnlineOlPsWhitelistA> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(odsOnlineOlPsWhitelistAService.getById(id));
    }
    /**
     * 新增白名单信息
     */
    @ApiOperation(value = "新增白名单信息")
    @Log(title = "白名单信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody OdsOnlineOlPsWhitelistA odsOnlineOlPsWhitelistA) {
        return toAjax(odsOnlineOlPsWhitelistAService.save(odsOnlineOlPsWhitelistA));
    }
    /**
     * 修改白名单信息
     */
    @ApiOperation(value = "修改白名单信息")
    @Log(title = "白名单信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody OdsOnlineOlPsWhitelistAVo odsOnlineOlPsWhitelistAVo) {
        if(ObjectUtil.isEmpty(odsOnlineOlPsWhitelistAVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(odsOnlineOlPsWhitelistAVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        OdsOnlineOlPsWhitelistA odsOnlineOlPsWhitelistA = odsOnlineOlPsWhitelistAService.getById(odsOnlineOlPsWhitelistAVo.getId());
        if(ObjectUtil.isEmpty(odsOnlineOlPsWhitelistA)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", odsOnlineOlPsWhitelistAVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(odsOnlineOlPsWhitelistAVo,odsOnlineOlPsWhitelistA);
        return toAjax(odsOnlineOlPsWhitelistAService.saveOrUpdate(odsOnlineOlPsWhitelistA));
    }

    /**
     * 删除白名单信息
     */
    @ApiOperation(value = "删除白名单信息")
    @RequiresPermissions("talroad:a:remove")
    @Log(title = "白名单信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(odsOnlineOlPsWhitelistAService.removeByIds(Arrays.asList(ids)));
    }
}
