package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.OdsOnlineOlPsinfoAQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.OdsOnlineOlPsinfoA;
import cn.cecep.talroad.vo.OdsOnlineOlPsinfoAVo;
import cn.cecep.talroad.service.IOdsOnlineOlPsinfoAService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 公司信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-06-10*/
@Api(tags = "公司信息")
@RestController
@RequestMapping("/onlinePsInfo")
public class OdsOnlineOlPsinfoAController extends BasesController
{
    @Autowired
    private IOdsOnlineOlPsinfoAService odsOnlineOlPsinfoAService;

    /**
     * 查询公司信息列表
     */
    @ApiOperation(value = "查询公司信息")
    @GetMapping("/list")
    public AjaxResults<IPage<OdsOnlineOlPsinfoA>> list(OdsOnlineOlPsinfoAQuery odsOnlineOlPsinfoA)
    {
        Page<OdsOnlineOlPsinfoA> page = new Page<>(odsOnlineOlPsinfoA.getPageNum(), odsOnlineOlPsinfoA.getPageSize());
        IPage<OdsOnlineOlPsinfoA> list = odsOnlineOlPsinfoAService.page(page, OdsOnlineOlPsinfoAQuery.createLambdaQueryWrapper(odsOnlineOlPsinfoA));
        return AjaxResults.success(list);
    }


    /**
     * 导出公司信息列表
     */
    @ApiOperation(value = "导出公司信息")
    @Log(title = "公司信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, OdsOnlineOlPsinfoA odsOnlineOlPsinfoA)
    {
        List<OdsOnlineOlPsinfoA> list = odsOnlineOlPsinfoAService.selectOdsOnlineOlPsinfoAList(odsOnlineOlPsinfoA);
        ExcelUtil<OdsOnlineOlPsinfoA> util = new ExcelUtil<OdsOnlineOlPsinfoA>(OdsOnlineOlPsinfoA.class);
        util.exportExcel(response, list, "公司信息数据");
    }

    /**
     * 获取公司信息详细信息
     */
    @ApiOperation(value = "获取公司信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<OdsOnlineOlPsinfoA> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(odsOnlineOlPsinfoAService.getById(id));
    }
    /**
     * 新增公司信息
     */
    @ApiOperation(value = "新增公司信息")
    @Log(title = "公司信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody OdsOnlineOlPsinfoA odsOnlineOlPsinfoA) {
        return toAjax(odsOnlineOlPsinfoAService.save(odsOnlineOlPsinfoA));
    }
    /**
     * 修改公司信息
     */
    @ApiOperation(value = "修改公司信息")
    @Log(title = "公司信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody OdsOnlineOlPsinfoAVo odsOnlineOlPsinfoAVo) {
        if(ObjectUtil.isEmpty(odsOnlineOlPsinfoAVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(odsOnlineOlPsinfoAVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        OdsOnlineOlPsinfoA odsOnlineOlPsinfoA = odsOnlineOlPsinfoAService.getById(odsOnlineOlPsinfoAVo.getId());
        if(ObjectUtil.isEmpty(odsOnlineOlPsinfoA)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", odsOnlineOlPsinfoAVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(odsOnlineOlPsinfoAVo,odsOnlineOlPsinfoA);
        return toAjax(odsOnlineOlPsinfoAService.saveOrUpdate(odsOnlineOlPsinfoA));
    }

    /**
     * 删除公司信息
     */
    @ApiOperation(value = "删除公司信息")
    @RequiresPermissions("talroad:a:remove")
    @Log(title = "公司信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(odsOnlineOlPsinfoAService.removeByIds(Arrays.asList(ids)));
    }
}
