package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.OdsOnlineOlStopinfoAQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.OdsOnlineOlStopinfoA;
import cn.cecep.talroad.vo.OdsOnlineOlStopinfoAVo;
import cn.cecep.talroad.service.IOdsOnlineOlStopinfoAService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 停产信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-06-10*/
@Api(tags = "停产信息")
@RestController
@RequestMapping("/onlineStopInfo")
public class OdsOnlineOlStopinfoAController extends BasesController
{
    @Autowired
    private IOdsOnlineOlStopinfoAService odsOnlineOlStopinfoAService;

    /**
     * 查询停产信息列表
     */
    @ApiOperation(value = "查询停产信息")
    @GetMapping("/list")
    public AjaxResults<IPage<OdsOnlineOlStopinfoA>> list(OdsOnlineOlStopinfoAQuery odsOnlineOlStopinfoA)
    {
        Page<OdsOnlineOlStopinfoA> page = new Page<>(odsOnlineOlStopinfoA.getPageNum(), odsOnlineOlStopinfoA.getPageSize());
        IPage<OdsOnlineOlStopinfoA> list = odsOnlineOlStopinfoAService.page(page, OdsOnlineOlStopinfoAQuery.createLambdaQueryWrapper(odsOnlineOlStopinfoA));
        return AjaxResults.success(list);
    }


    /**
     * 导出停产信息列表
     */
    @ApiOperation(value = "导出停产信息")
    @Log(title = "停产信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, OdsOnlineOlStopinfoA odsOnlineOlStopinfoA)
    {
        List<OdsOnlineOlStopinfoA> list = odsOnlineOlStopinfoAService.selectOdsOnlineOlStopinfoAList(odsOnlineOlStopinfoA);
        ExcelUtil<OdsOnlineOlStopinfoA> util = new ExcelUtil<OdsOnlineOlStopinfoA>(OdsOnlineOlStopinfoA.class);
        util.exportExcel(response, list, "停产信息数据");
    }

    /**
     * 获取停产信息详细信息
     */
    @ApiOperation(value = "获取停产信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<OdsOnlineOlStopinfoA> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(odsOnlineOlStopinfoAService.getById(id));
    }
    /**
     * 新增停产信息
     */
    @ApiOperation(value = "新增停产信息")
    @Log(title = "停产信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody OdsOnlineOlStopinfoA odsOnlineOlStopinfoA) {
        return toAjax(odsOnlineOlStopinfoAService.save(odsOnlineOlStopinfoA));
    }
    /**
     * 修改停产信息
     */
    @ApiOperation(value = "修改停产信息")
    @Log(title = "停产信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody OdsOnlineOlStopinfoAVo odsOnlineOlStopinfoAVo) {
        if(ObjectUtil.isEmpty(odsOnlineOlStopinfoAVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(odsOnlineOlStopinfoAVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        OdsOnlineOlStopinfoA odsOnlineOlStopinfoA = odsOnlineOlStopinfoAService.getById(odsOnlineOlStopinfoAVo.getId());
        if(ObjectUtil.isEmpty(odsOnlineOlStopinfoA)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", odsOnlineOlStopinfoAVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(odsOnlineOlStopinfoAVo,odsOnlineOlStopinfoA);
        return toAjax(odsOnlineOlStopinfoAService.saveOrUpdate(odsOnlineOlStopinfoA));
    }

    /**
     * 删除停产信息
     */
    @ApiOperation(value = "删除停产信息")
    @RequiresPermissions("talroad:a:remove")
    @Log(title = "停产信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(odsOnlineOlStopinfoAService.removeByIds(Arrays.asList(ids)));
    }
}
