package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.PcAdsl;
import cn.cecep.talroad.service.IPcAdslService;
import cn.cecep.talroad.vo.PcAdslVo;
import cn.cecep.talroad.vo.query.PcAdslQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 企业接入数据类型Controller 控制层
 *
 * @author szhpt
 * @date 2023-02-02*/
@Api(tags = "企业接入数据类型")
@RestController
@RequestMapping("/pcAdsl")
public class PcAdslController extends BasesController
{
    @Autowired
    private IPcAdslService pcAdslService;

    /**
     * 查询企业接入数据类型列表
     */
    @ApiOperation(value = "查询企业接入数据类型")
    @GetMapping("/list")
    public AjaxResults<IPage<PcAdsl>> list(PcAdslQuery pcAdsl)
    {
        Page<PcAdsl> page = new Page<>(pcAdsl.getPageNum(), pcAdsl.getPageSize());
        IPage<PcAdsl> list = pcAdslService.page(page, PcAdslQuery.createLambdaQueryWrapper(pcAdsl));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出企业接入数据类型列表
     */
    @ApiOperation(value = "导出企业接入数据类型")
    @Log(title = "企业接入数据类型", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PcAdsl pcAdsl)
    {
        List<PcAdsl> list = pcAdslService.selectPcAdslList(pcAdsl);
        ExcelUtil<PcAdsl> util = new ExcelUtil<PcAdsl>(PcAdsl.class);
        util.exportExcel(response, list, "企业接入数据类型数据");
    }

    /**
     * 获取企业接入数据类型详细信息
     */
    @ApiOperation(value = "获取企业接入数据类型详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<PcAdsl> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(pcAdslService.getById(id));
    }
    /**
     * 新增企业接入数据类型
     */
    @ApiOperation(value = "新增企业接入数据类型")
    @Log(title = "企业接入数据类型", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody PcAdsl pcAdsl) {
        return toAjax(pcAdslService.save(pcAdsl));
    }
    /**
     * 修改企业接入数据类型
     */
    @ApiOperation(value = "修改企业接入数据类型")
    @Log(title = "企业接入数据类型", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody PcAdslVo pcAdslVo) {
        if(ObjectUtil.isEmpty(pcAdslVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(pcAdslVo.getAdslId())){
            throw new InvalidParameterException("参数id为空");
        }
        PcAdsl pcAdsl = pcAdslService.getById(pcAdslVo.getAdslId());
        if(ObjectUtil.isEmpty(pcAdsl)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", pcAdslVo.getAdslId()));
        }
        //赋值
        BeanUtil.copyProperties(pcAdslVo,pcAdsl);
        return toAjax(pcAdslService.saveOrUpdate(pcAdsl));
    }

    /**
     * 删除企业接入数据类型
     */
    @ApiOperation(value = "删除企业接入数据类型")
    @RequiresPermissions("talroad:pcAdsl:remove")
    @Log(title = "企业接入数据类型", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(pcAdslService.removeByIds(Arrays.asList(ids)));
    }
}
