package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.PcAdslFactory;
import cn.cecep.talroad.service.IPcAdslFactoryService;
import cn.cecep.talroad.vo.PcAdslFactoryVo;
import cn.cecep.talroad.vo.query.PcAdslFactoryQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;


/**
 * * 开发公司：中节能天融科技有限公司
 * 企业与接入数据类型关联Controller 控制层
 *
 * @author szhpt
 * @date 2023-02-02*/
@Api(tags = "A企业与接入数据类型关联")
@RestController
@RequestMapping("/pcAdslFactory")
public class PcAdslFactoryController extends BasesController
{
    @Autowired
    private IPcAdslFactoryService pcAdslFactoryService;

    /**
     * 企业数据接入全面度
     */
    @ApiOperation(value = "企业数据接入全面度[highlight不为空则点亮]")
    @GetMapping(value = "/{factoryId}")
    public AjaxResults<List<Map<String, String>>> selectEntInDatas(@PathVariable("factoryId") String factoryId) {
        return AjaxResults.success(pcAdslFactoryService.selectEntInDatas(factoryId));
    }

    /**
     * 查询企业与接入数据类型关联列表
     */
    @ApiOperation(value = "查询企业与接入数据类型关联")
    @GetMapping("/list")
    public AjaxResults<IPage<PcAdslFactory>> list(PcAdslFactoryQuery pcAdslFactory)
    {
        Page<PcAdslFactory> page = new Page<>(pcAdslFactory.getPageNum(), pcAdslFactory.getPageSize());
        IPage<PcAdslFactory> list = pcAdslFactoryService.page(page, PcAdslFactoryQuery.createLambdaQueryWrapper(pcAdslFactory));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出企业与接入数据类型关联列表
     */
    @ApiOperation(value = "导出企业与接入数据类型关联")
    @Log(title = "企业与接入数据类型关联", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PcAdslFactory pcAdslFactory)
    {
        List<PcAdslFactory> list = pcAdslFactoryService.selectPcAdslFactoryList(pcAdslFactory);
        ExcelUtil<PcAdslFactory> util = new ExcelUtil<PcAdslFactory>(PcAdslFactory.class);
        util.exportExcel(response, list, "企业与接入数据类型关联数据");
    }

    /**
     * 获取企业与接入数据类型关联详细信息
     */
    @ApiOperation(value = "获取企业与接入数据类型关联详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<PcAdslFactory> getInfo(@PathVariable("id") Long id) {
        return AjaxResults.success(pcAdslFactoryService.getById(id));
    }
    /**
     * 新增企业与接入数据类型关联
     */
    @ApiOperation(value = "新增企业与接入数据类型关联")
    @Log(title = "企业与接入数据类型关联", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody PcAdslFactory pcAdslFactory) {
        return toAjax(pcAdslFactoryService.save(pcAdslFactory));
    }
    /**
     * 修改企业与接入数据类型关联
     */
    @ApiOperation(value = "修改企业与接入数据类型关联")
    @Log(title = "企业与接入数据类型关联", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody PcAdslFactoryVo pcAdslFactoryVo) {
        if(ObjectUtil.isEmpty(pcAdslFactoryVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(pcAdslFactoryVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        PcAdslFactory pcAdslFactory = pcAdslFactoryService.getById(pcAdslFactoryVo.getId());
        if(ObjectUtil.isEmpty(pcAdslFactory)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", pcAdslFactoryVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(pcAdslFactoryVo,pcAdslFactory);
        return toAjax(pcAdslFactoryService.saveOrUpdate(pcAdslFactory));
    }

    /**
     * 删除企业与接入数据类型关联
     */
    @ApiOperation(value = "删除企业与接入数据类型关联")
    @RequiresPermissions("talroad:pcAdslFactory:remove")
    @Log(title = "企业与接入数据类型关联", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(pcAdslFactoryService.removeByIds(Arrays.asList(ids)));
    }
}
