package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.PcFactoryPollutant;
import cn.cecep.talroad.service.IPcFactoryPollutantService;
import cn.cecep.talroad.vo.PcFactoryPollutantVo;
import cn.cecep.talroad.vo.query.PcFactoryPollutantQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 企业与污染物关系Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "基础信息--企业与污染物关系")
@RestController
@RequestMapping("/pcFactoryPollutant")
public class PcFactoryPollutantController extends BasesController
{
    @Autowired
    private IPcFactoryPollutantService pcFactoryPollutantService;

    /**
     * 查询企业与污染物关系列表
     */
    @ApiOperation(value = "查询企业与污染物关系")
    @GetMapping("/list")
    public AjaxResults<IPage<PcFactoryPollutant>> list(PcFactoryPollutantQuery pcFactoryPollutant)
    {
        Page<PcFactoryPollutant> page = new Page<>(pcFactoryPollutant.getPageNum(), pcFactoryPollutant.getPageSize());
        IPage<PcFactoryPollutant> list = pcFactoryPollutantService.page(page, PcFactoryPollutantQuery.createLambdaQueryWrapper(pcFactoryPollutant));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出企业与污染物关系列表
     */
    @ApiOperation(value = "导出企业与污染物关系")
    @Log(title = "企业与污染物关系", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PcFactoryPollutant pcFactoryPollutant)
    {
        List<PcFactoryPollutant> list = pcFactoryPollutantService.selectPcFactoryPollutantList(pcFactoryPollutant);
        ExcelUtil<PcFactoryPollutant> util = new ExcelUtil<PcFactoryPollutant>(PcFactoryPollutant.class);
        util.exportExcel(response, list, "企业与污染物关系数据");
    }

    /**
     * 获取企业与污染物关系详细信息
     */
    @ApiOperation(value = "获取企业与污染物关系详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<PcFactoryPollutant> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(pcFactoryPollutantService.getById(id));
    }
    /**
     * 新增企业与污染物关系
     */
    @ApiOperation(value = "新增企业与污染物关系")
    @Log(title = "企业与污染物关系", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody PcFactoryPollutant pcFactoryPollutant) {
        return toAjax(pcFactoryPollutantService.save(pcFactoryPollutant));
    }
    /**
     * 修改企业与污染物关系
     */
    @ApiOperation(value = "修改企业与污染物关系")
    @Log(title = "企业与污染物关系", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody PcFactoryPollutantVo pcFactoryPollutantVo) {
        if(ObjectUtil.isEmpty(pcFactoryPollutantVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(pcFactoryPollutantVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        PcFactoryPollutant pcFactoryPollutant = pcFactoryPollutantService.getById(pcFactoryPollutantVo.getId());
        if(ObjectUtil.isEmpty(pcFactoryPollutant)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", pcFactoryPollutantVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(pcFactoryPollutantVo,pcFactoryPollutant);
        return toAjax(pcFactoryPollutantService.saveOrUpdate(pcFactoryPollutant));
    }

    /**
     * 删除企业与污染物关系
     */
    @ApiOperation(value = "删除企业与污染物关系")
    @RequiresPermissions("talroad:pollutant:remove")
    @Log(title = "企业与污染物关系", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(pcFactoryPollutantService.removeByIds(Arrays.asList(ids)));
    }
}
