package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.PcManagementFacilities;
import cn.cecep.talroad.service.IPcManagementFacilitiesService;
import cn.cecep.talroad.vo.PcManagementFacilitiesVo;
import cn.cecep.talroad.vo.query.PcManagementFacilitiesQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 治理设施Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "治理设施")
@RestController
@RequestMapping("/pcManagementFacilities")
public class PcManagementFacilitiesController extends BasesController
{
    @Autowired
    private IPcManagementFacilitiesService pcManagementFacilitiesService;

    /**
     * 查询治理设施列表
     */
    @ApiOperation(value = "查询治理设施")
    @GetMapping("/list")
    public AjaxResults<IPage<PcManagementFacilities>> list(PcManagementFacilitiesQuery pcManagementFacilities)
    {
        Page<PcManagementFacilities> page = new Page<>(pcManagementFacilities.getPageNum(), pcManagementFacilities.getPageSize());
        IPage<PcManagementFacilities> list = pcManagementFacilitiesService.page(page, PcManagementFacilitiesQuery.createLambdaQueryWrapper(pcManagementFacilities));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出治理设施列表
     */
    @ApiOperation(value = "导出治理设施")
    @Log(title = "治理设施", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PcManagementFacilities pcManagementFacilities)
    {
        List<PcManagementFacilities> list = pcManagementFacilitiesService.selectPcManagementFacilitiesList(pcManagementFacilities);
        ExcelUtil<PcManagementFacilities> util = new ExcelUtil<PcManagementFacilities>(PcManagementFacilities.class);
        util.exportExcel(response, list, "治理设施数据");
    }

    /**
     * 获取治理设施详细信息
     */
    @ApiOperation(value = "获取治理设施详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<PcManagementFacilities> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(pcManagementFacilitiesService.getById(id));
    }
    /**
     * 新增治理设施
     */
    @ApiOperation(value = "新增治理设施")
    @Log(title = "治理设施", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody PcManagementFacilities pcManagementFacilities) {
        return toAjax(pcManagementFacilitiesService.save(pcManagementFacilities));
    }
    /**
     * 修改治理设施
     */
    @ApiOperation(value = "修改治理设施")
    @Log(title = "治理设施", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody PcManagementFacilitiesVo pcManagementFacilitiesVo) {
        if(ObjectUtil.isEmpty(pcManagementFacilitiesVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(pcManagementFacilitiesVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        PcManagementFacilities pcManagementFacilities = pcManagementFacilitiesService.getById(pcManagementFacilitiesVo.getId());
        if(ObjectUtil.isEmpty(pcManagementFacilities)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", pcManagementFacilitiesVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(pcManagementFacilitiesVo,pcManagementFacilities);
        return toAjax(pcManagementFacilitiesService.saveOrUpdate(pcManagementFacilities));
    }

    /**
     * 删除治理设施
     */
    @ApiOperation(value = "删除治理设施")
    @RequiresPermissions("talroad:facilities:remove")
    @Log(title = "治理设施", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(pcManagementFacilitiesService.removeByIds(Arrays.asList(ids)));
    }
}
