package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.PcMenu;
import cn.cecep.talroad.service.IPcMenuService;
import cn.cecep.talroad.vo.PcMenuVo;
import cn.cecep.talroad.vo.query.PcMenuQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 企业档案菜单管理Controller 控制层
 *
 * @author szhpt
 * @date 2023-02-02*/
@Api(tags = "企业档案菜单管理")
@RestController
@RequestMapping("/pcMenu")
public class PcMenuController extends BasesController
{
    @Autowired
    private IPcMenuService pcMenuService;

    /**
     * 查询企业档案菜单管理列表
     */
    @ApiOperation(value = "查询企业档案菜单管理")
    @GetMapping("/list")
    public AjaxResults<IPage<PcMenu>> list(PcMenuQuery pcMenu)
    {
        Page<PcMenu> page = new Page<>(pcMenu.getPageNum(), pcMenu.getPageSize());
        IPage<PcMenu> list = pcMenuService.page(page, PcMenuQuery.createLambdaQueryWrapper(pcMenu));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出企业档案菜单管理列表
     */
    @ApiOperation(value = "导出企业档案菜单管理")
    @Log(title = "企业档案菜单管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PcMenu pcMenu)
    {
        List<PcMenu> list = pcMenuService.selectPcMenuList(pcMenu);
        ExcelUtil<PcMenu> util = new ExcelUtil<PcMenu>(PcMenu.class);
        util.exportExcel(response, list, "企业档案菜单管理数据");
    }

    /**
     * 获取企业档案菜单管理详细信息
     */
    @ApiOperation(value = "获取企业档案菜单管理详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<PcMenu> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(pcMenuService.getById(id));
    }
    /**
     * 新增企业档案菜单管理
     */
    @ApiOperation(value = "新增企业档案菜单管理")
    @Log(title = "企业档案菜单管理", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody PcMenu pcMenu) {
        return toAjax(pcMenuService.save(pcMenu));
    }
    /**
     * 修改企业档案菜单管理
     */
    @ApiOperation(value = "修改企业档案菜单管理")
    @Log(title = "企业档案菜单管理", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody PcMenuVo pcMenuVo) {
        if(ObjectUtil.isEmpty(pcMenuVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(pcMenuVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        PcMenu pcMenu = pcMenuService.getById(pcMenuVo.getId());
        if(ObjectUtil.isEmpty(pcMenu)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", pcMenuVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(pcMenuVo,pcMenu);
        return toAjax(pcMenuService.saveOrUpdate(pcMenu));
    }

    /**
     * 删除企业档案菜单管理
     */
    @ApiOperation(value = "删除企业档案菜单管理")
    @RequiresPermissions("talroad:pcMenu:remove")
    @Log(title = "企业档案菜单管理", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(pcMenuService.removeByIds(Arrays.asList(ids)));
    }
}
