package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.PcMenuFactory;
import cn.cecep.talroad.service.IPcMenuFactoryService;
import cn.cecep.talroad.vo.PcMenuFactoryVo;
import cn.cecep.talroad.vo.query.PcMenuFactoryQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 企业档案菜单与企业关联中间Controller 控制层
 *
 * @author szhpt
 * @date 2023-02-02*/
@Api(tags = "企业档案菜单与企业关联中间")
@RestController
@RequestMapping("/pcMenuFactory")
public class PcMenuFactoryController extends BasesController
{
    @Autowired
    private IPcMenuFactoryService pcMenuFactoryService;

    /**
     * 查询企业档案菜单与企业关联中间列表
     */
    @ApiOperation(value = "查询企业档案菜单与企业关联中间")
    @GetMapping("/list")
    public AjaxResults<IPage<PcMenuFactory>> list(PcMenuFactoryQuery pcMenuFactory)
    {
        Page<PcMenuFactory> page = new Page<>(pcMenuFactory.getPageNum(), pcMenuFactory.getPageSize());
        IPage<PcMenuFactory> list = pcMenuFactoryService.page(page, PcMenuFactoryQuery.createLambdaQueryWrapper(pcMenuFactory));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出企业档案菜单与企业关联中间列表
     */
    @ApiOperation(value = "导出企业档案菜单与企业关联中间")
    @Log(title = "企业档案菜单与企业关联中间", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PcMenuFactory pcMenuFactory)
    {
        List<PcMenuFactory> list = pcMenuFactoryService.selectPcMenuFactoryList(pcMenuFactory);
        ExcelUtil<PcMenuFactory> util = new ExcelUtil<PcMenuFactory>(PcMenuFactory.class);
        util.exportExcel(response, list, "企业档案菜单与企业关联中间数据");
    }

    /**
     * 获取企业档案菜单与企业关联中间详细信息
     */
    @ApiOperation(value = "获取企业档案菜单与企业关联中间详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<PcMenuFactory> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(pcMenuFactoryService.getById(id));
    }
    /**
     * 新增企业档案菜单与企业关联中间
     */
    @ApiOperation(value = "新增企业档案菜单与企业关联中间")
    @Log(title = "企业档案菜单与企业关联中间", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody PcMenuFactory pcMenuFactory) {
        return toAjax(pcMenuFactoryService.save(pcMenuFactory));
    }
    /**
     * 修改企业档案菜单与企业关联中间
     */
    @ApiOperation(value = "修改企业档案菜单与企业关联中间")
    @Log(title = "企业档案菜单与企业关联中间", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody PcMenuFactoryVo pcMenuFactoryVo) {
        if(ObjectUtil.isEmpty(pcMenuFactoryVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(pcMenuFactoryVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        PcMenuFactory pcMenuFactory = pcMenuFactoryService.getById(pcMenuFactoryVo.getId());
        if(ObjectUtil.isEmpty(pcMenuFactory)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", pcMenuFactoryVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(pcMenuFactoryVo,pcMenuFactory);
        return toAjax(pcMenuFactoryService.saveOrUpdate(pcMenuFactory));
    }

    /**
     * 删除企业档案菜单与企业关联中间
     */
    @ApiOperation(value = "删除企业档案菜单与企业关联中间")
    @RequiresPermissions("talroad:pcMenuFactory:remove")
    @Log(title = "企业档案菜单与企业关联中间", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(pcMenuFactoryService.removeByIds(Arrays.asList(ids)));
    }
}
