package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.PcNoiseType;
import cn.cecep.talroad.service.IPcNoiseTypeService;
import cn.cecep.talroad.vo.PcNoiseTypeVo;
import cn.cecep.talroad.vo.query.PcNoiseTypeQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 噪声排放类型Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-11*/
@Api(tags = "基本档案--噪声排放类型")
@RestController
@RequestMapping("/pcNoiseType")
public class PcNoiseTypeController extends BasesController
{
    @Autowired
    private IPcNoiseTypeService pcNoiseTypeService;

    /**
     * 查询噪声排放类型列表
     */
    @ApiOperation(value = "查询噪声排放类型")
    @GetMapping("/list")
    public AjaxResults<IPage<PcNoiseType>> list(PcNoiseTypeQuery pcNoiseType)
    {
        Page<PcNoiseType> page = new Page<>(pcNoiseType.getPageNum(), pcNoiseType.getPageSize());
        IPage<PcNoiseType> list = pcNoiseTypeService.page(page, PcNoiseTypeQuery.createLambdaQueryWrapper(pcNoiseType));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出噪声排放类型列表
     */
    @ApiOperation(value = "导出噪声排放类型")
    @Log(title = "噪声排放类型", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PcNoiseType pcNoiseType)
    {
        List<PcNoiseType> list = pcNoiseTypeService.selectPcNoiseTypeList(pcNoiseType);
        ExcelUtil<PcNoiseType> util = new ExcelUtil<PcNoiseType>(PcNoiseType.class);
        util.exportExcel(response, list, "噪声排放类型数据");
    }

    /**
     * 获取噪声排放类型详细信息
     */
    @ApiOperation(value = "获取噪声排放类型详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<PcNoiseType> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(pcNoiseTypeService.getById(id));
    }
    /**
     * 新增噪声排放类型
     */
    @ApiOperation(value = "新增噪声排放类型")
    @Log(title = "噪声排放类型", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody PcNoiseType pcNoiseType) {
        return toAjax(pcNoiseTypeService.save(pcNoiseType));
    }
    /**
     * 修改噪声排放类型
     */
    @ApiOperation(value = "修改噪声排放类型")
    @Log(title = "噪声排放类型", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody PcNoiseTypeVo pcNoiseTypeVo) {
        if(ObjectUtil.isEmpty(pcNoiseTypeVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(pcNoiseTypeVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        PcNoiseType pcNoiseType = pcNoiseTypeService.getById(pcNoiseTypeVo.getId());
        if(ObjectUtil.isEmpty(pcNoiseType)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", pcNoiseTypeVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(pcNoiseTypeVo,pcNoiseType);
        return toAjax(pcNoiseTypeService.saveOrUpdate(pcNoiseType));
    }

    /**
     * 删除噪声排放类型
     */
    @ApiOperation(value = "删除噪声排放类型")
    @RequiresPermissions("talroad:type:remove")
    @Log(title = "噪声排放类型", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(pcNoiseTypeService.removeByIds(Arrays.asList(ids)));
    }
}
