package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.PcSolidHazardousType;
import cn.cecep.talroad.service.IPcSolidHazardousTypeService;
import cn.cecep.talroad.vo.PcSolidHazardousTypeVo;
import cn.cecep.talroad.vo.query.PcSolidHazardousTypeQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 固危废贮存信息--危险废物名录类型Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-21*/
@Api(tags = "固危废贮存信息--危险废物名录类型")
@RestController
@RequestMapping("/pcSolidHazardousType")
public class PcSolidHazardousTypeController extends BasesController
{
    @Autowired
    private IPcSolidHazardousTypeService pcSolidHazardousTypeService;

    /**
     * 查询固危废贮存信息--危险废物名录类型列表
     */
    @ApiOperation(value = "查询固危废贮存信息--危险废物名录类型")
    @GetMapping("/list")
    public AjaxResults<IPage<PcSolidHazardousType>> list(PcSolidHazardousTypeQuery pcSolidHazardousType)
    {
        Page<PcSolidHazardousType> page = new Page<>(pcSolidHazardousType.getPageNum(), pcSolidHazardousType.getPageSize());
        IPage<PcSolidHazardousType> list = pcSolidHazardousTypeService.page(page, PcSolidHazardousTypeQuery.createLambdaQueryWrapper(pcSolidHazardousType));
        return AjaxResults.success(list);
    }
    

    /**
     * 导出固危废贮存信息--危险废物名录类型列表
     */
    @ApiOperation(value = "导出固危废贮存信息--危险废物名录类型")
    @Log(title = "固危废贮存信息--危险废物名录类型", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PcSolidHazardousTypeQuery pcSolidHazardousType)
    {
        List<PcSolidHazardousType> list = pcSolidHazardousTypeService.list(PcSolidHazardousTypeQuery.createLambdaQueryWrapper(pcSolidHazardousType));
        ExcelUtil<PcSolidHazardousType> util = new ExcelUtil<>(PcSolidHazardousType.class);
        util.exportExcel(response, list, "固危废贮存信息--危险废物名录类型数据");
    }

    /**
     * 获取固危废贮存信息--危险废物名录类型详细信息
     */
    @ApiOperation(value = "获取固危废贮存信息--危险废物名录类型详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<PcSolidHazardousType> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(pcSolidHazardousTypeService.getById(id));
    }
    /**
     * 新增固危废贮存信息--危险废物名录类型
     */
    @ApiOperation(value = "新增固危废贮存信息--危险废物名录类型")
    @Log(title = "固危废贮存信息--危险废物名录类型", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody PcSolidHazardousType pcSolidHazardousType) {
        return toAjax(pcSolidHazardousTypeService.save(pcSolidHazardousType));
    }
    /**
     * 修改固危废贮存信息--危险废物名录类型
     */
    @ApiOperation(value = "修改固危废贮存信息--危险废物名录类型")
    @Log(title = "固危废贮存信息--危险废物名录类型", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody PcSolidHazardousTypeVo pcSolidHazardousTypeVo) {
        if(ObjectUtil.isEmpty(pcSolidHazardousTypeVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(pcSolidHazardousTypeVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        PcSolidHazardousType pcSolidHazardousType = pcSolidHazardousTypeService.getById(pcSolidHazardousTypeVo.getId());
        if(ObjectUtil.isEmpty(pcSolidHazardousType)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", pcSolidHazardousTypeVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(pcSolidHazardousTypeVo,pcSolidHazardousType);
        return toAjax(pcSolidHazardousTypeService.saveOrUpdate(pcSolidHazardousType));
    }

    /**
     * 删除固危废贮存信息--危险废物名录类型
     */
    @ApiOperation(value = "删除固危废贮存信息--危险废物名录类型")
    @RequiresPermissions("talroad:pcSolidHazardousType:remove")
    @Log(title = "固危废贮存信息--危险废物名录类型", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(pcSolidHazardousTypeService.removeByIds(Arrays.asList(ids)));
    }
}
