package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.PcStationBuilding;
import cn.cecep.talroad.service.IPcStationBuildingService;
import cn.cecep.talroad.vo.PcStationBuildingVo;
import cn.cecep.talroad.vo.query.PcStationBuildingQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 站房基本信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控--站房基本信息")
@RestController
@RequestMapping("/pcStationBuilding")
public class PcStationBuildingController extends BasesController
{
    @Autowired
    private IPcStationBuildingService pcStationBuildingService;

    /**
     * 查询废气有组织、无组织排口列表
     */
    @ApiOperation(value = "站房下拉框列表")
    @GetMapping("/selectList")
    public AjaxResults<List<PcStationBuilding>> selectList(PcStationBuilding pcStationBuilding)
    {
        BaseMapper<PcStationBuilding> baseMapper = pcStationBuildingService.getBaseMapper();
        QueryWrapper<PcStationBuilding> wrapper = new QueryWrapper<>(pcStationBuilding);
        wrapper.select("distinct factory_id, station_building_code, station_building_name");
        wrapper.orderByAsc("station_building_code");
        List<PcStationBuilding> pcStationBuildingList = baseMapper.selectList(wrapper);
        return AjaxResults.success(pcStationBuildingList);
    }

    /**
     * 查询站房基本信息列表
     */
    @ApiOperation(value = "查询站房基本信息")
    @GetMapping("/list")
    public AjaxResults<IPage<PcStationBuilding>> list(PcStationBuildingQuery pcStationBuilding)
    {
        if (StrUtil.isEmpty(pcStationBuilding.getStationBuildingCode())) {
            return AjaxResults.error("查询失败，站点未选择！");
        }
        Page<PcStationBuilding> page = new Page<>(pcStationBuilding.getPageNum(), pcStationBuilding.getPageSize());
        IPage<PcStationBuilding> list = pcStationBuildingService.page(page, PcStationBuildingQuery.createLambdaQueryWrapper(pcStationBuilding));
        return AjaxResults.success(list);
    }


    /**
     * 导出站房基本信息列表
     */
    @ApiOperation(value = "导出站房基本信息")
    @Log(title = "站房基本信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PcStationBuilding pcStationBuilding)
    {
        List<PcStationBuilding> list = pcStationBuildingService.selectPcStationBuildingList(pcStationBuilding);
        ExcelUtil<PcStationBuilding> util = new ExcelUtil<PcStationBuilding>(PcStationBuilding.class);
        util.exportExcel(response, list, "站房基本信息数据");
    }

    /**
     * 获取站房基本信息详细信息
     */
    @ApiOperation(value = "获取站房基本信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<PcStationBuilding> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(pcStationBuildingService.getById(id));
    }
    /**
     * 新增站房基本信息
     */
    @ApiOperation(value = "新增站房基本信息")
    @Log(title = "站房基本信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody PcStationBuilding pcStationBuilding) {
        return toAjax(pcStationBuildingService.save(pcStationBuilding));
    }
    /**
     * 修改站房基本信息
     */
    @ApiOperation(value = "修改站房基本信息")
    @Log(title = "站房基本信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody PcStationBuildingVo pcStationBuildingVo) {
        if(ObjectUtil.isEmpty(pcStationBuildingVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(pcStationBuildingVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        PcStationBuilding pcStationBuilding = pcStationBuildingService.getById(pcStationBuildingVo.getId());
        if(ObjectUtil.isEmpty(pcStationBuilding)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", pcStationBuildingVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(pcStationBuildingVo,pcStationBuilding);
        return toAjax(pcStationBuildingService.saveOrUpdate(pcStationBuilding));
    }

    /**
     * 删除站房基本信息
     */
    @ApiOperation(value = "删除站房基本信息")
    @RequiresPermissions("talroad:building:remove")
    @Log(title = "站房基本信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(pcStationBuildingService.removeByIds(Arrays.asList(ids)));
    }
}
