package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.PcStationBuildingEntranceGu;
import cn.cecep.talroad.service.IPcStationBuildingEntranceGuService;
import cn.cecep.talroad.vo.PcStationBuildingEntranceGuVo;
import cn.cecep.talroad.vo.query.PcStationBuildingEntranceGuQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 站房门禁监管信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控--站房门禁监管信息")
@RestController
@RequestMapping("/pcStationBuildingEntranceGu")
public class PcStationBuildingEntranceGuController extends BasesController
{
    @Autowired
    private IPcStationBuildingEntranceGuService pcStationBuildingEntranceGuService;

    /**
     * 查询站房门禁监管信息列表
     */
    @ApiOperation(value = "查询站房门禁监管信息")
    @GetMapping("/list")
    public AjaxResults<IPage<PcStationBuildingEntranceGu>> list(PcStationBuildingEntranceGuQuery pcStationBuildingEntranceGu)
    {
        if (StrUtil.isEmpty(pcStationBuildingEntranceGu.getStationBuildingId())) {
            return AjaxResults.error("查询失败，站点未选择！");
        }
        Page<PcStationBuildingEntranceGu> page = new Page<>(pcStationBuildingEntranceGu.getPageNum(), pcStationBuildingEntranceGu.getPageSize());
        IPage<PcStationBuildingEntranceGu> list = pcStationBuildingEntranceGuService.page(page, PcStationBuildingEntranceGuQuery.createLambdaQueryWrapper(pcStationBuildingEntranceGu));
        return AjaxResults.success(list);
    }


    /**
     * 导出站房门禁监管信息列表
     */
    @ApiOperation(value = "导出站房门禁监管信息")
    @Log(title = "站房门禁监管信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody PcStationBuildingEntranceGuQuery pcStationBuildingEntranceGu)
    {
        List<PcStationBuildingEntranceGu> list = pcStationBuildingEntranceGuService.list(PcStationBuildingEntranceGuQuery.createLambdaQueryWrapper(pcStationBuildingEntranceGu));
        ExcelUtil<PcStationBuildingEntranceGu> util = new ExcelUtil<PcStationBuildingEntranceGu>(PcStationBuildingEntranceGu.class);
        util.exportExcel(response, list, "站房门禁监管信息数据");
    }

    /**
     * 获取站房门禁监管信息详细信息
     */
    @ApiOperation(value = "获取站房门禁监管信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<PcStationBuildingEntranceGu> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(pcStationBuildingEntranceGuService.getById(id));
    }
    /**
     * 新增站房门禁监管信息
     */
    @ApiOperation(value = "新增站房门禁监管信息")
    @Log(title = "站房门禁监管信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody PcStationBuildingEntranceGu pcStationBuildingEntranceGu) {
        return toAjax(pcStationBuildingEntranceGuService.save(pcStationBuildingEntranceGu));
    }
    /**
     * 修改站房门禁监管信息
     */
    @ApiOperation(value = "修改站房门禁监管信息")
    @Log(title = "站房门禁监管信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody PcStationBuildingEntranceGuVo pcStationBuildingEntranceGuVo) {
        if(ObjectUtil.isEmpty(pcStationBuildingEntranceGuVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(pcStationBuildingEntranceGuVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        PcStationBuildingEntranceGu pcStationBuildingEntranceGu = pcStationBuildingEntranceGuService.getById(pcStationBuildingEntranceGuVo.getId());
        if(ObjectUtil.isEmpty(pcStationBuildingEntranceGu)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", pcStationBuildingEntranceGuVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(pcStationBuildingEntranceGuVo,pcStationBuildingEntranceGu);
        return toAjax(pcStationBuildingEntranceGuService.saveOrUpdate(pcStationBuildingEntranceGu));
    }

    /**
     * 删除站房门禁监管信息
     */
    @ApiOperation(value = "删除站房门禁监管信息")
    @RequiresPermissions("talroad:gu:remove")
    @Log(title = "站房门禁监管信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(pcStationBuildingEntranceGuService.removeByIds(Arrays.asList(ids)));
    }
}
