package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.PcStationBuildingOps;
import cn.cecep.talroad.service.IPcStationBuildingOpsService;
import cn.cecep.talroad.vo.PcStationBuildingOpsVo;
import cn.cecep.talroad.vo.query.PcStationBuildingOpsQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 站房运维签到信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控--站房运维签到信息")
@RestController
@RequestMapping("/pcStationBuildingOps")
public class PcStationBuildingOpsController extends BasesController
{
    @Autowired
    private IPcStationBuildingOpsService pcStationBuildingOpsService;

    /**
     * 查询站房运维签到信息列表
     */
    @ApiOperation(value = "查询站房运维签到信息")
    @GetMapping("/list")
    public AjaxResults<IPage<PcStationBuildingOps>> list(PcStationBuildingOpsQuery pcStationBuildingOps)
    {
        if (StrUtil.isEmpty(pcStationBuildingOps.getStationBuildingId())) {
            return AjaxResults.error("查询失败，站点未选择！");
        }
        Page<PcStationBuildingOps> page = new Page<>(pcStationBuildingOps.getPageNum(), pcStationBuildingOps.getPageSize());
        IPage<PcStationBuildingOps> list = pcStationBuildingOpsService.page(page, PcStationBuildingOpsQuery.createLambdaQueryWrapper(pcStationBuildingOps));
        return AjaxResults.success(list);
    }


    /**
     * 导出站房运维签到信息列表
     */
    @ApiOperation(value = "导出站房运维签到信息")
    @Log(title = "站房运维签到信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody PcStationBuildingOpsQuery pcStationBuildingOps)
    {
        List<PcStationBuildingOps> list = pcStationBuildingOpsService.list(PcStationBuildingOpsQuery.createLambdaQueryWrapper(pcStationBuildingOps));
        ExcelUtil<PcStationBuildingOps> util = new ExcelUtil<PcStationBuildingOps>(PcStationBuildingOps.class);
        util.exportExcel(response, list, "站房运维签到信息数据");
    }

    /**
     * 获取站房运维签到信息详细信息
     */
    @ApiOperation(value = "获取站房运维签到信息详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<PcStationBuildingOps> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(pcStationBuildingOpsService.getById(id));
    }
    /**
     * 新增站房运维签到信息
     */
    @ApiOperation(value = "新增站房运维签到信息")
    @Log(title = "站房运维签到信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody PcStationBuildingOps pcStationBuildingOps) {
        return toAjax(pcStationBuildingOpsService.save(pcStationBuildingOps));
    }
    /**
     * 修改站房运维签到信息
     */
    @ApiOperation(value = "修改站房运维签到信息")
    @Log(title = "站房运维签到信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody PcStationBuildingOpsVo pcStationBuildingOpsVo) {
        if(ObjectUtil.isEmpty(pcStationBuildingOpsVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(pcStationBuildingOpsVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        PcStationBuildingOps pcStationBuildingOps = pcStationBuildingOpsService.getById(pcStationBuildingOpsVo.getId());
        if(ObjectUtil.isEmpty(pcStationBuildingOps)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", pcStationBuildingOpsVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(pcStationBuildingOpsVo,pcStationBuildingOps);
        return toAjax(pcStationBuildingOpsService.saveOrUpdate(pcStationBuildingOps));
    }

    /**
     * 删除站房运维签到信息
     */
    @ApiOperation(value = "删除站房运维签到信息")
    @RequiresPermissions("talroad:ops:remove")
    @Log(title = "站房运维签到信息", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(pcStationBuildingOpsService.removeByIds(Arrays.asList(ids)));
    }
}
