package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.PcStationBuildingStatQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.PcStationBuildingStat;
import cn.cecep.talroad.vo.PcStationBuildingStatVo;
import cn.cecep.talroad.service.IPcStationBuildingStatService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 站房信息--站房门禁进出统计分析Controller 控制层
 *
 * @author szhpt
 * @date 2023-06-27*/
@Api(tags = "站房信息--站房门禁进出统计分析")
@RestController
@RequestMapping("/stat")
public class PcStationBuildingStatController extends BasesController
{
    @Autowired
    private IPcStationBuildingStatService pcStationBuildingStatService;

    /**
     * 站房门禁进出统计分析列表
     */
    @ApiOperation(value = "查询站房门禁进出统计分析列表")
    @GetMapping("/page")
    public AjaxResults<IPage<PcStationBuildingStatVo>> page(PcStationBuildingStatQuery query)
    {
        Page<PcStationBuildingStatVo> page = new Page<>(query.getPageNum(), query.getPageSize());
        pcStationBuildingStatService.selectPage(page, query);
        return AjaxResults.success(page);
    }

    /**
     * 查询站房信息--站房门禁进出统计分析列表
     */
    @ApiOperation(value = "查询站房信息--站房门禁进出统计分析")
    @GetMapping("/list")
    public AjaxResults<IPage<PcStationBuildingStat>> list(PcStationBuildingStatQuery pcStationBuildingStat)
    {
        Page<PcStationBuildingStat> page = new Page<>(pcStationBuildingStat.getPageNum(), pcStationBuildingStat.getPageSize());
        IPage<PcStationBuildingStat> list = pcStationBuildingStatService.page(page, PcStationBuildingStatQuery.createLambdaQueryWrapper(pcStationBuildingStat));
        return AjaxResults.success(list);
    }


    /**
     * 导出站房信息--站房门禁进出统计分析列表
     */
    @ApiOperation(value = "导出站房信息--站房门禁进出统计分析")
    @Log(title = "站房信息--站房门禁进出统计分析", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody PcStationBuildingStatQuery query)
    {
        Page<PcStationBuildingStatVo> page = new Page<>(query.getPageNum(), query.getPageSize());
        List<PcStationBuildingStatVo> list = pcStationBuildingStatService.selectPage(page, query);

        ExcelUtil<PcStationBuildingStatVo> util = new ExcelUtil<>(PcStationBuildingStatVo.class);
        util.exportExcel(response, list, "站房信息--站房门禁进出统计分析数据");
    }

    /**
     * 获取站房信息--站房门禁进出统计分析详细信息
     */
    @ApiOperation(value = "获取站房信息--站房门禁进出统计分析详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<PcStationBuildingStat> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(pcStationBuildingStatService.getById(id));
    }
    /**
     * 新增站房信息--站房门禁进出统计分析
     */
    @ApiOperation(value = "新增站房信息--站房门禁进出统计分析")
    @Log(title = "站房信息--站房门禁进出统计分析", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody PcStationBuildingStat pcStationBuildingStat) {
        return toAjax(pcStationBuildingStatService.save(pcStationBuildingStat));
    }
    /**
     * 修改站房信息--站房门禁进出统计分析
     */
    @ApiOperation(value = "修改站房信息--站房门禁进出统计分析")
    @Log(title = "站房信息--站房门禁进出统计分析", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody PcStationBuildingStatVo pcStationBuildingStatVo) {
        if(ObjectUtil.isEmpty(pcStationBuildingStatVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(pcStationBuildingStatVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        PcStationBuildingStat pcStationBuildingStat = pcStationBuildingStatService.getById(pcStationBuildingStatVo.getId());
        if(ObjectUtil.isEmpty(pcStationBuildingStat)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", pcStationBuildingStatVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(pcStationBuildingStatVo,pcStationBuildingStat);
        return toAjax(pcStationBuildingStatService.saveOrUpdate(pcStationBuildingStat));
    }

    /**
     * 删除站房信息--站房门禁进出统计分析
     */
    @ApiOperation(value = "删除站房信息--站房门禁进出统计分析")
    @RequiresPermissions("talroad:stat:remove")
    @Log(title = "站房信息--站房门禁进出统计分析", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable Long[] ids) {
        return toAjax(pcStationBuildingStatService.removeByIds(Arrays.asList(ids)));
    }
}
