package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.utils.DateUtils;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.service.PointIsOfflineService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.Date;

/**
 * @className: PointIsOfflineController
 * @description: 在线监测点位和无组织点位是否离线
 * @author: Mr.Yanghongtao
 * @date: 2023/04/15 5:13 PM
 * @Company: Copyright© [2023/04/15 5:13 PM] by [Mr.Yanghongtao]
 **/
@RestController
@RequestMapping("pointIsOfflineController")
@Slf4j
public class PointIsOfflineController {

    @Autowired
    private PointIsOfflineService pointIsOfflineServiceImpl;


    @ApiOperation(value = "统计无组织点位是否离线，每个小时10分钟执行")
    @GetMapping("/disorganizationPointIsOffline")
    public AjaxResults disorganizationPointIsOffline() {
        Date sTime = new Date();
        log.info("统计无组织点位是否离线（每小时10分钟执行）开始，当前时间：{}", DateUtils.parseDateToStr("yyyy-MM-dd HH:mm:ss",sTime));
        pointIsOfflineServiceImpl.disorganizationPointIsOffline();
        log.info("统计无组织点位是否离线（每小时10分钟执行）结束，当前时间：{},使用耗时：{}秒",DateUtils.parseDateToStr("yyyy-MM-dd HH:mm:ss",new Date()),(new BigDecimal(System.currentTimeMillis() - sTime.getTime()).divide(new BigDecimal(1000),2,BigDecimal.ROUND_HALF_UP)));
        return AjaxResults.success();
    }

    @ApiOperation(value = "统计在线监测点位是否离线，每10分钟执行")
    @GetMapping("/onLineMonitoringPointIsOffline")
    public AjaxResults onLineMonitoringPointIsOffline() {
        Date sTime = new Date();
        log.info("统计在线监测点位是否离线（每10分钟执行）开始，当前时间：{}", DateUtils.parseDateToStr("yyyy-MM-dd HH:mm:ss",sTime));
        pointIsOfflineServiceImpl.onLineMonitoringPointIsOffline();
        log.info("统计在线监测点位是否离线（每10分钟执行）结束，当前时间：{},使用耗时：{}秒",DateUtils.parseDateToStr("yyyy-MM-dd HH:mm:ss",new Date()),(new BigDecimal(System.currentTimeMillis() - sTime.getTime()).divide(new BigDecimal(1000),2,BigDecimal.ROUND_HALF_UP)));
        return AjaxResults.success();
    }

}
