package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.StringUtils;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.SEnvGasMonReal;
import cn.cecep.talroad.service.ISEnvGasMonRealService;
import cn.cecep.talroad.vo.EChartsVO;
import cn.cecep.talroad.vo.SEnvGasMonRealVo;
import cn.cecep.talroad.vo.query.BMainEmissionSourceAirWaterSolidQuery;
import cn.cecep.talroad.vo.query.SEnvGasMonRealQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;


/**
 * * 开发公司：中节能天融科技有限公司
 * 废气监测数据Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "基本档案--废气监测数据")
@RestController
@RequestMapping("/sEnvGasMonReal")
public class SEnvGasMonRealController extends BasesController
{
    @Autowired
    private ISEnvGasMonRealService sEnvGasMonRealService;

    /**
     * 查询企业排放信息-多来源排放量对比-废气列表
     */
    @ApiOperation(value = "废气-废水-固废(图表+列表)-企业排放监控")
    @GetMapping("/chart")
    public AjaxResults<Map<String, Object>> chart(@Valid BMainEmissionSourceAirWaterSolidQuery query)
    {
        return AjaxResults.success(sEnvGasMonRealService.chartNew(query));
    }

    /**
     * 查询废气监测数据列表
     */
    @ApiOperation(value = "查询废气监测数据")
    @GetMapping("/list")
    public AjaxResults<IPage<SEnvGasMonRealVo>> list(SEnvGasMonRealQuery sEnvGasMonReal)
    {
        if (StringUtils.isEmpty(sEnvGasMonReal.getDataType())) {
            throw new InvalidParameterException("时间类型参数为空");
        }
        IPage<SEnvGasMonRealVo> list = sEnvGasMonRealService.selectSEnvGasMonRealPage(sEnvGasMonReal);
        return AjaxResults.success(list);
    }

    @ApiOperation(value = "查询废气监测曲线图数据")
    @GetMapping("/charData")
    public AjaxResults<EChartsVO> getCharData(SEnvGasMonRealQuery sEnvGasMonReal)
    {
        EChartsVO charData = sEnvGasMonRealService.getCharData(sEnvGasMonReal);
        return AjaxResults.success(charData);
    }

    @ApiOperation(value = "查询废气排口下所有监测点位的曲线图数据")
    @GetMapping("/charDataList")
    public AjaxResults<List<EChartsVO>> charDataList(SEnvGasMonRealQuery sEnvGasMonReal)
    {
        List<EChartsVO> chartsVO = sEnvGasMonRealService.getCharDataList(sEnvGasMonReal);
        return AjaxResults.success(chartsVO);
    }

    /**
     * 导出废气监测数据列表
     */
    @ApiOperation(value = "导出废气监测数据")
    @Log(title = "废气监测数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SEnvGasMonRealQuery sEnvGasMonReal)
    {
        List<SEnvGasMonRealVo> list = sEnvGasMonRealService.selectSEnvGasMonRealList(sEnvGasMonReal);
        ExcelUtil<SEnvGasMonRealVo> util = new ExcelUtil<>(SEnvGasMonRealVo.class);
        util.exportExcel(response, list, "废气监测数据数据");
    }

    /**
     * 获取废气监测数据详细信息
     */
    @ApiOperation(value = "获取废气监测数据详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<SEnvGasMonReal> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sEnvGasMonRealService.getById(id));
    }
    /**
     * 新增废气监测数据
     */
    @ApiOperation(value = "新增废气监测数据")
    @Log(title = "废气监测数据", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SEnvGasMonReal sEnvGasMonReal) {
        return toAjax(sEnvGasMonRealService.save(sEnvGasMonReal));
    }
    /**
     * 修改废气监测数据
     */
    @ApiOperation(value = "修改废气监测数据")
    @Log(title = "废气监测数据", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SEnvGasMonRealVo sEnvGasMonRealVo) {
        if(ObjectUtil.isEmpty(sEnvGasMonRealVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(sEnvGasMonRealVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        SEnvGasMonReal sEnvGasMonReal = sEnvGasMonRealService.getById(sEnvGasMonRealVo.getId());
        if(ObjectUtil.isEmpty(sEnvGasMonReal)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sEnvGasMonRealVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(sEnvGasMonRealVo,sEnvGasMonReal);
        return toAjax(sEnvGasMonRealService.saveOrUpdate(sEnvGasMonReal));
    }

    /**
     * 删除废气监测数据
     */
    @ApiOperation(value = "删除废气监测数据")
    @RequiresPermissions("talroad:real:remove")
    @Log(title = "废气监测数据", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(sEnvGasMonRealService.removeByIds(Arrays.asList(ids)));
    }

}
