package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.StringUtils;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.SEnvGasReal;
import cn.cecep.talroad.service.ISEnvGasRealService;
import cn.cecep.talroad.vo.EChartsVO;
import cn.cecep.talroad.vo.SEnvGasRealVo;
import cn.cecep.talroad.vo.query.SEnvGasRealQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 无组织监控数据Controller 控制层
 *
 * @author szhpt
 * @date 2023-01-10*/
@Api(tags = "监测监控--无组织监控数据")
@RestController
@RequestMapping("/sEnvGasReal")
public class SEnvGasRealController extends BasesController
{
    @Autowired
    private ISEnvGasRealService sEnvGasRealService;

    /**
     * 查询无组织监控数据列表
     */
    @ApiOperation(value = "查询无组织监控数据")
    @GetMapping("/list")
    public AjaxResults<IPage<SEnvGasReal>> list(SEnvGasRealQuery sEnvGasReal)
    {
        if (StringUtils.isEmpty(sEnvGasReal.getDataType())) {
            throw new InvalidParameterException("时间类型参数为空");
        }

        Page<SEnvGasReal> page = new Page<>(sEnvGasReal.getPageNum(), sEnvGasReal.getPageSize());
        IPage<SEnvGasReal> list = sEnvGasRealService.page(page, SEnvGasRealQuery.createLambdaQueryWrapper(sEnvGasReal));
        return AjaxResults.success(list);
    }

    @ApiOperation(value = "查询无组织监控曲线图数据")
    @GetMapping("/charData")
    public AjaxResults<EChartsVO> getCharData(SEnvGasRealQuery sEnvGasReal)
    {
        EChartsVO charData = sEnvGasRealService.getCharData(sEnvGasReal);
        return AjaxResults.success(charData);
    }

    /**
     * 导出无组织监控数据列表
     */
    @ApiOperation(value = "导出无组织监控数据")
    @Log(title = "无组织监控数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SEnvGasRealQuery sEnvGasReal)
    {
        List<SEnvGasReal> list = sEnvGasRealService.list(SEnvGasRealQuery.createLambdaQueryWrapper(sEnvGasReal));
        ExcelUtil<SEnvGasReal> util = new ExcelUtil<SEnvGasReal>(SEnvGasReal.class);
        util.exportExcel(response, list, "无组织监控数据数据");
    }

    /**
     * 获取无组织监控数据详细信息
     */
    @ApiOperation(value = "获取无组织监控数据详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<SEnvGasReal> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sEnvGasRealService.getById(id));
    }
    /**
     * 新增无组织监控数据
     */
    @ApiOperation(value = "新增无组织监控数据")
    @Log(title = "无组织监控数据", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SEnvGasReal sEnvGasReal) {
        return toAjax(sEnvGasRealService.save(sEnvGasReal));
    }
    /**
     * 修改无组织监控数据
     */
    @ApiOperation(value = "修改无组织监控数据")
    @Log(title = "无组织监控数据", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SEnvGasRealVo sEnvGasRealVo) {
        if(ObjectUtil.isEmpty(sEnvGasRealVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(sEnvGasRealVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        SEnvGasReal sEnvGasReal = sEnvGasRealService.getById(sEnvGasRealVo.getId());
        if(ObjectUtil.isEmpty(sEnvGasReal)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sEnvGasRealVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(sEnvGasRealVo,sEnvGasReal);
        return toAjax(sEnvGasRealService.saveOrUpdate(sEnvGasReal));
    }

    /**
     * 删除无组织监控数据
     */
    @ApiOperation(value = "删除无组织监控数据")
    @RequiresPermissions("talroad:real:remove")
    @Log(title = "无组织监控数据", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(sEnvGasRealService.removeByIds(Arrays.asList(ids)));
    }
}
