package cn.cecep.talroad.controller;

import cn.cecep.talroad.common.core.domain.R;
import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.security.utils.SecurityUtils;
import cn.cecep.talroad.feign.BaseRegionService;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.SRaRealTimeAnalysisRunOneQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.SRaRealTimeAnalysisRunOne;
import cn.cecep.talroad.vo.SRaRealTimeAnalysisRunOneVo;
import cn.cecep.talroad.service.ISRaRealTimeAnalysisRunOneService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据Controller 控制层
 *
 * @author szhpt
 * @date 2023-05-17*/
@Api(tags = "关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据")
@RestController
@RequestMapping("/runone")
public class SRaRealTimeAnalysisRunOneController extends BasesController
{
    @Autowired
    private ISRaRealTimeAnalysisRunOneService sRaRealTimeAnalysisRunOneService;

    @Autowired
//    @Qualifier("baseRegionService")
    private BaseRegionService baseRegionService;
    /**
     * 查询关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据列表
     */
    @ApiOperation(value = "查询关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaRealTimeAnalysisRunOne>> list(SRaRealTimeAnalysisRunOneQuery sRaRealTimeAnalysisRunOne)
    {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();

        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisRunOne.setRegionCodes(regionCodes);
        IPage<SRaRealTimeAnalysisRunOne> list = sRaRealTimeAnalysisRunOneService.selectSRaRealTimeAnalysisRunOneListPage(sRaRealTimeAnalysisRunOne);
        return AjaxResults.success(list);
    }
    

    /**
     * 导出关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据列表
     */
    @ApiOperation(value = "导出关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据")
    @Log(title = "关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SRaRealTimeAnalysisRunOneQuery sRaRealTimeAnalysisRunOne)
    {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();

        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisRunOne.setRegionCodes(regionCodes);
        List<SRaRealTimeAnalysisRunOne> list = sRaRealTimeAnalysisRunOneService.selectSRaRealTimeAnalysisRunOneList(sRaRealTimeAnalysisRunOne);
        ExcelUtil<SRaRealTimeAnalysisRunOne> util = new ExcelUtil<SRaRealTimeAnalysisRunOne>(SRaRealTimeAnalysisRunOne.class);
        util.exportExcel(response, list, "关联分析系统-企业生产过程分析-企业装备运行情况实时分析数据");
    }

    /**
     * 获取关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据详细信息
     */
    @ApiOperation(value = "获取关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<SRaRealTimeAnalysisRunOne> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaRealTimeAnalysisRunOneService.getById(id));
    }
    /**
     * 新增关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据
     */
    @ApiOperation(value = "新增关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据")
    @Log(title = "关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaRealTimeAnalysisRunOne sRaRealTimeAnalysisRunOne) {
        return toAjax(sRaRealTimeAnalysisRunOneService.save(sRaRealTimeAnalysisRunOne));
    }
    /**
     * 修改关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据
     */
    @ApiOperation(value = "修改关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据")
    @Log(title = "关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaRealTimeAnalysisRunOneVo sRaRealTimeAnalysisRunOneVo) {
        if(ObjectUtil.isEmpty(sRaRealTimeAnalysisRunOneVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(sRaRealTimeAnalysisRunOneVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        SRaRealTimeAnalysisRunOne sRaRealTimeAnalysisRunOne = sRaRealTimeAnalysisRunOneService.getById(sRaRealTimeAnalysisRunOneVo.getDataId());
        if(ObjectUtil.isEmpty(sRaRealTimeAnalysisRunOne)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaRealTimeAnalysisRunOneVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaRealTimeAnalysisRunOneVo,sRaRealTimeAnalysisRunOne);
        return toAjax(sRaRealTimeAnalysisRunOneService.saveOrUpdate(sRaRealTimeAnalysisRunOne));
    }

    /**
     * 删除关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据
     */
    @ApiOperation(value = "删除关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据")
    @RequiresPermissions("talroad:one:remove")
    @Log(title = "关联分析系统-企业生产过程分析-企业装备运行情况实时分析每个设备只存一条数据", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable String[] dataIds) {
        return toAjax(sRaRealTimeAnalysisRunOneService.removeByIds(Arrays.asList(dataIds)));
    }
}
