package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.analysis.SRaAbNalysisCar;
import cn.cecep.talroad.service.analysis.ISRaAbNalysisCarService;
import cn.cecep.talroad.vo.analysis.SRaAbNalysisCarVo;
import cn.cecep.talroad.vo.query.analysis.SRaAbNalysisCarQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 管控期车辆异常分析Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-24
 */
@Api(tags = "管控期车辆异常分析")
@RestController
@RequestMapping("/car")
public class SRaAbNalysisCarController extends BasesController {
    @Autowired
    private ISRaAbNalysisCarService sRaAbNalysisCarService;

    /**
     * 查询管控期车辆异常分析列表
     */
    @ApiOperation(value = "查询管控期车辆异常分析")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaAbNalysisCar>> list(SRaAbNalysisCarQuery sRaAbNalysisCar) {
        Page<SRaAbNalysisCar> page = new Page<>(sRaAbNalysisCar.getPageNum(), sRaAbNalysisCar.getPageSize());
        IPage<SRaAbNalysisCar> list = sRaAbNalysisCarService.page(page, SRaAbNalysisCarQuery.createLambdaQueryWrapper(sRaAbNalysisCar));
        return AjaxResults.success(list);
    }


    /**
     * 导出管控期车辆异常分析列表
     */
    @ApiOperation(value = "导出管控期车辆异常分析")
    @Log(title = "管控期车辆异常分析", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SRaAbNalysisCar sRaAbNalysisCar) {
        List<SRaAbNalysisCar> list = sRaAbNalysisCarService.selectSRaAbNalysisCarList(sRaAbNalysisCar);
        ExcelUtil<SRaAbNalysisCar> util = new ExcelUtil<SRaAbNalysisCar>(SRaAbNalysisCar.class);
        util.exportExcel(response, list, "管控期车辆异常分析数据");
    }

    /**
     * 获取管控期车辆异常分析详细信息
     */
    @ApiOperation(value = "获取管控期车辆异常分析详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<SRaAbNalysisCar> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaAbNalysisCarService.getById(id));
    }

    /**
     * 新增管控期车辆异常分析
     */
    @ApiOperation(value = "新增管控期车辆异常分析")
    @Log(title = "管控期车辆异常分析", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaAbNalysisCar sRaAbNalysisCar) {
        return toAjax(sRaAbNalysisCarService.save(sRaAbNalysisCar));
    }

    /**
     * 修改管控期车辆异常分析
     */
    @ApiOperation(value = "修改管控期车辆异常分析")
    @Log(title = "管控期车辆异常分析", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaAbNalysisCarVo sRaAbNalysisCarVo) {
        if (ObjectUtil.isEmpty(sRaAbNalysisCarVo)) {
            throw new InvalidParameterException("参数为空");
        }
        if (ObjectUtil.isEmpty(sRaAbNalysisCarVo.getDataId())) {
            throw new InvalidParameterException("参数id为空");
        }
        SRaAbNalysisCar sRaAbNalysisCar = sRaAbNalysisCarService.getById(sRaAbNalysisCarVo.getDataId());
        if (ObjectUtil.isEmpty(sRaAbNalysisCar)) {
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaAbNalysisCarVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaAbNalysisCarVo, sRaAbNalysisCar);
        return toAjax(sRaAbNalysisCarService.saveOrUpdate(sRaAbNalysisCar));
    }

    /**
     * 删除管控期车辆异常分析
     */
    @ApiOperation(value = "删除管控期车辆异常分析")
    @RequiresPermissions("talroad:car:remove")
    @Log(title = "管控期车辆异常分析", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable String[] dataIds) {
        return toAjax(sRaAbNalysisCarService.removeByIds(Arrays.asList(dataIds)));
    }
}
