package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.analysis.SRaActionReportsActualEmissions;
import cn.cecep.talroad.service.analysis.ISRaActionReportsActualEmissionsService;
import cn.cecep.talroad.vo.analysis.SRaActionReportsActualEmissionsVo;
import cn.cecep.talroad.vo.query.analysis.SRaActionReportsActualEmissionsQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 执行报告实际排放情况Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-24
 */
@Api(tags = "执行报告实际排放情况")
@RestController
@RequestMapping("/emissions")
public class SRaActionReportsActualEmissionsController extends BasesController {
    @Autowired
    private ISRaActionReportsActualEmissionsService sRaActionReportsActualEmissionsService;

    /**
     * 查询执行报告实际排放情况列表
     */
    @ApiOperation(value = "查询执行报告实际排放情况")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaActionReportsActualEmissions>> list(SRaActionReportsActualEmissionsQuery sRaActionReportsActualEmissions) {
        Page<SRaActionReportsActualEmissions> page = new Page<>(sRaActionReportsActualEmissions.getPageNum(), sRaActionReportsActualEmissions.getPageSize());
        IPage<SRaActionReportsActualEmissions> list = sRaActionReportsActualEmissionsService.selectSRaActionReportsActualEmissionsPage(page, SRaActionReportsActualEmissionsQuery.createLambdaQueryWrapper(sRaActionReportsActualEmissions));
        return AjaxResults.success(list);
    }


    /**
     * 导出执行报告实际排放情况列表
     */
    @ApiOperation(value = "导出执行报告实际排放情况")
    @Log(title = "执行报告实际排放情况", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SRaActionReportsActualEmissionsQuery sRaActionReportsActualEmissions) {
        List<SRaActionReportsActualEmissions> list = sRaActionReportsActualEmissionsService.selectSRaActionReportsActualEmissionsList(SRaActionReportsActualEmissionsQuery.createLambdaQueryWrapper(sRaActionReportsActualEmissions));
        ExcelUtil<SRaActionReportsActualEmissions> util = new ExcelUtil<SRaActionReportsActualEmissions>(SRaActionReportsActualEmissions.class);
        util.exportExcel(response, list, "执行报告实际排放情况数据");
    }

    /**
     * 获取执行报告实际排放情况详细信息
     */
    @ApiOperation(value = "获取执行报告实际排放情况详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<SRaActionReportsActualEmissions> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaActionReportsActualEmissionsService.getById(id));
    }

    /**
     * 新增执行报告实际排放情况
     */
    @ApiOperation(value = "新增执行报告实际排放情况")
    @Log(title = "执行报告实际排放情况", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaActionReportsActualEmissions sRaActionReportsActualEmissions) {
        return toAjax(sRaActionReportsActualEmissionsService.save(sRaActionReportsActualEmissions));
    }

    /**
     * 修改执行报告实际排放情况
     */
    @ApiOperation(value = "修改执行报告实际排放情况")
    @Log(title = "执行报告实际排放情况", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaActionReportsActualEmissionsVo sRaActionReportsActualEmissionsVo) {
        if (ObjectUtil.isEmpty(sRaActionReportsActualEmissionsVo)) {
            throw new InvalidParameterException("参数为空");
        }
        if (ObjectUtil.isEmpty(sRaActionReportsActualEmissionsVo.getId())) {
            throw new InvalidParameterException("参数id为空");
        }
        SRaActionReportsActualEmissions sRaActionReportsActualEmissions = sRaActionReportsActualEmissionsService.getById(sRaActionReportsActualEmissionsVo.getId());
        if (ObjectUtil.isEmpty(sRaActionReportsActualEmissions)) {
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaActionReportsActualEmissionsVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaActionReportsActualEmissionsVo, sRaActionReportsActualEmissions);
        return toAjax(sRaActionReportsActualEmissionsService.saveOrUpdate(sRaActionReportsActualEmissions));
    }

    /**
     * 删除执行报告实际排放情况
     */
    @ApiOperation(value = "删除执行报告实际排放情况")
    @RequiresPermissions("talroad:emissions:remove")
    @Log(title = "执行报告实际排放情况", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(sRaActionReportsActualEmissionsService.removeByIds(Arrays.asList(ids)));
    }
}
