package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.analysis.SRaActionReportsActualYield;
import cn.cecep.talroad.service.analysis.ISRaActionReportsActualYieldService;
import cn.cecep.talroad.vo.analysis.SRaActionReportsActualYieldVo;
import cn.cecep.talroad.vo.query.analysis.SRaActionReportsActualYieldQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 执行报告实际产量情况Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-24
 */
@Api(tags = "执行报告实际产量情况")
@RestController
@RequestMapping("/yield")
public class SRaActionReportsActualYieldController extends BasesController {
    @Autowired
    private ISRaActionReportsActualYieldService sRaActionReportsActualYieldService;

    /**
     * 查询执行报告实际产量情况列表
     */
    @ApiOperation(value = "查询执行报告实际产量情况")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaActionReportsActualYield>> list(SRaActionReportsActualYieldQuery sRaActionReportsActualYield) {
        Page<SRaActionReportsActualYield> page = new Page<>(sRaActionReportsActualYield.getPageNum(), sRaActionReportsActualYield.getPageSize());
        IPage<SRaActionReportsActualYield> list = sRaActionReportsActualYieldService.selectSRaActionReportsActualYieldPage(page, SRaActionReportsActualYieldQuery.createLambdaQueryWrapper(sRaActionReportsActualYield));
        return AjaxResults.success(list);
    }


    /**
     * 导出执行报告实际产量情况列表
     */
    @ApiOperation(value = "导出执行报告实际产量情况")
    @Log(title = "执行报告实际产量情况", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SRaActionReportsActualYieldQuery sRaActionReportsActualYield) {
        List<SRaActionReportsActualYield> list = sRaActionReportsActualYieldService.selectSRaActionReportsActualYieldList(SRaActionReportsActualYieldQuery.createLambdaQueryWrapper(sRaActionReportsActualYield));
        ExcelUtil<SRaActionReportsActualYield> util = new ExcelUtil<SRaActionReportsActualYield>(SRaActionReportsActualYield.class);
        util.exportExcel(response, list, "执行报告实际产量情况数据");
    }

    /**
     * 获取执行报告实际产量情况详细信息
     */
    @ApiOperation(value = "获取执行报告实际产量情况详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<SRaActionReportsActualYield> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaActionReportsActualYieldService.getById(id));
    }

    /**
     * 新增执行报告实际产量情况
     */
    @ApiOperation(value = "新增执行报告实际产量情况")
    @Log(title = "执行报告实际产量情况", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaActionReportsActualYield sRaActionReportsActualYield) {
        return toAjax(sRaActionReportsActualYieldService.save(sRaActionReportsActualYield));
    }

    /**
     * 修改执行报告实际产量情况
     */
    @ApiOperation(value = "修改执行报告实际产量情况")
    @Log(title = "执行报告实际产量情况", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaActionReportsActualYieldVo sRaActionReportsActualYieldVo) {
        if (ObjectUtil.isEmpty(sRaActionReportsActualYieldVo)) {
            throw new InvalidParameterException("参数为空");
        }
        if (ObjectUtil.isEmpty(sRaActionReportsActualYieldVo.getId())) {
            throw new InvalidParameterException("参数id为空");
        }
        SRaActionReportsActualYield sRaActionReportsActualYield = sRaActionReportsActualYieldService.getById(sRaActionReportsActualYieldVo.getId());
        if (ObjectUtil.isEmpty(sRaActionReportsActualYield)) {
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaActionReportsActualYieldVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaActionReportsActualYieldVo, sRaActionReportsActualYield);
        return toAjax(sRaActionReportsActualYieldService.saveOrUpdate(sRaActionReportsActualYield));
    }

    /**
     * 删除执行报告实际产量情况
     */
    @ApiOperation(value = "删除执行报告实际产量情况")
    @RequiresPermissions("talroad:yield:remove")
    @Log(title = "执行报告实际产量情况", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(sRaActionReportsActualYieldService.removeByIds(Arrays.asList(ids)));
    }
}
