package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.core.web.page.PageData;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.analysis.SRaActionReportsAuditRecord;
import cn.cecep.talroad.domain.dto.AuditRecordDTO;
import cn.cecep.talroad.domain.dto.RecordListDTO;
import cn.cecep.talroad.domain.vo.AuditRecordLIstVO;
import cn.cecep.talroad.service.analysis.ISRaActionReportsAuditRecordService;
import cn.cecep.talroad.vo.analysis.SRaActionReportsAuditRecordVo;
import cn.cecep.talroad.vo.query.analysis.SRaActionReportsAuditRecordQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 执行报告规范性审核记录Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-24
 */
@Api(tags = "执行报告规范性审核记录")
@RestController
@RequestMapping("/record")
public class SRaActionReportsAuditRecordController extends BasesController {
    @Autowired
    private ISRaActionReportsAuditRecordService sRaActionReportsAuditRecordService;

    /**
     * 查询执行报告规范性审核记录列表
     */
    @ApiOperation(value = "查询执行报告规范性审核记录列表")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaActionReportsAuditRecord>> list(SRaActionReportsAuditRecordQuery sRaActionReportsAuditRecord) {
        Page<SRaActionReportsAuditRecord> page = new Page<>(sRaActionReportsAuditRecord.getPageNum(), sRaActionReportsAuditRecord.getPageSize());
        IPage<SRaActionReportsAuditRecord> list = sRaActionReportsAuditRecordService.page(page,
                SRaActionReportsAuditRecordQuery.createLambdaQueryWrapper(sRaActionReportsAuditRecord).orderByDesc("update_time","create_time"));
        return AjaxResults.success(list);
    }


    /**
     * 导出执行报告规范性审核记录列表
     */
    @ApiOperation(value = "导出执行报告规范性审核记录")
    @Log(title = "执行报告规范性审核记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SRaActionReportsAuditRecord sRaActionReportsAuditRecord) {
        List<SRaActionReportsAuditRecord> list = sRaActionReportsAuditRecordService.selectSRaActionReportsAuditRecordList(sRaActionReportsAuditRecord);
        ExcelUtil<SRaActionReportsAuditRecord> util = new ExcelUtil<SRaActionReportsAuditRecord>(SRaActionReportsAuditRecord.class);
        util.exportExcel(response, list, "执行报告规范性审核记录数据");
    }

    /**
     * 获取执行报告规范性审核记录详细信息
     */
    @ApiOperation(value = "获取执行报告规范性审核记录详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<SRaActionReportsAuditRecord> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaActionReportsAuditRecordService.getById(id));
    }

    /**
     * 新增执行报告规范性审核记录
     */
    @ApiOperation(value = "新增执行报告规范性审核记录")
    @Log(title = "执行报告规范性审核记录", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody AuditRecordDTO auditRecordDTO) {
        return sRaActionReportsAuditRecordService.saveRecord(auditRecordDTO);
    }

    /**
     * 修改执行报告规范性审核记录
     */
    @ApiOperation(value = "修改执行报告规范性审核记录")
    @Log(title = "执行报告规范性审核记录", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaActionReportsAuditRecordVo sRaActionReportsAuditRecordVo) {
        if (ObjectUtil.isEmpty(sRaActionReportsAuditRecordVo)) {
            throw new InvalidParameterException("参数为空");
        }
        if (ObjectUtil.isEmpty(sRaActionReportsAuditRecordVo.getId())) {
            throw new InvalidParameterException("参数id为空");
        }
        SRaActionReportsAuditRecord sRaActionReportsAuditRecord = sRaActionReportsAuditRecordService.getById(sRaActionReportsAuditRecordVo.getId());
        if (ObjectUtil.isEmpty(sRaActionReportsAuditRecord)) {
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaActionReportsAuditRecordVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaActionReportsAuditRecordVo, sRaActionReportsAuditRecord);
        return toAjax(sRaActionReportsAuditRecordService.saveOrUpdate(sRaActionReportsAuditRecord));
    }

    /**
     * 删除执行报告规范性审核记录
     */
    @ApiOperation(value = "删除执行报告规范性审核记录")
    @RequiresPermissions("talroad:record:remove")
    @Log(title = "执行报告规范性审核记录", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(sRaActionReportsAuditRecordService.removeByIds(Arrays.asList(ids)));
    }


    /**
     * 执行报告规范性审核记录添加查询企业列表
     */
    @ApiOperation(value = "执行报告规范性审核记录添加查询企业列表")
    @Log(title = "执行报告规范性审核记录添加查询企业列表")
    @PostMapping("/recordList")
    public AjaxResults<Page<AuditRecordLIstVO>> recordList(@RequestBody RecordListDTO recordListDTO) {
        return sRaActionReportsAuditRecordService.recordList(recordListDTO);
    }
}
