package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.analysis.SRaActionReportsAuditResult;
import cn.cecep.talroad.service.analysis.ISRaActionReportsAuditResultService;
import cn.cecep.talroad.vo.analysis.SRaActionReportsAuditResultVo;
import cn.cecep.talroad.vo.query.analysis.SRaActionReportsAuditResultQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import freemarker.template.TemplateException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * * 开发公司：中节能天融科技有限公司
 * 执行报告规范性审核结果Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-24
 */
@Api(tags = "执行报告规范性审核结果")
@RestController
@RequestMapping("/result")
public class SRaActionReportsAuditResultController extends BasesController {
    @Autowired
    private ISRaActionReportsAuditResultService sRaActionReportsAuditResultService;

    /**
     * 查询执行报告规范性审核结果列表
     */
    @ApiOperation(value = "查询执行报告规范性审核结果")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaActionReportsAuditResult>> list(SRaActionReportsAuditResultQuery sRaActionReportsAuditResult) {
        Page<SRaActionReportsAuditResult> page = new Page<>(sRaActionReportsAuditResult.getPageNum(), sRaActionReportsAuditResult.getPageSize());
        IPage<SRaActionReportsAuditResult> list = sRaActionReportsAuditResultService.resultList(page,
                SRaActionReportsAuditResultQuery.createLambdaQueryWrapper(sRaActionReportsAuditResult));
        return AjaxResults.success(list);
    }


    /**
     * 导出执行报告规范性审核结果列表
     */
    @ApiOperation(value = "导出执行报告规范性审核结果")
    @Log(title = "执行报告规范性审核结果", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SRaActionReportsAuditResult sRaActionReportsAuditResult) {
        List<SRaActionReportsAuditResult> list = sRaActionReportsAuditResultService.selectSRaActionReportsAuditResultList(sRaActionReportsAuditResult);
        ExcelUtil<SRaActionReportsAuditResult> util = new ExcelUtil<>(SRaActionReportsAuditResult.class);
        util.exportExcel(response, list, "执行报告规范性审核结果数据","排污许可证执行报告内容规范性审核表");
    }

    /**
     * 获取执行报告规范性审核结果详细信息
     */
    @ApiOperation(value = "获取执行报告规范性审核结果详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<SRaActionReportsAuditResult> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaActionReportsAuditResultService.getById(id));
    }

    /**
     * 新增执行报告规范性审核结果
     */
    @ApiOperation(value = "新增执行报告规范性审核结果")
    @Log(title = "执行报告规范性审核结果", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaActionReportsAuditResult sRaActionReportsAuditResult) {
        return toAjax(sRaActionReportsAuditResultService.save(sRaActionReportsAuditResult));
    }

    /**
     * 修改执行报告规范性审核结果
     */
    @ApiOperation(value = "修改执行报告规范性审核结果")
    @Log(title = "执行报告规范性审核结果", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaActionReportsAuditResultVo sRaActionReportsAuditResultVo) {
        if (ObjectUtil.isEmpty(sRaActionReportsAuditResultVo)) {
            throw new InvalidParameterException("参数为空");
        }
        if (ObjectUtil.isEmpty(sRaActionReportsAuditResultVo.getId())) {
            throw new InvalidParameterException("参数id为空");
        }
        SRaActionReportsAuditResult sRaActionReportsAuditResult = sRaActionReportsAuditResultService.getById(sRaActionReportsAuditResultVo.getId());
        if (ObjectUtil.isEmpty(sRaActionReportsAuditResult)) {
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaActionReportsAuditResultVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaActionReportsAuditResultVo, sRaActionReportsAuditResult);
        return toAjax(sRaActionReportsAuditResultService.saveOrUpdate(sRaActionReportsAuditResult));
    }

    /**
     * 删除执行报告规范性审核结果
     */
    @ApiOperation(value = "删除执行报告规范性审核结果")
    @RequiresPermissions("talroad:result:remove")
    @Log(title = "执行报告规范性审核结果", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(sRaActionReportsAuditResultService.removeByIds(Arrays.asList(ids)));
    }

    /**
     * 执行报告规范性审核结果企业审核表下载
     */
    @ApiOperation(value = "执行报告规范性审核结果企业审核表下载")
    @Log(title = "执行报告规范性审核结果企业审核表下载", businessType = BusinessType.EXPORT)
    @PostMapping("/getResultFactoryDetailWord/{id}")
    public void getResultFactoryDetailWord(@PathVariable String id,HttpServletResponse response) throws IOException, TemplateException, IllegalAccessException {
        sRaActionReportsAuditResultService.getResultFactoryDetailWord(id,response);
    }


    /**
     * 发送问题给该企业
     */
    @ApiOperation(value = "发送问题给该企业")
    @Log(title = "发送问题给该企业", businessType = BusinessType.EXPORT)
    @PostMapping("/sendQuestion/{id}")
    public AjaxResults<String> sendQuestion(@PathVariable String id){
        return sRaActionReportsAuditResultService.sendQuestion(id);
    }
}
