package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.analysis.SRaAreaRegulatoryIssues;
import cn.cecep.talroad.service.analysis.ISRaAreaRegulatoryIssuesService;
import cn.cecep.talroad.vo.analysis.SRaAreaRegulatoryIssuesVo;
import cn.cecep.talroad.vo.query.analysis.SRaAreaRegulatoryIssuesQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 区域证后监管问题总表Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-24
 */
@Api(tags = "区域证后监管问题总表")
@RestController
@RequestMapping("/area/issues")
public class SRaAreaRegulatoryIssuesController extends BasesController {
    @Autowired
    private ISRaAreaRegulatoryIssuesService sRaAreaRegulatoryIssuesService;

    /**
     * 区域证后监管问题总表列表
     */
    @ApiOperation(value = "查询区域证后监管问题总表")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaAreaRegulatoryIssues>> list(SRaAreaRegulatoryIssuesQuery sRaAreaRegulatoryIssues) {
        Page<SRaAreaRegulatoryIssues> page = new Page<>(sRaAreaRegulatoryIssues.getPageNum(), sRaAreaRegulatoryIssues.getPageSize());
        IPage<SRaAreaRegulatoryIssues> list = sRaAreaRegulatoryIssuesService.page(page, SRaAreaRegulatoryIssuesQuery.createLambdaQueryWrapper(sRaAreaRegulatoryIssues));
        return AjaxResults.success(list);
    }


    /**
     * 导出区域证后监管问题总表列表
     */
    @ApiOperation(value = "导出区域证后监管问题总表")
    @Log(title = "导出区域证后监管问题总表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SRaAreaRegulatoryIssuesQuery sRaAreaRegulatoryIssues) {
        List<SRaAreaRegulatoryIssues> list = sRaAreaRegulatoryIssuesService.list(SRaAreaRegulatoryIssuesQuery.createLambdaQueryWrapper(sRaAreaRegulatoryIssues));
        ExcelUtil<SRaAreaRegulatoryIssues> util = new ExcelUtil<SRaAreaRegulatoryIssues>(SRaAreaRegulatoryIssues.class);
        util.exportExcel(response, list, "区域证后监管问题总表数据");
    }

    /**
     * 获取区域证后监管问题总表详细信息
     */
    @ApiOperation(value = "获取区域证后监管问题总表详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<SRaAreaRegulatoryIssues> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaAreaRegulatoryIssuesService.getById(id));
    }

    /**
     * 新增区域证后监管问题总表
     */
    @ApiOperation(value = "新增区域证后监管问题总表")
    @Log(title = "新增区域证后监管问题总表", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaAreaRegulatoryIssues sRaAreaRegulatoryIssues) {
        return toAjax(sRaAreaRegulatoryIssuesService.save(sRaAreaRegulatoryIssues));
    }

    /**
     * 修改区域证后监管问题总表
     */
    @ApiOperation(value = "修改区域证后监管问题总表")
    @Log(title = "修改区域证后监管问题总表", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaAreaRegulatoryIssuesVo sRaAreaRegulatoryIssuesVo) {
        if (ObjectUtil.isEmpty(sRaAreaRegulatoryIssuesVo)) {
            throw new InvalidParameterException("参数为空");
        }
        if (ObjectUtil.isEmpty(sRaAreaRegulatoryIssuesVo.getId())) {
            throw new InvalidParameterException("参数id为空");
        }
        SRaAreaRegulatoryIssues sRaAreaRegulatoryIssues = sRaAreaRegulatoryIssuesService.getById(sRaAreaRegulatoryIssuesVo.getId());
        if (ObjectUtil.isEmpty(sRaAreaRegulatoryIssues)) {
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaAreaRegulatoryIssuesVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaAreaRegulatoryIssuesVo, sRaAreaRegulatoryIssues);
        return toAjax(sRaAreaRegulatoryIssuesService.saveOrUpdate(sRaAreaRegulatoryIssues));
    }

    /**
     * 删除区域证后监管问题总表
     */
    @ApiOperation(value = "删除区域证后监管问题总表")
    @RequiresPermissions("talroad:issues:remove")
    @Log(title = "删除区域证后监管问题总表", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(sRaAreaRegulatoryIssuesService.removeByIds(Arrays.asList(ids)));
    }
}
