package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.analysis.SRaAreaTransferRateAnalysis;
import cn.cecep.talroad.service.analysis.ISRaAreaTransferRateAnalysisService;
import cn.cecep.talroad.vo.analysis.SRaAreaTransferRateAnalysisVo;
import cn.cecep.talroad.vo.query.analysis.SRaAreaTransferRateAnalysisQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 区域传输率分析Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-24
 */
@Api(tags = "区域传输率分析")
@RestController
@RequestMapping("/area/analysis")
public class SRaAreaTransferRateAnalysisController extends BasesController {

    @Autowired
    private ISRaAreaTransferRateAnalysisService sRaAreaTransferRateAnalysisService;

    /**
     * 查询区域传输率分析列表
     */
    @ApiOperation(value = "查询区域传输率分析")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaAreaTransferRateAnalysis>> list(SRaAreaTransferRateAnalysisQuery sRaAreaTransferRateAnalysis) {
        Page<SRaAreaTransferRateAnalysis> page = new Page<>(sRaAreaTransferRateAnalysis.getPageNum(), sRaAreaTransferRateAnalysis.getPageSize());
        IPage<SRaAreaTransferRateAnalysis> list = sRaAreaTransferRateAnalysisService.page(page, SRaAreaTransferRateAnalysisQuery.createLambdaQueryWrapper(sRaAreaTransferRateAnalysis));
        return AjaxResults.success(list);
    }


    /**
     * 导出区域传输率分析列表
     */
    @ApiOperation(value = "导出区域传输率分析")
    @Log(title = "区域传输率分析", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SRaAreaTransferRateAnalysisQuery sRaAreaTransferRateAnalysis) {
        List<SRaAreaTransferRateAnalysis> list = sRaAreaTransferRateAnalysisService.list(SRaAreaTransferRateAnalysisQuery.createLambdaQueryWrapper(sRaAreaTransferRateAnalysis));
        ExcelUtil<SRaAreaTransferRateAnalysis> util = new ExcelUtil<>(SRaAreaTransferRateAnalysis.class);
        util.exportExcel(response, list, "区域传输率分析数据");
    }

    /**
     * 获取区域传输率分析详细信息
     */
    @ApiOperation(value = "获取区域传输率分析详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<SRaAreaTransferRateAnalysis> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaAreaTransferRateAnalysisService.getById(id));
    }

    /**
     * 新增区域传输率分析
     */
    @ApiOperation(value = "新增区域传输率分析")
    @Log(title = "区域传输率分析", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaAreaTransferRateAnalysis sRaAreaTransferRateAnalysis) {
        return toAjax(sRaAreaTransferRateAnalysisService.save(sRaAreaTransferRateAnalysis));
    }

    /**
     * 修改区域传输率分析
     */
    @ApiOperation(value = "修改区域传输率分析")
    @Log(title = "区域传输率分析", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaAreaTransferRateAnalysisVo sRaAreaTransferRateAnalysisVo) {
        if (ObjectUtil.isEmpty(sRaAreaTransferRateAnalysisVo)) {
            throw new InvalidParameterException("参数为空");
        }
        if (ObjectUtil.isEmpty(sRaAreaTransferRateAnalysisVo.getId())) {
            throw new InvalidParameterException("参数id为空");
        }
        SRaAreaTransferRateAnalysis sRaAreaTransferRateAnalysis = sRaAreaTransferRateAnalysisService.getById(sRaAreaTransferRateAnalysisVo.getId());
        if (ObjectUtil.isEmpty(sRaAreaTransferRateAnalysis)) {
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaAreaTransferRateAnalysisVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaAreaTransferRateAnalysisVo, sRaAreaTransferRateAnalysis);
        return toAjax(sRaAreaTransferRateAnalysisService.saveOrUpdate(sRaAreaTransferRateAnalysis));
    }

    /**
     * 删除区域传输率分析
     */
    @ApiOperation(value = "删除区域传输率分析")
    @RequiresPermissions("talroad:analysis:remove")
    @Log(title = "区域传输率分析", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(sRaAreaTransferRateAnalysisService.removeByIds(Arrays.asList(ids)));
    }
}
