package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.analysis.SRaControlSchemeQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.analysis.SRaControlScheme;
import cn.cecep.talroad.vo.analysis.SRaControlSchemeVo;
import cn.cecep.talroad.service.analysis.ISRaControlSchemeService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 关联分析系统-减排方案管理-管控时间段信息Controller 控制层
 *
 * @author szhpt
 * @date 2023-04-02*/
@Api(tags = "关联分析系统-减排方案管理-管控时间段信息")
@RestController
@RequestMapping("/scheme")
public class SRaControlSchemeController extends BasesController
{
    @Autowired
    private ISRaControlSchemeService sRaControlSchemeService;

    /**
     * 查询关联分析系统-减排方案管理-管控时间段信息列表
     */
    @ApiOperation(value = "查询关联分析系统-减排方案管理-管控时间段信息")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaControlScheme>> list(SRaControlSchemeQuery sRaControlScheme)
    {
        Page<SRaControlScheme> page = new Page<>(sRaControlScheme.getPageNum(), sRaControlScheme.getPageSize());
        IPage<SRaControlScheme> list = sRaControlSchemeService.page(page, SRaControlSchemeQuery.createLambdaQueryWrapper(sRaControlScheme));
        return AjaxResults.success(list);
    }


    /**
     * 导出关联分析系统-减排方案管理-管控时间段信息列表
     */
    @ApiOperation(value = "导出关联分析系统-减排方案管理-管控时间段信息")
    @Log(title = "关联分析系统-减排方案管理-管控时间段信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SRaControlScheme sRaControlScheme)
    {
        List<SRaControlScheme> list = sRaControlSchemeService.selectSRaControlSchemeList(sRaControlScheme);
        ExcelUtil<SRaControlScheme> util = new ExcelUtil<>(SRaControlScheme.class);
        util.exportExcel(response, list, "关联分析系统-减排方案管理-管控时间段信息数据");
    }

    /**
     * 获取关联分析系统-减排方案管理-管控时间段信息详细信息
     */
    @ApiOperation(value = "获取关联分析系统-减排方案管理-管控时间段信息详细信息")
    @PostMapping(value = "/{schemeId}")
    public AjaxResults<SRaControlScheme> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaControlSchemeService.getById(id));
    }
    /**
     * 新增关联分析系统-减排方案管理-管控时间段信息
     */
    @ApiOperation(value = "新增关联分析系统-减排方案管理-管控时间段信息")
    @Log(title = "关联分析系统-减排方案管理-管控时间段信息", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaControlScheme sRaControlScheme) {
        return toAjax(sRaControlSchemeService.save(sRaControlScheme));
    }
    /**
     * 修改关联分析系统-减排方案管理-管控时间段信息
     */
    @ApiOperation(value = "修改关联分析系统-减排方案管理-管控时间段信息")
    @Log(title = "关联分析系统-减排方案管理-管控时间段信息", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaControlSchemeVo sRaControlSchemeVo) {
        if(ObjectUtil.isEmpty(sRaControlSchemeVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(sRaControlSchemeVo.getSchemeId())){
            throw new InvalidParameterException("参数id为空");
        }
        SRaControlScheme sRaControlScheme = sRaControlSchemeService.getById(sRaControlSchemeVo.getSchemeId());
        if(ObjectUtil.isEmpty(sRaControlScheme)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaControlSchemeVo.getSchemeId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaControlSchemeVo,sRaControlScheme);
        return toAjax(sRaControlSchemeService.saveOrUpdate(sRaControlScheme));
    }

    /**
     * 删除关联分析系统-减排方案管理-管控时间段信息
     */
    @ApiOperation(value = "删除关联分析系统-减排方案管理-管控时间段信息")
    @RequiresPermissions("talroad:scheme:remove")
    @Log(title = "关联分析系统-减排方案管理-管控时间段信息", businessType = BusinessType.DELETE)
    @PostMapping("/{schemeIds}")
    public AjaxResults<Boolean> remove(@PathVariable String[] schemeIds) {
        return toAjax(sRaControlSchemeService.removeByIds(Arrays.asList(schemeIds)));
    }
}
