package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.analysis.SRaEmissionsDiff;
import cn.cecep.talroad.service.analysis.ISRaEmissionsDiffService;
import cn.cecep.talroad.vo.analysis.SRaEmissionsDiffVo;
import cn.cecep.talroad.vo.query.analysis.SRaEmissionsDiffQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 企业排放量差异分析Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-24
 */
@Api(tags = "企业排放量差异分析")
@RestController
@RequestMapping("/diff")
public class SRaEmissionsDiffController extends BasesController {
    @Autowired
    private ISRaEmissionsDiffService sRaEmissionsDiffService;

    /**
     * 查询企业排放量差异分析列表
     */
    @ApiOperation(value = "查询企业排放量差异分析")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaEmissionsDiffVo>> list(SRaEmissionsDiffQuery query) {
        Page<SRaEmissionsDiffVo> page = new Page<>(query.getPageNum(), query.getPageSize());
        sRaEmissionsDiffService.selectEmissionsDiffList(page, query);
        return AjaxResults.success(page);
    }


    /**
     * 导出企业排放量差异分析列表
     */
    @ApiOperation(value = "导出企业排放量差异分析")
    @Log(title = "企业排放量差异分析", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response,@RequestBody SRaEmissionsDiffQuery query) {
        Page<SRaEmissionsDiffVo> page = new Page<>(query.getPageNum(), query.getPageSize());
        List<SRaEmissionsDiffVo> list = sRaEmissionsDiffService.selectEmissionsDiffList(page, query);
        ExcelUtil<SRaEmissionsDiffVo> util = new ExcelUtil<>(SRaEmissionsDiffVo.class);
        util.exportExcel(response, list, "企业排放量差异分析数据");
    }

    /**
     * 获取企业排放量差异分析详细信息
     */
    @ApiOperation(value = "获取企业排放量差异分析详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<SRaEmissionsDiff> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaEmissionsDiffService.getById(id));
    }

    /**
     * 新增企业排放量差异分析
     */
    @ApiOperation(value = "新增企业排放量差异分析")
    @Log(title = "企业排放量差异分析", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaEmissionsDiff sRaEmissionsDiff) {
        return toAjax(sRaEmissionsDiffService.save(sRaEmissionsDiff));
    }

    /**
     * 修改企业排放量差异分析
     */
    @ApiOperation(value = "修改企业排放量差异分析")
    @Log(title = "企业排放量差异分析", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaEmissionsDiffVo sRaEmissionsDiffVo) {
        if (ObjectUtil.isEmpty(sRaEmissionsDiffVo)) {
            throw new InvalidParameterException("参数为空");
        }
        if (ObjectUtil.isEmpty(sRaEmissionsDiffVo.getId())) {
            throw new InvalidParameterException("参数id为空");
        }
        SRaEmissionsDiff sRaEmissionsDiff = sRaEmissionsDiffService.getById(sRaEmissionsDiffVo.getId());
        if (ObjectUtil.isEmpty(sRaEmissionsDiff)) {
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaEmissionsDiffVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaEmissionsDiffVo, sRaEmissionsDiff);
        return toAjax(sRaEmissionsDiffService.saveOrUpdate(sRaEmissionsDiff));
    }

    /**
     * 删除企业排放量差异分析
     */
    @ApiOperation(value = "删除企业排放量差异分析")
    @RequiresPermissions("talroad:diff:remove")
    @Log(title = "企业排放量差异分析", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(sRaEmissionsDiffService.removeByIds(Arrays.asList(ids)));
    }
}
