package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.exception.InvalidParameterException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.analysis.SRaFactoryRegulatoryIssues;
import cn.cecep.talroad.service.analysis.ISRaFactoryRegulatoryIssuesService;
import cn.cecep.talroad.vo.analysis.SRaFactoryRegulatoryIssuesVo;
import cn.cecep.talroad.vo.query.analysis.SRaFactoryRegulatoryIssuesQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * * 开发公司：中节能天融科技有限公司
 * 企业证后监管问题总表Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-28*/
@Api(tags = "企业证后监管问题总表")
@RestController
@RequestMapping("/issuess")
public class SRaFactoryRegulatoryIssuesController extends BasesController
{
    @Autowired
    private ISRaFactoryRegulatoryIssuesService sRaFactoryRegulatoryIssuessService;

    /**
     * 查询企业证后监管问题总表列表
     */
    @ApiOperation(value = "查询企业证后监管问题总表")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaFactoryRegulatoryIssues>> list(SRaFactoryRegulatoryIssuesQuery sRaFactoryRegulatoryIssuess)
    {
        Page<SRaFactoryRegulatoryIssues> page = new Page<>(sRaFactoryRegulatoryIssuess.getPageNum(), sRaFactoryRegulatoryIssuess.getPageSize());
        IPage<SRaFactoryRegulatoryIssues> list = sRaFactoryRegulatoryIssuessService.selectSRaFactoryRegulatoryIssuesPage(page, SRaFactoryRegulatoryIssuesQuery.createLambdaQueryWrapper(sRaFactoryRegulatoryIssuess));
        return AjaxResults.success(list);
    }


    /**
     * 导出企业证后监管问题总表列表
     */
    @ApiOperation(value = "导出企业证后监管问题总表")
    @Log(title = "企业证后监管问题总表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SRaFactoryRegulatoryIssuesQuery sRaFactoryRegulatoryIssuess)
    {
        List<SRaFactoryRegulatoryIssues> list = sRaFactoryRegulatoryIssuessService.selectSRaFactoryRegulatoryIssuesList(SRaFactoryRegulatoryIssuesQuery.createLambdaQueryWrapper(sRaFactoryRegulatoryIssuess));
        ExcelUtil<SRaFactoryRegulatoryIssues> util = new ExcelUtil<>(SRaFactoryRegulatoryIssues.class);
        util.exportExcel(response, list, "企业证后监管问题总表数据");
    }

    /**
     * 获取企业证后监管问题总表详细信息
     */
    @ApiOperation(value = "获取企业证后监管问题总表详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<SRaFactoryRegulatoryIssues> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaFactoryRegulatoryIssuessService.getById(id));
    }
    /**
     * 新增企业证后监管问题总表
     */
    @ApiOperation(value = "新增企业证后监管问题总表")
    @Log(title = "企业证后监管问题总表", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaFactoryRegulatoryIssues sRaFactoryRegulatoryIssuess) {
        return toAjax(sRaFactoryRegulatoryIssuessService.save(sRaFactoryRegulatoryIssuess));
    }
    /**
     * 修改企业证后监管问题总表
     */
    @ApiOperation(value = "修改企业证后监管问题总表")
    @Log(title = "企业证后监管问题总表", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaFactoryRegulatoryIssuesVo sRaFactoryRegulatoryIssuessVo) {
        if(ObjectUtil.isEmpty(sRaFactoryRegulatoryIssuessVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(sRaFactoryRegulatoryIssuessVo.getId())){
            throw new InvalidParameterException("参数id为空");
        }
        SRaFactoryRegulatoryIssues sRaFactoryRegulatoryIssuess = sRaFactoryRegulatoryIssuessService.getById(sRaFactoryRegulatoryIssuessVo.getId());
        if(ObjectUtil.isEmpty(sRaFactoryRegulatoryIssuess)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaFactoryRegulatoryIssuessVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaFactoryRegulatoryIssuessVo,sRaFactoryRegulatoryIssuess);
        return toAjax(sRaFactoryRegulatoryIssuessService.saveOrUpdate(sRaFactoryRegulatoryIssuess));
    }

    /**
     * 删除企业证后监管问题总表
     */
    @ApiOperation(value = "删除企业证后监管问题总表")
    @RequiresPermissions("talroad:issuess:remove")
    @Log(title = "企业证后监管问题总表", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(sRaFactoryRegulatoryIssuessService.removeByIds(Arrays.asList(ids)));
    }
}
