package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.analysis.SRaFactoryTransferRateAnalysis;
import cn.cecep.talroad.service.analysis.ISRaFactoryTransferRateAnalysisService;
import cn.cecep.talroad.vo.analysis.SRaFactoryTransferRateAnalysisVo;
import cn.cecep.talroad.vo.query.analysis.SRaFactoryTransferRateAnalysisQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 企业传输率分析Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-24
 */
@Api(tags = "企业传输率分析")
@RestController
@RequestMapping("/ent/analysis")
public class SRaFactoryTransferRateAnalysisController extends BasesController {
    @Autowired
    private ISRaFactoryTransferRateAnalysisService sRaFactoryTransferRateAnalysisService;

    /**
     * 查询企业传输率分析列表
     */
    @ApiOperation(value = "查询企业传输率分析")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaFactoryTransferRateAnalysis>> list(SRaFactoryTransferRateAnalysisQuery sRaFactoryTransferRateAnalysis) {
        Page<SRaFactoryTransferRateAnalysis> page = new Page<>(sRaFactoryTransferRateAnalysis.getPageNum(), sRaFactoryTransferRateAnalysis.getPageSize());
        IPage<SRaFactoryTransferRateAnalysis> list =
                sRaFactoryTransferRateAnalysisService.page(page,
                        SRaFactoryTransferRateAnalysisQuery.createLambdaQueryWrapper(sRaFactoryTransferRateAnalysis));
        return AjaxResults.success(list);
    }


    /**
     * 导出企业传输率分析列表
     */
    @ApiOperation(value = "导出企业传输率分析")
    @Log(title = "企业传输率分析", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public void export(HttpServletResponse response,@RequestBody SRaFactoryTransferRateAnalysisQuery sRaFactoryTransferRateAnalysis) {
        List<SRaFactoryTransferRateAnalysis> list =
                sRaFactoryTransferRateAnalysisService.list(SRaFactoryTransferRateAnalysisQuery.createLambdaQueryWrapper(sRaFactoryTransferRateAnalysis));
        ExcelUtil<SRaFactoryTransferRateAnalysis> util = new ExcelUtil<>(SRaFactoryTransferRateAnalysis.class);
        util.exportExcel(response, list, "企业传输率分析数据");
    }

    /**
     * 获取企业传输率分析详细信息
     */
    @ApiOperation(value = "获取企业传输率分析详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<SRaFactoryTransferRateAnalysis> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaFactoryTransferRateAnalysisService.getById(id));
    }

    /**
     * 新增企业传输率分析
     */
    @ApiOperation(value = "新增企业传输率分析")
    @Log(title = "企业传输率分析", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaFactoryTransferRateAnalysis sRaFactoryTransferRateAnalysis) {
        return toAjax(sRaFactoryTransferRateAnalysisService.save(sRaFactoryTransferRateAnalysis));
    }

    /**
     * 修改企业传输率分析
     */
    @ApiOperation(value = "修改企业传输率分析")
    @Log(title = "企业传输率分析", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaFactoryTransferRateAnalysisVo sRaFactoryTransferRateAnalysisVo) {
        if (ObjectUtil.isEmpty(sRaFactoryTransferRateAnalysisVo)) {
            throw new InvalidParameterException("参数为空");
        }
        if (ObjectUtil.isEmpty(sRaFactoryTransferRateAnalysisVo.getId())) {
            throw new InvalidParameterException("参数id为空");
        }
        SRaFactoryTransferRateAnalysis sRaFactoryTransferRateAnalysis = sRaFactoryTransferRateAnalysisService.getById(sRaFactoryTransferRateAnalysisVo.getId());
        if (ObjectUtil.isEmpty(sRaFactoryTransferRateAnalysis)) {
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaFactoryTransferRateAnalysisVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaFactoryTransferRateAnalysisVo, sRaFactoryTransferRateAnalysis);
        return toAjax(sRaFactoryTransferRateAnalysisService.saveOrUpdate(sRaFactoryTransferRateAnalysis));
    }

    /**
     * 删除企业传输率分析
     */
    @ApiOperation(value = "删除企业传输率分析")
    @RequiresPermissions("talroad:analysis:remove")
    @Log(title = "企业传输率分析", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(sRaFactoryTransferRateAnalysisService.removeByIds(Arrays.asList(ids)));
    }
}
