package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.domain.analysis.SRaGasPermittedEmissions;
import cn.cecep.talroad.service.analysis.ISRaGasPermittedEmissionsService;
import cn.cecep.talroad.vo.analysis.SRaGasPermittedEmissionsVo;
import cn.cecep.talroad.vo.query.analysis.SRaGasPermittedEmissionsQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.List;
import java.util.Arrays;
import java.util.Map;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;


/**
 * * 开发公司：中节能天融科技有限公司
 * 许可排放量废气明细Controller 控制层
 *
 * @author szhpt
 * @date 2023-06-02*/
@Api(tags = "许可排放量废气明细")
@RestController
@RequestMapping("/emissions/gas")
public class SRaGasPermittedEmissionsController extends BasesController
{
    @Autowired
    private ISRaGasPermittedEmissionsService sRaGasPermittedEmissionsService;

    /**
     * 查询许可排放量废气明细列表
     */
    @ApiOperation(value = "查询许可排放量废气明细")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaGasPermittedEmissions>> list(SRaGasPermittedEmissionsQuery sRaGasPermittedEmissions)
    {
        IPage<SRaGasPermittedEmissions> list = sRaGasPermittedEmissionsService.selectSRaGasPermittedEmissionsListBy( sRaGasPermittedEmissions);
        return AjaxResults.success(list);
    }
    

    /**
     * 导出许可排放量废气明细列表
     */
    @ApiOperation(value = "导出许可排放量废气明细")
    @Log(title = "许可排放量废气明细", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SRaGasPermittedEmissionsQuery sRaGasPermittedEmissions)
    {
        List<SRaGasPermittedEmissions> list = sRaGasPermittedEmissionsService.selectSRaGasPermittedEmissionsList(sRaGasPermittedEmissions);
        ExcelUtil<SRaGasPermittedEmissions> util = new ExcelUtil<SRaGasPermittedEmissions>(SRaGasPermittedEmissions.class);
        util.exportExcel(response, list, "许可排放量废气明细数据");
    }

    /**
     * 获取许可排放量废气明细详细信息
     */
    @ApiOperation(value = "获取许可排放量废气明细详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<SRaGasPermittedEmissions> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaGasPermittedEmissionsService.getById(id));
    }
    /**
     * 新增许可排放量废气明细
     */
    @ApiOperation(value = "新增许可排放量废气明细")
    @Log(title = "许可排放量废气明细", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaGasPermittedEmissions sRaGasPermittedEmissions) {
        return toAjax(sRaGasPermittedEmissionsService.save(sRaGasPermittedEmissions));
    }
    /**
     * 修改许可排放量废气明细
     */
    @ApiOperation(value = "修改许可排放量废气明细")
    @Log(title = "许可排放量废气明细", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaGasPermittedEmissionsVo sRaGasPermittedEmissionsVo) {
        if(ObjectUtil.isEmpty(sRaGasPermittedEmissionsVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(sRaGasPermittedEmissionsVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        SRaGasPermittedEmissions sRaGasPermittedEmissions = sRaGasPermittedEmissionsService.getById(sRaGasPermittedEmissionsVo.getDataId());
        if(ObjectUtil.isEmpty(sRaGasPermittedEmissions)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaGasPermittedEmissionsVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaGasPermittedEmissionsVo,sRaGasPermittedEmissions);
        return toAjax(sRaGasPermittedEmissionsService.saveOrUpdate(sRaGasPermittedEmissions));
    }

    /**
     * 删除许可排放量废气明细
     */
    @ApiOperation(value = "删除许可排放量废气明细")
    @RequiresPermissions("talroad:emissions:remove")
    @Log(title = "许可排放量废气明细", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable String[] dataIds) {
        return toAjax(sRaGasPermittedEmissionsService.removeByIds(Arrays.asList(dataIds)));
    }
    /**
     * 监测点名称options
     */
    @ApiOperation(value = "监测点名称options")
    @GetMapping("/pullMonitorNameOptions")
    public AjaxResults<List<Map<String, Object>>> pullMonitorNameOptions(@RequestParam(value = "factoryName", required = false) String factoryName) {
        return AjaxResults.success(sRaGasPermittedEmissionsService.pullMonitorNameOptions(factoryName));
    }
}
