package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.analysis.SRaOnlineEmissionsGas;
import cn.cecep.talroad.service.analysis.ISRaOnlineEmissionsGasService;
import cn.cecep.talroad.vo.analysis.SRaOnlineEmissionsGasVo;
import cn.cecep.talroad.vo.query.analysis.SRaOnlineEmissionsGasQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 在线监测排放监控-气污染要素Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-24
 */
@Api(tags = "在线监测排放监控-气污染要素")
@RestController
@RequestMapping("/online/gas")
public class SRaOnlineEmissionsGasController extends BasesController {
    @Autowired
    private ISRaOnlineEmissionsGasService sRaOnlineEmissionsGasService;

    /**
     * 查询在线监测排放监控-气污染要素列表
     */
    @ApiOperation(value = "查询在线监测排放监控-气污染要素")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaOnlineEmissionsGas>> list(SRaOnlineEmissionsGasQuery sRaOnlineEmissionsGas) {
        Page<SRaOnlineEmissionsGas> page = new Page<>(sRaOnlineEmissionsGas.getPageNum(), sRaOnlineEmissionsGas.getPageSize());
        IPage<SRaOnlineEmissionsGas> list = sRaOnlineEmissionsGasService.page(page, SRaOnlineEmissionsGasQuery.createLambdaQueryWrapper(sRaOnlineEmissionsGas));
        return AjaxResults.success(list);
    }


    /**
     * 导出在线监测排放监控-气污染要素列表
     */
    @ApiOperation(value = "导出在线监测排放监控-气污染要素")
    @Log(title = "在线监测排放监控-气污染要素", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SRaOnlineEmissionsGas sRaOnlineEmissionsGas) {
        List<SRaOnlineEmissionsGas> list = sRaOnlineEmissionsGasService.selectSRaOnlineEmissionsGasList(sRaOnlineEmissionsGas);
        ExcelUtil<SRaOnlineEmissionsGas> util = new ExcelUtil<SRaOnlineEmissionsGas>(SRaOnlineEmissionsGas.class);
        util.exportExcel(response, list, "在线监测排放监控-气污染要素数据");
    }

    /**
     * 获取在线监测排放监控-气污染要素详细信息
     */
    @ApiOperation(value = "获取在线监测排放监控-气污染要素详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<SRaOnlineEmissionsGas> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaOnlineEmissionsGasService.getById(id));
    }

    /**
     * 新增在线监测排放监控-气污染要素
     */
    @ApiOperation(value = "新增在线监测排放监控-气污染要素")
    @Log(title = "在线监测排放监控-气污染要素", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaOnlineEmissionsGas sRaOnlineEmissionsGas) {
        return toAjax(sRaOnlineEmissionsGasService.save(sRaOnlineEmissionsGas));
    }

    /**
     * 修改在线监测排放监控-气污染要素
     */
    @ApiOperation(value = "修改在线监测排放监控-气污染要素")
    @Log(title = "在线监测排放监控-气污染要素", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaOnlineEmissionsGasVo sRaOnlineEmissionsGasVo) {
        if (ObjectUtil.isEmpty(sRaOnlineEmissionsGasVo)) {
            throw new InvalidParameterException("参数为空");
        }
        if (ObjectUtil.isEmpty(sRaOnlineEmissionsGasVo.getId())) {
            throw new InvalidParameterException("参数id为空");
        }
        SRaOnlineEmissionsGas sRaOnlineEmissionsGas = sRaOnlineEmissionsGasService.getById(sRaOnlineEmissionsGasVo.getId());
        if (ObjectUtil.isEmpty(sRaOnlineEmissionsGas)) {
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaOnlineEmissionsGasVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaOnlineEmissionsGasVo, sRaOnlineEmissionsGas);
        return toAjax(sRaOnlineEmissionsGasService.saveOrUpdate(sRaOnlineEmissionsGas));
    }

    /**
     * 删除在线监测排放监控-气污染要素
     */
    @ApiOperation(value = "删除在线监测排放监控-气污染要素")
    @RequiresPermissions("talroad:gas:remove")
    @Log(title = "在线监测排放监控-气污染要素", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(sRaOnlineEmissionsGasService.removeByIds(Arrays.asList(ids)));
    }
}
