package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.domain.analysis.SRaOnlineEmissionsWater;
import cn.cecep.talroad.service.analysis.ISRaOnlineEmissionsWaterService;
import cn.cecep.talroad.vo.analysis.SRaOnlineEmissionsWaterVo;
import cn.cecep.talroad.vo.query.analysis.SRaOnlineEmissionsWaterQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.Arrays;
import java.util.List;


/**
 * * 开发公司：中节能天融科技有限公司
 * 在线监测排放监控-水污染要素Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-24
 */
@Api(tags = "在线监测排放监控-水污染要素")
@RestController
@RequestMapping("/online/water")
public class SRaOnlineEmissionsWaterController extends BasesController {
    @Autowired
    private ISRaOnlineEmissionsWaterService sRaOnlineEmissionsWaterService;

    /**
     * 查询在线监测排放监控-水污染要素列表
     */
    @ApiOperation(value = "查询在线监测排放监控-水污染要素")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaOnlineEmissionsWater>> list(SRaOnlineEmissionsWaterQuery sRaOnlineEmissionsWater) {
        Page<SRaOnlineEmissionsWater> page = new Page<>(sRaOnlineEmissionsWater.getPageNum(), sRaOnlineEmissionsWater.getPageSize());
        IPage<SRaOnlineEmissionsWater> list = sRaOnlineEmissionsWaterService.page(page, SRaOnlineEmissionsWaterQuery.createLambdaQueryWrapper(sRaOnlineEmissionsWater));
        return AjaxResults.success(list);
    }


    /**
     * 导出在线监测排放监控-水污染要素列表
     */
    @ApiOperation(value = "导出在线监测排放监控-水污染要素")
    @Log(title = "在线监测排放监控-水污染要素", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SRaOnlineEmissionsWater sRaOnlineEmissionsWater) {
        List<SRaOnlineEmissionsWater> list = sRaOnlineEmissionsWaterService.selectSRaOnlineEmissionsWaterList(sRaOnlineEmissionsWater);
        ExcelUtil<SRaOnlineEmissionsWater> util = new ExcelUtil<SRaOnlineEmissionsWater>(SRaOnlineEmissionsWater.class);
        util.exportExcel(response, list, "在线监测排放监控-水污染要素数据");
    }

    /**
     * 获取在线监测排放监控-水污染要素详细信息
     */
    @ApiOperation(value = "获取在线监测排放监控-水污染要素详细信息")
    @PostMapping(value = "/{id}")
    public AjaxResults<SRaOnlineEmissionsWater> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaOnlineEmissionsWaterService.getById(id));
    }

    /**
     * 新增在线监测排放监控-水污染要素
     */
    @ApiOperation(value = "新增在线监测排放监控-水污染要素")
    @Log(title = "在线监测排放监控-水污染要素", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaOnlineEmissionsWater sRaOnlineEmissionsWater) {
        return toAjax(sRaOnlineEmissionsWaterService.save(sRaOnlineEmissionsWater));
    }

    /**
     * 修改在线监测排放监控-水污染要素
     */
    @ApiOperation(value = "修改在线监测排放监控-水污染要素")
    @Log(title = "在线监测排放监控-水污染要素", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaOnlineEmissionsWaterVo sRaOnlineEmissionsWaterVo) {
        if (ObjectUtil.isEmpty(sRaOnlineEmissionsWaterVo)) {
            throw new InvalidParameterException("参数为空");
        }
        if (ObjectUtil.isEmpty(sRaOnlineEmissionsWaterVo.getId())) {
            throw new InvalidParameterException("参数id为空");
        }
        SRaOnlineEmissionsWater sRaOnlineEmissionsWater = sRaOnlineEmissionsWaterService.getById(sRaOnlineEmissionsWaterVo.getId());
        if (ObjectUtil.isEmpty(sRaOnlineEmissionsWater)) {
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaOnlineEmissionsWaterVo.getId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaOnlineEmissionsWaterVo, sRaOnlineEmissionsWater);
        return toAjax(sRaOnlineEmissionsWaterService.saveOrUpdate(sRaOnlineEmissionsWater));
    }

    /**
     * 删除在线监测排放监控-水污染要素
     */
    @ApiOperation(value = "删除在线监测排放监控-水污染要素")
    @RequiresPermissions("talroad:water:remove")
    @Log(title = "在线监测排放监控-水污染要素", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResults<Boolean> remove(@PathVariable String[] ids) {
        return toAjax(sRaOnlineEmissionsWaterService.removeByIds(Arrays.asList(ids)));
    }
}
