package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.domain.R;
import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.security.utils.SecurityUtils;
import cn.cecep.talroad.feign.BaseRegionService;
import cn.cecep.talroad.vo.EChartsVO;
import cn.cecep.talroad.vo.query.analysis.SRaRealTimeAnalysisCokeQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;
import java.util.Map;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.analysis.SRaRealTimeAnalysisCoke;
import cn.cecep.talroad.vo.analysis.SRaRealTimeAnalysisCokeVo;
import cn.cecep.talroad.service.analysis.ISRaRealTimeAnalysisCokeService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 关联分析系统-企业生产过程分析-焦炭产量分析Controller 控制层
 *
 * @author szhpt
 * @date 2023-04-03*/
@Api(tags = "关联分析系统-企业生产过程分析-焦炭产量分析")
@RestController
@RequestMapping("/coke")
public class SRaRealTimeAnalysisCokeController extends BasesController
{
    @Autowired
    private ISRaRealTimeAnalysisCokeService sRaRealTimeAnalysisCokeService;

    @Autowired
//    @Qualifier("baseRegionService")
    private BaseRegionService baseRegionService;
    /**
     * 查询关联分析系统-企业生产过程分析-焦炭产量分析列表
     */
    @ApiOperation(value = "查询")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaRealTimeAnalysisCoke>> list(SRaRealTimeAnalysisCokeQuery sRaRealTimeAnalysisCoke)
    {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();

        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisCoke.setRegionCodes(regionCodes);
        Page<SRaRealTimeAnalysisCoke> page = new Page<>(sRaRealTimeAnalysisCoke.getPageNum(), sRaRealTimeAnalysisCoke.getPageSize());
        IPage<SRaRealTimeAnalysisCoke> list = sRaRealTimeAnalysisCokeService.page(page, SRaRealTimeAnalysisCokeQuery.createLambdaQueryWrapper(sRaRealTimeAnalysisCoke));
        return AjaxResults.success(list);
    }

    @ApiOperation(value = "焦炭产量分析分页查询")
    @GetMapping("/statisticsList")
    public AjaxResults<IPage<SRaRealTimeAnalysisCoke>> statisticsList(SRaRealTimeAnalysisCokeQuery sRaRealTimeAnalysisCoke) {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();

        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisCoke.setRegionCodes(regionCodes);
        IPage<SRaRealTimeAnalysisCoke> list = sRaRealTimeAnalysisCokeService.statisticsList(sRaRealTimeAnalysisCoke);
        return AjaxResults.success(list);
    }
    @ApiOperation(value = "焦炭产量分析导出")
    @Log(title = "焦炭产量分析", businessType = BusinessType.EXPORT)
    @PostMapping("/statisticsExport")
    public void statisticsExport(HttpServletResponse response, @RequestBody SRaRealTimeAnalysisCokeQuery sRaRealTimeAnalysisCoke) {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();

        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisCoke.setRegionCodes(regionCodes);
        List<SRaRealTimeAnalysisCoke> list = sRaRealTimeAnalysisCokeService.statisticsExport(sRaRealTimeAnalysisCoke);
        if(sRaRealTimeAnalysisCoke.getType().equals("1")){
            List<SRaRealTimeAnalysisCoke.SRaRealTimeAnalysisFactoryCoke> factoryCoalList=list.stream().map(o -> {
                SRaRealTimeAnalysisCoke.SRaRealTimeAnalysisFactoryCoke vo = new SRaRealTimeAnalysisCoke.SRaRealTimeAnalysisFactoryCoke();
                BeanUtils.copyProperties(o, vo);
                vo.setAnalyseTimeStr(sRaRealTimeAnalysisCoke.getAnalyseStartTime()+"至"+sRaRealTimeAnalysisCoke.getAnalyseEndTime());
                return vo;
            }).collect(Collectors.toList());
            ExcelUtil<SRaRealTimeAnalysisCoke.SRaRealTimeAnalysisFactoryCoke> util = new ExcelUtil<SRaRealTimeAnalysisCoke.SRaRealTimeAnalysisFactoryCoke>(SRaRealTimeAnalysisCoke.SRaRealTimeAnalysisFactoryCoke.class);
            util.exportExcel(response, factoryCoalList, "焦炭产量分析");
        }else {
            List<SRaRealTimeAnalysisCoke.SRaRealTimeAnalysisDeciveCoke> factoryCoalList=list.stream().map(o -> {
                SRaRealTimeAnalysisCoke.SRaRealTimeAnalysisDeciveCoke vo = new SRaRealTimeAnalysisCoke.SRaRealTimeAnalysisDeciveCoke();
                BeanUtils.copyProperties(o, vo);
                vo.setAnalyseTimeStr(sRaRealTimeAnalysisCoke.getAnalyseStartTime()+"至"+sRaRealTimeAnalysisCoke.getAnalyseEndTime());
                return vo;
            }).collect(Collectors.toList());
            ExcelUtil<SRaRealTimeAnalysisCoke.SRaRealTimeAnalysisDeciveCoke> util = new ExcelUtil<SRaRealTimeAnalysisCoke.SRaRealTimeAnalysisDeciveCoke>(SRaRealTimeAnalysisCoke.SRaRealTimeAnalysisDeciveCoke.class);
            util.exportExcel(response, factoryCoalList, "焦炭产量分析");
        }

    }

    @ApiOperation(value = "焦炭产量分析-曲线图")
    @GetMapping("/resultCurve")
    public AjaxResults<EChartsVO> resultCurve(SRaRealTimeAnalysisCokeQuery sRaRealTimeAnalysisCoa) {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();

        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisCoa.setRegionCodes(regionCodes);
        return AjaxResults.success(sRaRealTimeAnalysisCokeService.resultCurve(sRaRealTimeAnalysisCoa));
    }
    /**
     * 导出关联分析系统-企业生产过程分析-焦炭产量分析列表
     */
    @ApiOperation(value = "导出")
    @Log(title = "关联分析系统-企业生产过程分析-焦炭产量分析", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SRaRealTimeAnalysisCokeQuery sRaRealTimeAnalysisCoke)
    {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();

        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisCoke.setRegionCodes(regionCodes);
        List<SRaRealTimeAnalysisCoke> list = sRaRealTimeAnalysisCokeService.selectSRaRealTimeAnalysisCokeList(sRaRealTimeAnalysisCoke);
        ExcelUtil<SRaRealTimeAnalysisCoke> util = new ExcelUtil<SRaRealTimeAnalysisCoke>(SRaRealTimeAnalysisCoke.class);
        util.exportExcel(response, list, "关联分析系统-企业生产过程分析-焦炭产量分析数据");
    }

    /**
     * 获取关联分析系统-企业生产过程分析-焦炭产量分析详细信息
     */
    @ApiOperation(value = "获取详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<SRaRealTimeAnalysisCoke> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaRealTimeAnalysisCokeService.getById(id));
    }
    /**
     * 新增关联分析系统-企业生产过程分析-焦炭产量分析
     */
    @ApiOperation(value = "新增")
    @Log(title = "关联分析系统-企业生产过程分析-焦炭产量分析", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaRealTimeAnalysisCoke sRaRealTimeAnalysisCoke) {
        return toAjax(sRaRealTimeAnalysisCokeService.save(sRaRealTimeAnalysisCoke));
    }
    /**
     * 修改关联分析系统-企业生产过程分析-焦炭产量分析
     */
    @ApiOperation(value = "修改")
    @Log(title = "关联分析系统-企业生产过程分析-焦炭产量分析", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaRealTimeAnalysisCokeVo sRaRealTimeAnalysisCokeVo) {
        if(ObjectUtil.isEmpty(sRaRealTimeAnalysisCokeVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(sRaRealTimeAnalysisCokeVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        SRaRealTimeAnalysisCoke sRaRealTimeAnalysisCoke = sRaRealTimeAnalysisCokeService.getById(sRaRealTimeAnalysisCokeVo.getDataId());
        if(ObjectUtil.isEmpty(sRaRealTimeAnalysisCoke)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaRealTimeAnalysisCokeVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaRealTimeAnalysisCokeVo,sRaRealTimeAnalysisCoke);
        return toAjax(sRaRealTimeAnalysisCokeService.saveOrUpdate(sRaRealTimeAnalysisCoke));
    }

    /**
     * 删除关联分析系统-企业生产过程分析-焦炭产量分析
     */
    @ApiOperation(value = "删除")
    @RequiresPermissions("talroad:coke:remove")
    @Log(title = "关联分析系统-企业生产过程分析-焦炭产量分析", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable String[] dataIds) {
        return toAjax(sRaRealTimeAnalysisCokeService.removeByIds(Arrays.asList(dataIds)));
    }
    /**
     * 设备名称options
     */
    @ApiOperation(value = "设备名称options")
    @GetMapping("/pullDeviceNameOptions")
    public AjaxResults<List<Map<String, Object>>> pullDeviceNameOptions(@RequestParam(value = "factoryName", required = false) String factoryName) {
        return AjaxResults.success(sRaRealTimeAnalysisCokeService.pullDeviceNameOptions(factoryName));
    }
}
