package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.domain.R;
import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.common.security.utils.SecurityUtils;
import cn.cecep.talroad.domain.analysis.SRaRealTimeAnalysisGlass;
import cn.cecep.talroad.domain.analysis.SRaRealTimeAnalysisGlass;
import cn.cecep.talroad.feign.BaseRegionService;
import cn.cecep.talroad.service.analysis.ISRaRealTimeAnalysisGlassService;
import cn.cecep.talroad.vo.EChartsVO;
import cn.cecep.talroad.vo.analysis.SRaRealTimeAnalysisGlassVo;
import cn.cecep.talroad.vo.query.analysis.SRaRealTimeAnalysisGlassQuery;
import cn.cecep.talroad.vo.query.analysis.SRaRealTimeAnalysisGlassQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.*;


/**
 * * 开发公司：中节能天融科技有限公司
 * 关联分析系统-企业生产过程分析-玻璃产量分析Controller 控制层
 *
 * @author szhpt
 * @date 2023-04-02*/
@Api(tags = "关联分析系统-企业生产过程分析-玻璃产量分析")
@RestController
@RequestMapping("/glass")
public class SRaRealTimeAnalysisGlassController extends BasesController
{
    @Autowired
    private ISRaRealTimeAnalysisGlassService sRaRealTimeAnalysisGlassService;

    @Autowired
//    @Qualifier("baseRegionService")
    private BaseRegionService baseRegionService;

    /**
     * 查询关联分析系统-企业生产过程分析-玻璃产量分析列表
     */
    @ApiOperation(value = "查询关联分析系统-企业生产过程分析-玻璃产量分析")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaRealTimeAnalysisGlass>> list(SRaRealTimeAnalysisGlassQuery sRaRealTimeAnalysisGlass)
    {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();
        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisGlass.setRegionCodes(regionCodes);
        Page<SRaRealTimeAnalysisGlass> page = new Page<>(sRaRealTimeAnalysisGlass.getPageNum(), sRaRealTimeAnalysisGlass.getPageSize());
        IPage<SRaRealTimeAnalysisGlass> list = sRaRealTimeAnalysisGlassService.page(page, SRaRealTimeAnalysisGlassQuery.createLambdaQueryWrapper(sRaRealTimeAnalysisGlass));
        return AjaxResults.success(list);
    }


    @ApiOperation(value = "玻璃产量统计分页查询")
    @GetMapping("/statisticsList")
    public AjaxResults<IPage<SRaRealTimeAnalysisGlass>> statisticsList(SRaRealTimeAnalysisGlassQuery sRaRealTimeAnalysisGlass) {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();
        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisGlass.setRegionCodes(regionCodes);
        IPage<SRaRealTimeAnalysisGlass> list = sRaRealTimeAnalysisGlassService.statisticsList(sRaRealTimeAnalysisGlass);
        return AjaxResults.success(list);
    }
    @ApiOperation(value = "玻璃产量统计导出")
    @Log(title = "玻璃产量统计", businessType = BusinessType.EXPORT)
    @PostMapping("/statisticsExport")
    public void statisticsExport(HttpServletResponse response, @RequestBody SRaRealTimeAnalysisGlassQuery sRaRealTimeAnalysisGlass) {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();
        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisGlass.setRegionCodes(regionCodes);
        List<SRaRealTimeAnalysisGlass> list = sRaRealTimeAnalysisGlassService.statisticsExport(sRaRealTimeAnalysisGlass);
        if(sRaRealTimeAnalysisGlass.getType().equals("1")){
            List<SRaRealTimeAnalysisGlass.SRaRealTimeAnalysisFactoryGlass> factoryCoalList=list.stream().map(o -> {
                SRaRealTimeAnalysisGlass.SRaRealTimeAnalysisFactoryGlass vo = new SRaRealTimeAnalysisGlass.SRaRealTimeAnalysisFactoryGlass();
                BeanUtils.copyProperties(o, vo);
                vo.setAnalyseTimeStr(sRaRealTimeAnalysisGlass.getAnalyseStartTime()+"至"+sRaRealTimeAnalysisGlass.getAnalyseEndTime());
                return vo;
            }).collect(Collectors.toList());
            ExcelUtil<SRaRealTimeAnalysisGlass.SRaRealTimeAnalysisFactoryGlass> util = new ExcelUtil<SRaRealTimeAnalysisGlass.SRaRealTimeAnalysisFactoryGlass>(SRaRealTimeAnalysisGlass.SRaRealTimeAnalysisFactoryGlass.class);
            util.exportExcel(response, factoryCoalList, "玻璃产量统计");
        }else {
            List<SRaRealTimeAnalysisGlass.SRaRealTimeAnalysisDeviceGlass> factoryCoalList=list.stream().map(o -> {
                SRaRealTimeAnalysisGlass.SRaRealTimeAnalysisDeviceGlass vo = new SRaRealTimeAnalysisGlass.SRaRealTimeAnalysisDeviceGlass();
                BeanUtils.copyProperties(o, vo);
                vo.setAnalyseTimeStr(sRaRealTimeAnalysisGlass.getAnalyseStartTime()+"至"+sRaRealTimeAnalysisGlass.getAnalyseEndTime());
                return vo;
            }).collect(Collectors.toList());
            ExcelUtil<SRaRealTimeAnalysisGlass.SRaRealTimeAnalysisDeviceGlass> util = new ExcelUtil<SRaRealTimeAnalysisGlass.SRaRealTimeAnalysisDeviceGlass>(SRaRealTimeAnalysisGlass.SRaRealTimeAnalysisDeviceGlass.class);
            util.exportExcel(response, factoryCoalList, "玻璃产量统计");
        }

    }

    @ApiOperation(value = "玻璃产量统计-曲线图")
    @GetMapping("/resultCurve")
    public AjaxResults<EChartsVO> resultCurve(SRaRealTimeAnalysisGlassQuery sRaRealTimeAnalysisCoa) {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();
        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisCoa.setRegionCodes(regionCodes);
        return AjaxResults.success(sRaRealTimeAnalysisGlassService.resultCurve(sRaRealTimeAnalysisCoa));
    }
    /**
     * 导出关联分析系统-企业生产过程分析-玻璃产量分析列表
     */
    @ApiOperation(value = "导出关联分析系统-企业生产过程分析-玻璃产量分析")
    @Log(title = "关联分析系统-企业生产过程分析-玻璃产量分析", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SRaRealTimeAnalysisGlassQuery sRaRealTimeAnalysisGlass)
    {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();
        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisGlass.setRegionCodes(regionCodes);
        List<SRaRealTimeAnalysisGlass> list = sRaRealTimeAnalysisGlassService.selectSRaRealTimeAnalysisGlassList(sRaRealTimeAnalysisGlass);
        ExcelUtil<SRaRealTimeAnalysisGlass> util = new ExcelUtil<SRaRealTimeAnalysisGlass>(SRaRealTimeAnalysisGlass.class);
        util.exportExcel(response, list, "关联分析系统-企业生产过程分析-玻璃产量分析数据");
    }

    /**
     * 获取关联分析系统-企业生产过程分析-玻璃产量分析详细信息
     */
    @ApiOperation(value = "获取关联分析系统-企业生产过程分析-玻璃产量分析详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<SRaRealTimeAnalysisGlass> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaRealTimeAnalysisGlassService.getById(id));
    }
    /**
     * 新增关联分析系统-企业生产过程分析-玻璃产量分析
     */
    @ApiOperation(value = "新增关联分析系统-企业生产过程分析-玻璃产量分析")
    @Log(title = "关联分析系统-企业生产过程分析-玻璃产量分析", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaRealTimeAnalysisGlass sRaRealTimeAnalysisGlass) {
        return toAjax(sRaRealTimeAnalysisGlassService.save(sRaRealTimeAnalysisGlass));
    }
    /**
     * 修改关联分析系统-企业生产过程分析-玻璃产量分析
     */
    @ApiOperation(value = "修改关联分析系统-企业生产过程分析-玻璃产量分析")
    @Log(title = "关联分析系统-企业生产过程分析-玻璃产量分析", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaRealTimeAnalysisGlassVo sRaRealTimeAnalysisGlassVo) {
        if(ObjectUtil.isEmpty(sRaRealTimeAnalysisGlassVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(sRaRealTimeAnalysisGlassVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        SRaRealTimeAnalysisGlass sRaRealTimeAnalysisGlass = sRaRealTimeAnalysisGlassService.getById(sRaRealTimeAnalysisGlassVo.getDataId());
        if(ObjectUtil.isEmpty(sRaRealTimeAnalysisGlass)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaRealTimeAnalysisGlassVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaRealTimeAnalysisGlassVo,sRaRealTimeAnalysisGlass);
        return toAjax(sRaRealTimeAnalysisGlassService.saveOrUpdate(sRaRealTimeAnalysisGlass));
    }

    /**
     * 删除关联分析系统-企业生产过程分析-玻璃产量分析
     */
    @ApiOperation(value = "删除关联分析系统-企业生产过程分析-玻璃产量分析")
    @RequiresPermissions("talroad:glass:remove")
    @Log(title = "关联分析系统-企业生产过程分析-玻璃产量分析", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable String[] dataIds) {
        return toAjax(sRaRealTimeAnalysisGlassService.removeByIds(Arrays.asList(dataIds)));
    }
    /**
     * 设备名称options
     */
    @ApiOperation(value = "设备名称options")
    @GetMapping("/pullDeviceNameOptions")
    public AjaxResults<List<Map<String, Object>>> pullDeviceNameOptions(@RequestParam(value = "factoryName", required = false) String factoryName) {
        return AjaxResults.success(sRaRealTimeAnalysisGlassService.pullDeviceNameOptions(factoryName));
    }
}
