package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.domain.R;
import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;
import cn.cecep.talroad.common.security.utils.SecurityUtils;
import cn.cecep.talroad.domain.analysis.SRaRealTimeAnalysisKwh;
import cn.cecep.talroad.feign.BaseRegionService;
import cn.cecep.talroad.service.analysis.ISRaRealTimeAnalysisKwhService;
import cn.cecep.talroad.vo.analysis.SRaRealTimeAnalysisKwhVo;
import cn.cecep.talroad.vo.query.analysis.SRaRealTimeAnalysisKwhQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;


/**
 * * 开发公司：中节能天融科技有限公司
 * 企业用电水平分析Controller 控制层
 *
 * @author szhpt
 * @date 2023-03-24
 */
@Api(tags = "企业用电水平分析")
@RestController
@RequestMapping("/kwh")
public class SRaRealTimeAnalysisKwhController extends BasesController {
    @Autowired
    private ISRaRealTimeAnalysisKwhService sRaRealTimeAnalysisKwhService;

    @Autowired
//    @Qualifier("baseRegionService")
    private BaseRegionService baseRegionService;
    /**
     * 查询企业用电水平分析列表
     */
    @ApiOperation(value = "查询企业用电水平分析")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaRealTimeAnalysisKwh>> list(SRaRealTimeAnalysisKwhQuery sRaRealTimeAnalysisKwh) {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();
        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisKwh.setRegionCodes(regionCodes);
        Page<SRaRealTimeAnalysisKwh> page = new Page<>(sRaRealTimeAnalysisKwh.getPageNum(), sRaRealTimeAnalysisKwh.getPageSize());
        IPage<SRaRealTimeAnalysisKwh> list = sRaRealTimeAnalysisKwhService.page(page, SRaRealTimeAnalysisKwhQuery.createLambdaQueryWrapper(sRaRealTimeAnalysisKwh));
        return AjaxResults.success(list);
    }


    /**
     * 导出企业用电水平分析列表
     */
    @ApiOperation(value = "导出企业用电水平分析")
    @Log(title = "企业用电水平分析", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SRaRealTimeAnalysisKwhQuery sRaRealTimeAnalysisKwh) {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();
        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisKwh.setRegionCodes(regionCodes);
        List<SRaRealTimeAnalysisKwh> list = sRaRealTimeAnalysisKwhService.selectSRaRealTimeAnalysisKwhList(sRaRealTimeAnalysisKwh);
        ExcelUtil<SRaRealTimeAnalysisKwh> util = new ExcelUtil<SRaRealTimeAnalysisKwh>(SRaRealTimeAnalysisKwh.class);
        util.exportExcel(response, list, "企业用电水平分析数据");
    }

    /**
     * 获取企业用电水平分析详细信息
     */
    @ApiOperation(value = "获取企业用电水平分析详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<SRaRealTimeAnalysisKwh> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaRealTimeAnalysisKwhService.getById(id));
    }

    /**
     * 新增企业用电水平分析
     */
    @ApiOperation(value = "新增企业用电水平分析")
    @Log(title = "企业用电水平分析", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaRealTimeAnalysisKwh sRaRealTimeAnalysisKwh) {
        return toAjax(sRaRealTimeAnalysisKwhService.save(sRaRealTimeAnalysisKwh));
    }

    /**
     * 修改企业用电水平分析
     */
    @ApiOperation(value = "修改企业用电水平分析")
    @Log(title = "企业用电水平分析", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaRealTimeAnalysisKwhVo sRaRealTimeAnalysisKwhVo) {
        if (ObjectUtil.isEmpty(sRaRealTimeAnalysisKwhVo)) {
            throw new InvalidParameterException("参数为空");
        }
        if (ObjectUtil.isEmpty(sRaRealTimeAnalysisKwhVo.getDataId())) {
            throw new InvalidParameterException("参数id为空");
        }
        SRaRealTimeAnalysisKwh sRaRealTimeAnalysisKwh = sRaRealTimeAnalysisKwhService.getById(sRaRealTimeAnalysisKwhVo.getDataId());
        if (ObjectUtil.isEmpty(sRaRealTimeAnalysisKwh)) {
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaRealTimeAnalysisKwhVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaRealTimeAnalysisKwhVo, sRaRealTimeAnalysisKwh);
        return toAjax(sRaRealTimeAnalysisKwhService.saveOrUpdate(sRaRealTimeAnalysisKwh));
    }

    /**
     * 删除企业用电水平分析
     */
    @ApiOperation(value = "删除企业用电水平分析")
    @RequiresPermissions("talroad:kwh:remove")
    @Log(title = "企业用电水平分析", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable String[] dataIds) {
        return toAjax(sRaRealTimeAnalysisKwhService.removeByIds(Arrays.asList(dataIds)));
    }
    /**
     * 参数名称options
     */
    @ApiOperation(value = "参数名称options")
    @GetMapping("/pullPollutantOptions")
    public AjaxResults<List<Map<String, String>>> pullPollutantOptions() {
        return AjaxResults.success(sRaRealTimeAnalysisKwhService.pullPollutantOptions());
    }

    /**
     * 排放规模options
     */
    @ApiOperation(value = "排放规模options")
    @GetMapping("/pullEmissionScaleOptions")
    public AjaxResults<List<Map<String, String>>> pullEmissionScaleOptions() {
        return AjaxResults.success(sRaRealTimeAnalysisKwhService.pullEmissionScaleOptions());
    }
}
