package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.domain.R;
import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.security.utils.SecurityUtils;
import cn.cecep.talroad.domain.analysis.SRaRealTimeAnalysisPower;
import cn.cecep.talroad.feign.BaseRegionService;
import cn.cecep.talroad.vo.EChartsVO;
import cn.cecep.talroad.vo.query.analysis.SRaRealTimeAnalysisPowerQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.cecep.talroad.vo.query.analysis.SRaRealTimeAnalysisPowerQuery;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;
import java.util.Map;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;
import cn.cecep.talroad.domain.analysis.SRaRealTimeAnalysisPower;
import cn.cecep.talroad.vo.analysis.SRaRealTimeAnalysisPowerVo;
import cn.cecep.talroad.service.analysis.ISRaRealTimeAnalysisPowerService;


/**
 * * 开发公司：中节能天融科技有限公司
 * 关联分析系统-企业生产过程分析-机组负荷分析Controller 控制层
 *
 * @author szhpt
 * @date 2023-04-02*/
@Api(tags = "关联分析系统-企业生产过程分析-机组负荷分析")
@RestController
@RequestMapping("/power")
public class SRaRealTimeAnalysisPowerController extends BasesController
{
    @Autowired
    private ISRaRealTimeAnalysisPowerService sRaRealTimeAnalysisPowerService;

    @Autowired
//    @Qualifier("baseRegionService")
    private BaseRegionService baseRegionService;
    /**
     * 查询关联分析系统-企业生产过程分析-机组负荷分析列表
     */
    @ApiOperation(value = "查询关联分析系统-企业生产过程分析-机组负荷分析")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaRealTimeAnalysisPower>> list(SRaRealTimeAnalysisPowerQuery sRaRealTimeAnalysisPower)
    {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();
        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisPower.setRegionCodes(regionCodes);
        Page<SRaRealTimeAnalysisPower> page = new Page<>(sRaRealTimeAnalysisPower.getPageNum(), sRaRealTimeAnalysisPower.getPageSize());
        IPage<SRaRealTimeAnalysisPower> list = sRaRealTimeAnalysisPowerService.page(page, SRaRealTimeAnalysisPowerQuery.createLambdaQueryWrapper(sRaRealTimeAnalysisPower));
        return AjaxResults.success(list);
    }


    @ApiOperation(value = "机组负荷统计分页查询")
    @GetMapping("/statisticsList")
    public AjaxResults<IPage<SRaRealTimeAnalysisPower>> statisticsList(SRaRealTimeAnalysisPowerQuery sRaRealTimeAnalysisPower) {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();
        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisPower.setRegionCodes(regionCodes);
        IPage<SRaRealTimeAnalysisPower> list = sRaRealTimeAnalysisPowerService.statisticsList(sRaRealTimeAnalysisPower);
        return AjaxResults.success(list);
    }
    @ApiOperation(value = "机组负荷统计导出")
    @Log(title = "机组负荷统计", businessType = BusinessType.EXPORT)
    @PostMapping("/statisticsExport")
    public void statisticsExport(HttpServletResponse response, @RequestBody SRaRealTimeAnalysisPowerQuery sRaRealTimeAnalysisPower) {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();
        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisPower.setRegionCodes(regionCodes);
        List<SRaRealTimeAnalysisPower> list = sRaRealTimeAnalysisPowerService.statisticsExport(sRaRealTimeAnalysisPower);
        if(sRaRealTimeAnalysisPower.getType().equals("1")){
            List<SRaRealTimeAnalysisPower.SRaRealTimeAnalysisFactoryPower> factoryCoalList=list.stream().map(o -> {
                SRaRealTimeAnalysisPower.SRaRealTimeAnalysisFactoryPower vo = new SRaRealTimeAnalysisPower.SRaRealTimeAnalysisFactoryPower();
                BeanUtils.copyProperties(o, vo);
                vo.setAnalyseTimeStr(sRaRealTimeAnalysisPower.getAnalyseStartTime()+"至"+sRaRealTimeAnalysisPower.getAnalyseEndTime());
                return vo;
            }).collect(Collectors.toList());
            ExcelUtil<SRaRealTimeAnalysisPower.SRaRealTimeAnalysisFactoryPower> util = new ExcelUtil<SRaRealTimeAnalysisPower.SRaRealTimeAnalysisFactoryPower>(SRaRealTimeAnalysisPower.SRaRealTimeAnalysisFactoryPower.class);
            util.exportExcel(response, factoryCoalList, "机组负荷统计");
        }else {
            List<SRaRealTimeAnalysisPower.SRaRealTimeAnalysisDevicePower> factoryCoalList=list.stream().map(o -> {
                SRaRealTimeAnalysisPower.SRaRealTimeAnalysisDevicePower vo = new SRaRealTimeAnalysisPower.SRaRealTimeAnalysisDevicePower();
                BeanUtils.copyProperties(o, vo);
                vo.setAnalyseTimeStr(sRaRealTimeAnalysisPower.getAnalyseStartTime()+"至"+sRaRealTimeAnalysisPower.getAnalyseEndTime());
                return vo;
            }).collect(Collectors.toList());
            ExcelUtil<SRaRealTimeAnalysisPower.SRaRealTimeAnalysisDevicePower> util = new ExcelUtil<SRaRealTimeAnalysisPower.SRaRealTimeAnalysisDevicePower>(SRaRealTimeAnalysisPower.SRaRealTimeAnalysisDevicePower.class);
            util.exportExcel(response, factoryCoalList, "机组负荷统计");
        }

    }

    @ApiOperation(value = "机组负荷统计-曲线图")
    @GetMapping("/resultCurve")
    public AjaxResults<EChartsVO> resultCurve(SRaRealTimeAnalysisPowerQuery sRaRealTimeAnalysisCoa) {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();
        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisCoa.setRegionCodes(regionCodes);
        return AjaxResults.success(sRaRealTimeAnalysisPowerService.resultCurve(sRaRealTimeAnalysisCoa));
    }
    /**
     * 导出关联分析系统-企业生产过程分析-机组负荷分析列表
     */
    @ApiOperation(value = "导出关联分析系统-企业生产过程分析-机组负荷分析")
    @Log(title = "关联分析系统-企业生产过程分析-机组负荷分析", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SRaRealTimeAnalysisPowerQuery sRaRealTimeAnalysisPower)
    {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();
        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaRealTimeAnalysisPower.setRegionCodes(regionCodes);
        List<SRaRealTimeAnalysisPower> list = sRaRealTimeAnalysisPowerService.selectSRaRealTimeAnalysisPowerList(sRaRealTimeAnalysisPower);
        ExcelUtil<SRaRealTimeAnalysisPower> util = new ExcelUtil<SRaRealTimeAnalysisPower>(SRaRealTimeAnalysisPower.class);
        util.exportExcel(response, list, "关联分析系统-企业生产过程分析-机组负荷分析数据");
    }

    /**
     * 获取关联分析系统-企业生产过程分析-机组负荷分析详细信息
     */
    @ApiOperation(value = "获取关联分析系统-企业生产过程分析-机组负荷分析详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<SRaRealTimeAnalysisPower> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaRealTimeAnalysisPowerService.getById(id));
    }
    /**
     * 新增关联分析系统-企业生产过程分析-机组负荷分析
     */
    @ApiOperation(value = "新增关联分析系统-企业生产过程分析-机组负荷分析")
    @Log(title = "关联分析系统-企业生产过程分析-机组负荷分析", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaRealTimeAnalysisPower sRaRealTimeAnalysisPower) {
        return toAjax(sRaRealTimeAnalysisPowerService.save(sRaRealTimeAnalysisPower));
    }
    /**
     * 修改关联分析系统-企业生产过程分析-机组负荷分析
     */
    @ApiOperation(value = "修改关联分析系统-企业生产过程分析-机组负荷分析")
    @Log(title = "关联分析系统-企业生产过程分析-机组负荷分析", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaRealTimeAnalysisPowerVo sRaRealTimeAnalysisPowerVo) {
        if(ObjectUtil.isEmpty(sRaRealTimeAnalysisPowerVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(sRaRealTimeAnalysisPowerVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        SRaRealTimeAnalysisPower sRaRealTimeAnalysisPower = sRaRealTimeAnalysisPowerService.getById(sRaRealTimeAnalysisPowerVo.getDataId());
        if(ObjectUtil.isEmpty(sRaRealTimeAnalysisPower)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaRealTimeAnalysisPowerVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaRealTimeAnalysisPowerVo,sRaRealTimeAnalysisPower);
        return toAjax(sRaRealTimeAnalysisPowerService.saveOrUpdate(sRaRealTimeAnalysisPower));
    }

    /**
     * 删除关联分析系统-企业生产过程分析-机组负荷分析
     */
    @ApiOperation(value = "删除关联分析系统-企业生产过程分析-机组负荷分析")
    @RequiresPermissions("talroad:power:remove")
    @Log(title = "关联分析系统-企业生产过程分析-机组负荷分析", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable String[] dataIds) {
        return toAjax(sRaRealTimeAnalysisPowerService.removeByIds(Arrays.asList(dataIds)));
    }
    /**
     * 设备名称options
     */
    @ApiOperation(value = "设备名称options")
    @GetMapping("/pullDeviceNameOptions")
    public AjaxResults<List<Map<String, Object>>> pullDeviceNameOptions(@RequestParam(value = "factoryName", required = false) String factoryName) {
        return AjaxResults.success(sRaRealTimeAnalysisPowerService.pullDeviceNameOptions(factoryName));
    }
}
