package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.domain.R;
import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.security.utils.SecurityUtils;
import cn.cecep.talroad.domain.analysis.SRaWaterPermittedEmissions;
import cn.cecep.talroad.feign.BaseRegionService;
import cn.cecep.talroad.service.analysis.ISRaWaterPermittedEmissionsService;
import cn.cecep.talroad.vo.analysis.SRaWaterPermittedEmissionsVo;
import cn.cecep.talroad.vo.query.analysis.SRaWaterPermittedEmissionsQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;
import java.util.Map;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;


/**
 * * 开发公司：中节能天融科技有限公司
 * 许可排放量废水明细Controller 控制层
 *
 * @author szhpt
 * @date 2023-06-02*/
@Api(tags = "许可排放量废水明细")
@RestController
@RequestMapping("/emissions/water")
public class SRaWaterPermittedEmissionsController extends BasesController
{
    @Autowired
    private ISRaWaterPermittedEmissionsService sRaWaterPermittedEmissionsService;

    @Autowired
//    @Qualifier("baseRegionService")
    private BaseRegionService baseRegionService;
    /**
     * 查询许可排放量废水明细列表
     */
    @ApiOperation(value = "查询许可排放量废水明细")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaWaterPermittedEmissions>> list(SRaWaterPermittedEmissionsQuery sRaWaterPermittedEmissions)
    {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();

        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaWaterPermittedEmissions.setRegionCodes(regionCodes);
        IPage<SRaWaterPermittedEmissions> list = sRaWaterPermittedEmissionsService.selectSRaWaterPermittedEmissionsListBy(sRaWaterPermittedEmissions);
        return AjaxResults.success(list);
    }
    

    /**
     * 导出许可排放量废水明细列表
     */
    @ApiOperation(value = "导出许可排放量废水明细")
    @Log(title = "许可排放量废水明细", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SRaWaterPermittedEmissionsQuery sRaWaterPermittedEmissions)
    {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();

        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaWaterPermittedEmissions.setRegionCodes(regionCodes);
        List<SRaWaterPermittedEmissions> list = sRaWaterPermittedEmissionsService.selectSRaWaterPermittedEmissionsList(sRaWaterPermittedEmissions);
        ExcelUtil<SRaWaterPermittedEmissions> util = new ExcelUtil<SRaWaterPermittedEmissions>(SRaWaterPermittedEmissions.class);
        util.exportExcel(response, list, "许可排放量废水明细数据");
    }

    /**
     * 获取许可排放量废水明细详细信息
     */
    @ApiOperation(value = "获取许可排放量废水明细详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<SRaWaterPermittedEmissions> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaWaterPermittedEmissionsService.getById(id));
    }
    /**
     * 新增许可排放量废水明细
     */
    @ApiOperation(value = "新增许可排放量废水明细")
    @Log(title = "许可排放量废水明细", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaWaterPermittedEmissions sRaWaterPermittedEmissions) {
        return toAjax(sRaWaterPermittedEmissionsService.save(sRaWaterPermittedEmissions));
    }
    /**
     * 修改许可排放量废水明细
     */
    @ApiOperation(value = "修改许可排放量废水明细")
    @Log(title = "许可排放量废水明细", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaWaterPermittedEmissionsVo sRaWaterPermittedEmissionsVo) {
        if(ObjectUtil.isEmpty(sRaWaterPermittedEmissionsVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(sRaWaterPermittedEmissionsVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        SRaWaterPermittedEmissions sRaWaterPermittedEmissions = sRaWaterPermittedEmissionsService.getById(sRaWaterPermittedEmissionsVo.getDataId());
        if(ObjectUtil.isEmpty(sRaWaterPermittedEmissions)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaWaterPermittedEmissionsVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaWaterPermittedEmissionsVo,sRaWaterPermittedEmissions);
        return toAjax(sRaWaterPermittedEmissionsService.saveOrUpdate(sRaWaterPermittedEmissions));
    }

    /**
     * 删除许可排放量废水明细
     */
    @ApiOperation(value = "删除许可排放量废水明细")
    @RequiresPermissions("talroad:emissions:remove")
    @Log(title = "许可排放量废水明细", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable String[] dataIds) {
        return toAjax(sRaWaterPermittedEmissionsService.removeByIds(Arrays.asList(dataIds)));
    }

    /**
     * 监测点名称options
     */
    @ApiOperation(value = "监测点名称options")
    @GetMapping("/pullMonitorNameOptions")
    public AjaxResults<List<Map<String, Object>>> pullMonitorNameOptions(@RequestParam(value = "factoryName", required = false) String factoryName) {
        return AjaxResults.success(sRaWaterPermittedEmissionsService.pullMonitorNameOptions(factoryName));
    }
}
