package cn.cecep.talroad.controller.analysis;

import cn.cecep.talroad.common.core.domain.R;
import cn.cecep.talroad.common.core.exception.CommonException;
import cn.cecep.talroad.common.core.utils.StringUtils;
import cn.cecep.talroad.common.core.web.domain.AjaxResults;
import cn.cecep.talroad.common.security.utils.SecurityUtils;
import cn.cecep.talroad.domain.analysis.SRaWaterPermittedOnline;
import cn.cecep.talroad.feign.BaseRegionService;
import cn.cecep.talroad.service.analysis.ISRaWaterPermittedOnlineService;
import cn.cecep.talroad.vo.analysis.SRaWaterPermittedOnlineVo;
import cn.cecep.talroad.vo.query.analysis.SRaWaterPermittedOnlineQuery;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;
import java.util.Map;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.*;
import cn.cecep.talroad.common.log.annotation.Log;
import cn.cecep.talroad.common.log.enums.BusinessType;
import cn.cecep.talroad.common.security.annotation.RequiresPermissions;

import cn.cecep.talroad.common.core.web.controller.BasesController;
import cn.cecep.talroad.common.core.utils.poi.ExcelUtil;


/**
 * * 开发公司：中节能天融科技有限公司
 * 许可排放量废水明细Controller 控制层
 *
 * @author szhpt
 * @date 2023-06-02*/
@Api(tags = "许可排放量废水明细")
@RestController
@RequestMapping("/permitted/water")
public class SRaWaterPermittedOnlineController extends BasesController
{
    @Autowired
    private ISRaWaterPermittedOnlineService sRaWaterPermittedOnlineService;
    @Autowired
//    @Qualifier("baseRegionService")
    private BaseRegionService baseRegionService;
    /**
     * 查询许可排放量废水明细列表
     */
    @ApiOperation(value = "查询在线、许可排放量废水明细")
    @GetMapping("/list")
    public AjaxResults<IPage<SRaWaterPermittedOnline>> list(SRaWaterPermittedOnlineQuery sRaWaterPermittedOnline)
    {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();

        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaWaterPermittedOnline.setRegionCodes(regionCodes);
        IPage<SRaWaterPermittedOnline> list = sRaWaterPermittedOnlineService.selectSRaWaterPermittedOnlineListBy(sRaWaterPermittedOnline);
        return AjaxResults.success(list);
    }
    

    /**
     * 导出许可排放量废水明细列表
     */
    @ApiOperation(value = "导出在线、许可排放量废水明细",produces = "application/octet-stream")
    @Log(title = "许可排放量废水明细", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SRaWaterPermittedOnlineQuery sRaWaterPermittedOnline)
    {
        String regionCode = SecurityUtils.getLoginUser().getSysUser().getRegionCode();

        List<String> regionCodes=new ArrayList<>();
        R<List<String>> r = baseRegionService.getRegionCodeListChild(regionCode);
        if(r.getCode()==200){
            regionCodes=r.getData();
        }
        sRaWaterPermittedOnline.setRegionCodes(regionCodes);
        List<SRaWaterPermittedOnline> list = sRaWaterPermittedOnlineService.selectSRaWaterPermittedOnlineList(sRaWaterPermittedOnline);
        ExcelUtil<SRaWaterPermittedOnline> util = new ExcelUtil<SRaWaterPermittedOnline>(SRaWaterPermittedOnline.class);
        if(sRaWaterPermittedOnline.getType().equals("1")){
            if(StringUtils.isNotEmpty(sRaWaterPermittedOnline.getHideIndex())){
                if(sRaWaterPermittedOnline.getHideIndex().equals("cod")){
                    util.hideColumnNative(1,2,6,8,9,10,11,12,13,14);
                }
                if(sRaWaterPermittedOnline.getHideIndex().equals("ammonia")){
                    util.hideColumnNative(1,2,6,7,9,10,11,12,13,14);
                }
                if(sRaWaterPermittedOnline.getHideIndex().equals("phosphorus")){
                    util.hideColumnNative(1,2,6,7,8,10,11,12,13,14);
                }
                if(sRaWaterPermittedOnline.getHideIndex().equals("nitrogen")){
                    util.hideColumnNative(1,2,6,7,8,9,11,12,13,14);
                }
            }else {
                util.hideColumnNative(6,11,12,13,14);
            }
            util.exportExcel(response, list, "在线排放量废水明细数据");

        }else {
            if(StringUtils.isNotEmpty(sRaWaterPermittedOnline.getHideIndex())){
                if(sRaWaterPermittedOnline.getHideIndex().equals("cod")){
                    util.hideColumnNative(1,2,8,9,10,11,12,13,14);
                }
                if(sRaWaterPermittedOnline.getHideIndex().equals("ammonia")){
                    util.hideColumnNative(1,2,7,9,10,11,12,13,14);
                }
                if(sRaWaterPermittedOnline.getHideIndex().equals("phosphorus")){
                    util.hideColumnNative(1,2,7,8,10,11,12,13,14);
                }
                if(sRaWaterPermittedOnline.getHideIndex().equals("nitrogen")){
                    util.hideColumnNative(1,2,7,8,9,11,12,13,14);
                }
            }else {
                util.hideColumnNative(11,12,13,14);
            }
            util.exportExcel(response, list, "许可排放量废水明细数据");
        }
    }

    public static void main(String[] args) {
        String str = "1,2";
        str = str.trim().replace("\"", "");
        System.out.println(str);
    }
    /**
     * 获取许可排放量废水明细详细信息
     */
    @ApiOperation(value = "获取许可排放量废水明细详细信息")
    @PostMapping(value = "/{dataId}")
    public AjaxResults<SRaWaterPermittedOnline> getInfo(@PathVariable("id") Integer id) {
        return AjaxResults.success(sRaWaterPermittedOnlineService.getById(id));
    }
    /**
     * 新增许可排放量废水明细
     */
    @ApiOperation(value = "新增许可排放量废水明细")
    @Log(title = "许可排放量废水明细", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    public AjaxResults<Boolean> add(@RequestBody SRaWaterPermittedOnline sRaWaterPermittedOnline) {
        return toAjax(sRaWaterPermittedOnlineService.save(sRaWaterPermittedOnline));
    }
    /**
     * 修改许可排放量废水明细
     */
    @ApiOperation(value = "修改许可排放量废水明细")
    @Log(title = "许可排放量废水明细", businessType = BusinessType.UPDATE)
    @PostMapping(value = "/edit")
    public AjaxResults<Boolean> edit(@RequestBody SRaWaterPermittedOnlineVo sRaWaterPermittedOnlineVo) {
        if(ObjectUtil.isEmpty(sRaWaterPermittedOnlineVo)){
            throw new InvalidParameterException("参数为空");
        }
        if(ObjectUtil.isEmpty(sRaWaterPermittedOnlineVo.getDataId())){
            throw new InvalidParameterException("参数id为空");
        }
        SRaWaterPermittedOnline sRaWaterPermittedOnline = sRaWaterPermittedOnlineService.getById(sRaWaterPermittedOnlineVo.getDataId());
        if(ObjectUtil.isEmpty(sRaWaterPermittedOnline)){
            throw new CommonException(StrUtil.format("根据id{}查询不到数据", sRaWaterPermittedOnlineVo.getDataId()));
        }
        //赋值
        BeanUtil.copyProperties(sRaWaterPermittedOnlineVo,sRaWaterPermittedOnline);
        return toAjax(sRaWaterPermittedOnlineService.saveOrUpdate(sRaWaterPermittedOnline));
    }

    /**
     * 删除许可排放量废水明细
     */
    @ApiOperation(value = "删除许可排放量废水明细")
    @RequiresPermissions("talroad:online:remove")
    @Log(title = "许可排放量废水明细", businessType = BusinessType.DELETE)
    @PostMapping("/{dataIds}")
    public AjaxResults<Boolean> remove(@PathVariable String[] dataIds) {
        return toAjax(sRaWaterPermittedOnlineService.removeByIds(Arrays.asList(dataIds)));
    }
    /**
     * 监测点名称options
     */
    @ApiOperation(value = "监测点名称options")
    @GetMapping("/pullMonitorNameOptions")
    public AjaxResults<List<Map<String, Object>>> pullMonitorNameOptions(@RequestParam(value = "factoryName", required = false) String factoryName) {
        return AjaxResults.success(sRaWaterPermittedOnlineService.pullMonitorNameOptions(factoryName));
    }
}
