package cn.cecep.talroad.domain;

import cn.cecep.talroad.common.core.annotation.Excel;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import lombok.Data;
import org.apache.ibatis.jdbc.Null;

/**
    * 管控浓度标准
    */
@ApiModel(description="管控浓度标准")
@Data
public class BaseControlConcentStandards implements Serializable {
    /**
    * 主键
    */
    @ApiModelProperty(value="主键")
    private String id;

    /**
    * 企业id
    */
    @ApiModelProperty(value="企业id")
    @Excel(name="企业id",sort = -1)
    private String factoryId;

    /**
    * 污染物种类
    */
    @ApiModelProperty(value="污染物种类")
    @Excel(name="污染物种类Code",sort = -2)
    private String pollutantType;

    /**
    * 管控浓度标准（mg/m3）
    */
    @ApiModelProperty(value="管控浓度标准（mg/m3）")
    @Excel(name = "管控浓度标准（mg/m3）",sort = 6,cellType = Excel.ColumnType.NUMERIC)
    private BigDecimal controlStandard;

    /**
    * 纳入计算：0 否  1是
    */
    @ApiModelProperty(value="纳入计算：0 否  1是")
    @Excel(name = "是否纳入计算",sort = 8, readConverterExp = "0=否,1=是",combo = {"是", "否"})
    private String includeCalculations;

    @ApiModelProperty(value="重点管控点位：0 否  1是")
    @Excel(name = "是否重点管控点位",sort = 7, readConverterExp = "0=否,1=是",combo = {"是", "否"})
    private String controlPoints;

    /**
    * 创建时间
    */
    @ApiModelProperty(value="创建时间")
    private Date createTime;

    /**
    * 更新时间
    */
    @ApiModelProperty(value="更新时间")
    private Date updateTime;

    /**
    * 对应监测点位名称
    */
    @ApiModelProperty(value="对应监测点位id")
    @Excel(name="对应监测点位id",sort = -3)
    private String outletId;

    @ApiModelProperty(value="建议排放量小时均值，通过定时任务计算，每月更新一次")
    private BigDecimal adviceEmissionAvg;

    private static final long serialVersionUID = 1L;
}