package cn.cecep.talroad.domain;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
*站点表
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@TableName("p_code_station")
@ApiModel(value="PCodeStation对象", description="站点表")
public class PCodeStation implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "主键")
    @TableId(value = "station_id")
    private String stationId;

    @TableField("province_code")
    private String provinceCode;

    @ApiModelProperty(value = "省")
    @TableField("province")
    private String province;

    @TableField("city_code")
    private String cityCode;

    @ApiModelProperty(value = "市")
    @TableField("city")
    private String city;

    @TableField("district_code")
    private String districtCode;

    @ApiModelProperty(value = "区")
    @TableField("district")
    private String district;

    @ApiModelProperty(value = "街镇code")
    @TableField("township_code")
    private String townShipCode;

    @ApiModelProperty(value = "街镇")
    @TableField("township")
    private String townShip;

    @ApiModelProperty(value = "站点名称")
    @TableField("station_name")
    private String stationName;

    @ApiModelProperty(value = "站点编号")
    @TableField("station_code")
    private String stationCode;

    @ApiModelProperty(value = "站点状态0 离线，1在线，2超标,3异常")
    @TableField("status")
    private Integer status;

    @ApiModelProperty(value = "所属行政区")
    @TableField("region_code")
    private Integer regionCode;
    @ApiModelProperty(value = "城市考核点 0:否  1:是")
    @TableField("city_assessment_point")
    private Integer cityAssessmentPoint;
    @ApiModelProperty(value = "区县考核点 0:否  1:是")
    @TableField("district_assessment_point")
    private Integer districtAssessmentPoint;
    @ApiModelProperty(value = "街道考核点 0:否  1:是")
    @TableField("street_assessment_point")
    private Integer streetAssessmentPoint;


    @ApiModelProperty(value = "所属功能区 商业/工业/住宅/绿化/文化教育")
    @TableField("function_id")
    private Integer functionId;

    @ApiModelProperty(value = "1：国控站，2：省控站，3，市控站，4：微站 5：六参数微站，6：小型站，7：β射线扬尘站 8： TVOC微站，9：颗粒物微站，10：气象站 11：尾气遥测，12：超级站，13：黑烟车监测 14：园区站")
    @TableField("station_level")
    private Integer stationLevel;

    @ApiModelProperty(value = "站点类型 1大气环境 2污染源废气 3水监测 4污染源废水")
    @TableField("station_type")
    private Integer stationType;

    @ApiModelProperty(value = "站点父节点类型：0没有，1企业，2道路，3采石场")
    @TableField("parent_type")
    private Integer parentType;

    @ApiModelProperty(value = "父节点ID")
    @TableField("fk_parent_id")
    private String fkParentId;

    @ApiModelProperty(value = "经度")
    @TableField("longitude")
    private BigDecimal longitude;

    @ApiModelProperty(value = "维度")
    @TableField("latitude")
    private BigDecimal latitude;

    @ApiModelProperty(value = "百度地图经度")
    @TableField("bmap_longitude")
    private BigDecimal bmapLongitude;

    @ApiModelProperty(value = "百度地图纬度")
    @TableField("bmap_latitude")
    private BigDecimal bmapLatitude;

    @ApiModelProperty(value = "GCG102 经度")
    @TableField("gcg_longitude")
    private String gcgLongitude;

    @ApiModelProperty(value = "GCG102 纬度")
    @TableField("gcg_latitude")
    private String gcgLatitude;

    @ApiModelProperty(value = "站点地址")
    @TableField("address")
    private String address;

    @ApiModelProperty(value = "站点负责人")
    @TableField("principal")
    private String principal;

    @ApiModelProperty(value = "负责人电话")
    @TableField("phone")
    private String phone;

    @ApiModelProperty(value = "创建人")
    @TableField("create_user")
    private String createUser;

    @ApiModelProperty(value = "创建时间")
    @TableField("create_time")
    private String createTime;

    @ApiModelProperty(value = "是否参与城市评价 0 不参与，1参与")
    @TableField("is_assessment_city")
    private Boolean isAssessmentCity;

    @ApiModelProperty(value = "是否开启自动审核，0 不开启， 1 开启， 建立自动审核数据表，有该站点数据表示开启。")
    @TableField("is_auto_audit")
    private Boolean isAutoAudit;

    @ApiModelProperty(value = "是否统计数据 0 不统计数据，1统计日数据，2 统计小时、日数据 默认不统计数据")
    @TableField("is_auto_statistic")
    private Boolean isAutoStatistic;

    @ApiModelProperty(value = "删除标识(0-使用；1-不使用)")
    @TableField("is_used")
    private Boolean isUsed;

    @ApiModelProperty(value = "('是否维护状态 1：是，0：否')")
    @TableField("is_maintain")
    private Boolean isMaintain;

    @ApiModelProperty(value = "气象数据源：0 站点自行发送，1取站点所属区域气象数据，2取站点所属父级行政区域")
    @TableField("meteorological_data")
    private Integer meteorologicalData;

    @ApiModelProperty(value = "是否自动预测预警 1：是，0：否")
    @TableField("is_forecast")
    private Boolean isForecast;

    @TableField(value = "update_time", fill = FieldFill.INSERT_UPDATE)
    private LocalDateTime updateTime;

    @ApiModelProperty(value = "是否从chinaAqi爬取数据,0-否，1-是")
    @TableField("is_chinaaqi")
    private Boolean isChinaaqi;

    @ApiModelProperty(value = "数据类型(1-城市空气;2-废气类;3-镇办空气;4-VOCs)")
    @TableField("data_type")
    private Boolean dataType;

    @ApiModelProperty(value = "备注")
    @TableField("remark")
    private String remark;
}
