package cn.cecep.talroad.service.analysis;

import java.lang.reflect.InvocationTargetException;
import java.text.ParseException;

/**
 * 排放量计算服务类
 * @author liuzhiwei
 */
public interface IDischargeMathService {
    /**
     * 大气污染物在线小时排放量
     */
    void doGasPollutantHourDischarge(String mathDate) throws ParseException, NoSuchMethodException, InvocationTargetException, IllegalAccessException;


    /**
     * 水污染物在线日排放量
     */
    void doWaterPollutantDayDischarge(String mathDate) throws NoSuchMethodException, InvocationTargetException, IllegalAccessException, ParseException;


    /**
     * 水污染物在线排放量统计值
     */
    void doWaterPollutantDayDischargeTotal(String mathDate) throws NoSuchMethodException, InvocationTargetException, IllegalAccessException, ParseException;


    /**
     * 水污染物在线排放量统计值
     */
    void doGasPollutantHourDischargeTotal(String mathDate) throws NoSuchMethodException, InvocationTargetException, IllegalAccessException, ParseException;


    /**
     * 废气手工月排放量
     */
    void doGasHandiworkMonthDischarge();

    /**
     * 废水手工月排放量
     */
    void doWaterHandiworkMonthDischarge();


    /**
     * 废气排放量超标
     */
    void doGasDischargeExceeded() throws NoSuchMethodException, InvocationTargetException, IllegalAccessException;

    /**
     * 废水排放量超标
     */
    void doWaterDischargeExceeded() throws NoSuchMethodException, InvocationTargetException, IllegalAccessException;

    /**
     * 废气排放量预警
     */
    void doGasDischargeAlarm() throws NoSuchMethodException, InvocationTargetException, IllegalAccessException;

    /**
     * 废水排放量预警
     */
    void doWaterDischargeAlarm() throws NoSuchMethodException, InvocationTargetException, IllegalAccessException;

    /**
     * 大气污染物小时排放量，状态标记
     */
    void doProcessGasStationDataFlag();

    /**
     * 水污染物日排放量，状态标记
     */
    void doProcessWaterStationDataFlag();

    /**
     * 污染物小时均值连续8H处于标准值的20
     */
    void doProcessHourPollutantConcentration();
}
