package cn.cecep.talroad.vo.query.analysis;

import cn.cecep.talroad.common.core.domain.query.PageQuery;
import cn.cecep.talroad.common.core.utils.DateUtils;
import cn.cecep.talroad.common.core.utils.StringUtils;
import cn.cecep.talroad.domain.analysis.SRaRealTimeAnalysisCoal;
import cn.cecep.talroad.domain.analysis.SRaRealTimeAnalysisCoke;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.List;


/**
 * 关联分析系统-企业生产过程分析-焦炭产量分析对象 s_ra_real_time_analysis_coke
 *
 * @author szhpt
 * @date 2023-04-03*/
@ApiModel(value = "关联分析系统-企业生产过程分析-焦炭产量分析")
@EqualsAndHashCode(callSuper = false)
@Data
public class SRaRealTimeAnalysisCokeQuery extends PageQuery
{

    @ApiModelProperty(value = "数据主键")
    private String dataId;
    @ApiModelProperty(value = "企业主键")
    private String factoryId;
    @ApiModelProperty(value = "企业名称")
    private String factoryName;
    @ApiModelProperty(value = "所在区县名称")
    private String districtName;
    @ApiModelProperty(value = "所在区县编码")
    private String districtCode;
    @ApiModelProperty(value = "设备名称")
    private String deviceName;
    @ApiModelProperty(value = "设备编号")
    private String deviceId;
    @ApiModelProperty(value = "设备类型")
    private String deviceType;
    @ApiModelProperty(value = "炭化室有效容积(m3)")
    private String volumeNumber;
    @ApiModelProperty(value = "小时推焦数")
    private Long pushNumber;
    @ApiModelProperty(value = "焦炭产量(t)")
    private String cokeNumber;
    @ApiModelProperty(value = "更新时间")
    private String updateTime;
    @ApiModelProperty(value = "创建时间(默认当前时间)")
    private String createTime;

    @ApiModelProperty(value = "统计时间")
    private String analyseTime;
    @ApiModelProperty(value = "统计开始时间")
    private String analyseStartTime;
    @ApiModelProperty(value = "统计截至时间")
    private String analyseEndTime;
    @ApiModelProperty("统计类型1按企业合计，2按设备合计")
    private String type;
    private List<String> regionCodes;
    /**
     *
     * @Title: getQueryWrapper
     * @Description: 查询参数组装
     * @param: @return
     * @return: QueryWrapper<BasIndustry>
     * @throws
     */
    public static QueryWrapper<SRaRealTimeAnalysisCoke> createLambdaQueryWrapper(SRaRealTimeAnalysisCokeQuery query) {
        QueryWrapper<SRaRealTimeAnalysisCoke> wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(query.getFactoryName()), "factory_name", query.getFactoryName());
        wrapper.eq(StringUtils.isNotEmpty(query.getDistrictCode()), "district_code", query.getDistrictCode());
        wrapper.in(CollectionUtil.isNotEmpty(query.getRegionCodes()), "district_code", query.getRegionCodes());
        wrapper.eq(StringUtils.isNotEmpty(query.getDeviceName()), "device_name", query.getDeviceName());
        wrapper.eq(StringUtils.isNotEmpty(query.getDeviceId()), "device_id", query.getDeviceId());
        wrapper.between(StringUtils.isNotEmpty(query.getAnalyseStartTime()) && StringUtils.isNotEmpty(query.getAnalyseEndTime()), "analyse_time", DateUtils.parseDate(query.getAnalyseStartTime()), DateUtils.parseDate(query.getAnalyseEndTime()));
        wrapper.orderByDesc("analyse_time");
        return wrapper;
        }
}
